/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TCreateDocModelTS.cpp
 * Description    : Perform doc model creation functions
 *
 */


#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreTS.hpp>
#include <DKDatastoreDefTS.hpp>
#include <DKDatastoreAdminTS.hpp>
#include <DKDocModelTS.hpp>
#include <DKDocSectionTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 3)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TCreateDocModelTS <srchSrv> <docModelName>" << endl;
     return;
    }
    // ### get input parameters
    DKString srchSrv = DKString(argv[1]);
    DKString docModelName = DKString(argv[2]);
    DKDatastoreTS dsTS; // must be outside try/catch block

    try {
      DKDatastoreDefTS* dsDef = 0;
      DKDatastoreAdminTS* dsAdmin = 0;
      DKDocModelTS* docModel = new DKDocModelTS();
      DKDocSectionTS* docSection = new DKDocSectionTS();
      cout << "Datastore TS created" << endl;
      docModel->setCCSID(DK_TS_CCSID_00850);
      docModel->setName(docModelName);
      docSection->setName("SAMPTITLE");
      docSection->setTag("TITLE");
      docModel->addDocSection(docSection);
      cout << "connecting to datastore" << endl;
      dsTS.connect(srchSrv,"","");
      cout << "datastore connected " << srchSrv << endl;
      dsDef = (DKDatastoreDefTS*)dsTS.datastoreDef();
      dsAdmin = (DKDatastoreAdminTS*)dsDef->datastoreAdmin();
      cout << "create doc model for instance " << docModelName << endl;
      dsAdmin->createDocModel("",docModel);
      cout << "doc model created for instance" << endl;

      dsTS.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id" << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;
}

