/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
                         
/*
 * Filename       : TSampleIQryQBIC.cpp
 * Description    : connect to QBIC Search.
 *                  create a image query object and submit query.
 *                  get the results in a collection.
 *                  display items with QBIC Search Information.
 *                  disconnect from QBIC Search
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreQBIC.hpp>
#include <DKImageQuery.hpp>
#include <dkResultSetCursor.hpp>
#include <DKResults.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKDDO.hpp>

void main(int argc, char *argv[])
{

    cout << "begin..." << endl;
    DKDatastoreQBIC* dsQBIC; // must be outside try/catch block
    
    try {
  
    cout << "Creating Datastore QBIC" << endl;
    dsQBIC = new DKDatastoreQBIC();
    cout << "Datastore QBIC created" << endl;
    cout << "connecting to datastore" << endl;
    dsQBIC->connect("QBICSRV", "QBICUSER", "PASSWORD");
    cout << "datastore connected" << endl;

    dsQBIC->openCatalog("DEMO", "qbic0725");
    cout << "catalog opened" << endl;
	
    DKAny a;
    DKString strData;
    DKString strDataName;
    DKAny* element;
    DKDDO* item;
  
    DKString cmd = "QbColor color=<255,0,0>";
    dkQuery* pQry = dsQBIC->createQuery(cmd);
    pQry->execute();
    cout << "qbic query done... " << endl;

    DKAny any = pQry->result();
    DKResults* pResults = (DKResults*)((dkCollection*)any);
    dkIterator* pIter = pResults->createIterator();
    while (pIter->more()) {
	element = pIter->next();
	item = (DKDDO*)element->value();
        if (item != 0) 
          {
            for (int j = 1; j <= item->dataCount(); j++)
              {
                a = item->getData(j);
                strDataName = item->getDataName(j);

                switch (a.typeCode())
	            {
                    case DKAny::tc_null :
                    {
                          cout << "attribute name : " << strDataName << " value : null " << endl;
                          break;
                    }
                    default :
                    {
                         cout << "attribute name : " << strDataName << " value :" << a << endl;
                          break;
                    }
	            }
              }
          }
    }
    delete pIter;
    delete pResults;
    delete pQry;   

    dsQBIC->closeCatalog();
    cout << "catalog closed" << endl;

    dsQBIC->disconnect();
    cout << "datastore disconnected" << endl;
    delete dsQBIC;
    cout << "datastore deleted" << endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id "     << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }

    cout << "done ..." << endl;
    
}

