/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TListCatalogFed.cpp
 * Description    : List Fed servers, entities and attributes via the DKDatastoreFed
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKSequentialCollection.hpp>
#include <DKServerDefFed.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)  {
       cout <<"Usage: "<<endl;
       cout <<"\t TListCatalogFed <cmbdb> <userid> <pw>" << endl;
       return;
    }
    // get input parameters
    DKString cmbdb(argv[1]);
    DKString userid(argv[2]);
    DKString pw    (argv[3]);
    DKSequentialCollection *pCol = 0;
    dkIterator *pIter = 0;
    DKServerDefFed *pSV = 0;
    DKString strServerName;
    DKString strRepType;
    DKAny a;
    long i = 0;
    DKDatastoreFed dsFed; // must be outside try block to avoid exception trap error.

    try {

      cout << "list servers" << endl;
      a = dsFed.listDataSources();
      pCol = (DKSequentialCollection*)((dkCollection*)a);
      pIter = pCol->createIterator();
      while (pIter->more() == TRUE)
      {
       i++;
       pSV = (DKServerDefFed*)((void*)(*pIter->next()));
       strServerName = pSV->getName();
       strRepType = pSV->getRepositoryType();
       cout << "Server Name [" << i << "] - " << strServerName << " Server Repository Type - " << strRepType << endl; 
       delete pSV;
      }
      delete pIter;
      delete pCol;
      cout << i << " servers listed" << endl;
      // All set; Connect to federate database
      cout << " *** Connecting to " << cmbdb << " *** " << endl;
      dsFed.connect(cmbdb, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;

      // get the Def and Admin
      dkDatastoreDef* defFed   = dsFed.datastoreDef();
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();
         
      //=======================================================================================================
      // list back-end servers
      cout << "\nList server names " << endl;
      long len, i;
      DKString* servers = adm->listServerNames(len);
      DKServerMapDefFed* serverdat = NULL;

      for (i = 0; i < len; i++) {
          cout << "server [" <<  i << "] = " << servers[i] << endl;
          serverdat = adm->getServer(servers[i]);
          cout << " DKServerMapDefFed = " << serverdat->getName() << endl;
          cout << "     name: " << serverdat->getName()           << endl;
          cout << "     type: " << serverdat->getServerType()     << endl;
          cout << " connStr : " << serverdat->getConnectString()  << endl;
          cout << "    desc : " << serverdat->getDescription()    << endl;
          cout << " refTime : " << serverdat->getRefreshTime()    << endl;
          delete serverdat;

      }  
      delete [] servers;

      //=======================================================================================================
      // list federated entity and attribute names

      long siz;
	  DKString* names = dsFed.listEntityNames(len);
	  dkEntityDef* entityFed = NULL;

      for (i = 0; i < len; i++) {
      	    cout << "name[" << i << "] = " << names[i] << endl;
		    entityFed   = defFed->getEntity(names[i]);
		    cout << "\tDescription= " << entityFed->getDescription() << endl;

		    DKString* attrNames   = entityFed->listAttrNames(siz);
		    if (attrNames != NULL) {
			    for (int j=0; j < siz; j++) {	
				    cout << "\tattrName = " << attrNames[j] << endl;
				    dkAttrDef* attr = entityFed->getAttr(attrNames[j]);
				    cout << "\t\tentityName:   " << attr->getEntityName() << endl;
				    cout << "\t\tdsName:       " << attr->datastoreName() << endl;
				    cout << "\t\tdsType:       " << attr->datastoreType() << endl;
				    cout << "\t\ttype:         " << attr->getType() << endl;
				    cout << "\t\tdescription:  " << attr->getDescription() << endl;
				    cout << "\t\tsize:         " << attr->getSize() << endl;
				    cout << "\t\tid:           " << attr->getId() << endl;
				    cout << "\t\tprecision:    " << attr->getPrecision() << endl;
				    cout << "\t\tscale:        " << attr->getScale() << endl;
				    cout << "\t\tmax:          " << attr->getMax() << endl;
				    cout << "\t\tmin:          " << attr->getMin() << endl;
				    cout << "\t\tstringType:   " << attr->getStringType() << endl;

				    if (attr->isNullable())
					    cout << "\t\tisNullable:   true" << endl;
				    else 
					    cout << "\t\tisNullable:   false" << endl;
	
				    if (attr->isUpdatable())
					    cout << "\t\tisUpdatable:  true" << endl;
				    else 
					    cout << "\t\tisUpdatable:  false" << endl;
			
				    if (attr->isQueryable())
					    cout << "\t\tisQueryable:  true" << endl;
				    else 
					    cout << "\t\tisQueryable:  false" << endl;
				    cout << " " << endl;

                    delete attr;
			    }
		    }
            delete [] attrNames;
            delete entityFed;
	    }
        delete [] names;


      // disconnects
      cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
      
    }
    catch(DKException &exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      
      cout << "Exception Class Name: " << exc.name() << endl;
    }
   
    cout << "done ..." << endl;   
}
