/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving an Attribute Definition
    Attribute Definitions exist in two forms, the general Attribute Definition and the
    tailored Attribute Definition for a specific Component Type.  Once an Item or 
    Component Type is created using an Attribute Definition, a seperate instance will
    be stored within the Component Type definition that will not be affected by any 
    modification of the general Attribute Definition.  This prevents breaking, destroying,
    or corrupting existing Data Models and existing Items of those Data Models.
    
    As explained in both SAttributeDefinitionCreationICM & SItemTypeCreationICM, Attributes
    may be tailored for a particular situation applicable to the specific Item/Component
    Type that it belongs to.  The desired definition must be retrieved from the correct
    source.
    
    Datastore Definition's Attributes
        To obtain the general Attribute Definition that the instances added to Item Types
        are based on, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To obtain the specific tailored version of an Attribute Definition for a perticular
        Item or Component Type, it must be retrieved from the Component Type Definition.
        
        Hint:  Use DKComponentTypeDef.retrieveAttr().  Please refer to SItemTypeRetrievalICM
               for more information on how to retrieve an existing Component Type Definition.

    Detecting Non-Existant Attribute Definitions
        When retrieving Attribute Definitions, if it does not exist, null is returned.  
        Usage of the null object will result in a NullPointerException.  If it is unknown
        whether or not an Item or Component Type may have this Attribute the returned object
        should be checked first for null.

This Sample will retrieve general Attribute Definitions from the Datastore Definition.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeDefinitionRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionRetrievalICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving & Printing Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Attribute Definitions
 *                    Printing Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeDefinitionRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes must already be defined.  If not, please run Sample
 *                    SAttributeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeDefinitionRetrievalICM.hpp
 *                    SAttributeDefinitionRetrievalICM_launch.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeDefinitionRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SAttributeDefinitionRetrievalICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================="     << endl;
        cout << "IBM Information Integrator for Content       v" << ver << endl;
        cout << "Sample Program:  SAttributeDefinitionRetrievalICM"     << endl;
        cout << "-------------------------------------------------"     << endl;
        cout << " Database: " << database                               << endl;
        cout << " UserName: " << userName                               << endl;
        cout << "=================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_driverLicense Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            //
            // NOTE:  When retrieving Attribute Definitions, if it does not exist, null is 
            //        returned.  Usage of the null object will result in a NullPointerException.
            cout << "Retrieving the Attribute Definition of 'S_driverLicense'..." << endl;

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();           // Get the datastore definition object from the connected datastore.
                DKAttrDefICM* attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_driverLicense"); // Retrieve the definition.
                
            cout << "Retrieved the Attribute Definition of 'S_driverLicense'." << endl;
            
            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            cout << "Printing the Attribute Definition of 'S_driverLicense'..." << endl;

                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
                delete(attrDef);       // Free Memory
            
            cout << "Printed the Attribute Definition of 'S_driverLicense'." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_claimNumber Attribute Definition
            //----------------------------------------------------------------
            cout << "Retrieving the Attribute Definition of 'S_claimNumber'..." << endl;

                attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_claimNumber"); // Retrieve the definition.
                
            cout << "Retrieved the Attribute Definition of 'S_claimNumber'." << endl;

            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            cout << "Printing the Attribute Definition of 'S_claimNumber'..." << endl;

                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
                delete(attrDef);       // Free Memory

            cout << "Printed the Attribute Definition of 'S_claimNumber'." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_accidentDate Attribute Definition
            //----------------------------------------------------------------
            cout << "Retrieving the Attribute Definition of 'S_accidentDate'..." << endl;

                attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_accidentDate"); // Retrieve the definition.
                
            cout << "Retrieved the Attribute Definition of 'S_accidentDate'." << endl;

            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            cout << "Printing the Attribute Definition of 'S_accidentDate'..." << endl;
                
                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
                delete(attrDef);       // Free Memory

            cout << "Printed the Attribute Definition of 'S_accidentDate'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print an Attribute Definition.
    * @param attrDef  Attribute Definition, DKAttrDefICM.
    **/
    void SAttributeDefinitionRetrievalICM::printAttrDef(DKAttrDefICM * attrDef){
        cout << "---------------------------------------------"   << endl;
        cout << "            Attribute Definition"                << endl;
        cout << "---------------------------------------------"   << endl;
        cout << "           Name:  " << attrDef->getName()        << endl;
        cout << "    Description:  " << attrDef->getDescription() << endl;
        short type = attrDef->getType();
        switch(type){
        case DK_CM_BLOB:
                cout << "           Type:  Blob"          << endl;
                break;
            case DK_CM_CHAR:
                cout << "           Type:  Char"          << endl;
                break;
            case DK_CM_CLOB:
                cout << "           Type:  Clob"          << endl;
                break;
            case DK_CM_DATE:
                cout << "           Type:  Date"          << endl;
                break;
            case DK_CM_DECIMAL:
                cout << "           Type:  Decimal"       << endl;
                break;
            case DK_CM_DOUBLE:
                cout << "           Type:  Double"        << endl;
                break;
            case DK_CM_INTEGER:
                cout << "           Type:  Integer"       << endl;
                break;
            case DK_CM_SHORT:
                cout << "           Type:  Short"         << endl;
                break;
            case DK_CM_TIME:
                cout << "           Type:  Time"          << endl;
                break;
            case DK_CM_TIMESTAMP:
                cout << "           Type:  Timestamp"     << endl;
                break;
            case DK_CM_VARCHAR:
                cout << "           Type:  Varchar"       << endl;
                break;
            default:
                cout << "           Type:  Other (" << type << ")" << endl;
                break;
        }//end switch
        //Not C++:  if((attrDef->getDefaultValue())==NULL)
        //Not C++:      cout << "  Default Value: null"               << endl;
        //Not C++:  else{
            // Determine if Current Time, Date, Timestamp, or Zero Number are Set
            if( (strcmp(attrDef->getDefaultValue(),"")==0) && (attrDef->getDefaultType()!=DK_ATTR_TYPE_DEFAULT_UNDEFINED)){ // If not off, assume valid
                if(type==DK_CM_DATE)
                    cout << "  Default Value:  Current Date" << endl;
                else if(type==DK_CM_TIME)
                    cout << "  Default Value:  Current Time" << endl;
                else if(type==DK_CM_TIMESTAMP)
                    cout << "  Default Value:  Current Timstamp" << endl;
                else if( (type==DK_CM_SHORT) || (type==DK_CM_INTEGER) || (type==DK_CM_DOUBLE) || (type==DK_CM_DECIMAL) )
                    cout << "  Default Value:  Zero Number" << endl;
                else
                    cout << "  Default Value:  " << attrDef->getDefaultValue() << endl;
            } else // Otherwise normal default value
                cout << "  Default Value:  " << attrDef->getDefaultValue() << endl;
        //Not C++:  }
        switch(attrDef->getDefaultType()){
            case DK_ATTR_TYPE_DEFAULT_UNDEFINED:
                cout << "           Type:  Undefined / Off" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_STRING:
                cout << "           Type:  String / On" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_FLOAT:
                cout << "           Type:  Float" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_DECIMAL:
                cout << "           Type:  Decimal" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_INTEGER:
                cout << "           Type:  Integer" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_HEXSTRING:
                cout << "           Type:  Hex String" << endl;
                break;
            case DK_ATTR_TYPE_DEFAULT_OTHER:
                cout << "           Type:  Other" << endl;
                break;
            default:
                cout << "           Type:  Unknown (" << type << ")" << endl;
                break;
            
        }//end switch on Default Type
            
        if(attrDef->isNullable())
            cout << "       Nullable:  True"              << endl;
        else
            cout << "       Nullable:  False"             << endl;
        if(attrDef->isUnique())
            cout << "         Unique:  True"              << endl;
        else
            cout << "         Unique:  False"             << endl;
        if(attrDef->isTextSearchable())
            cout << "Text Searchable:  True"              << endl;
        else
            cout << "Text Searchable:  False"             << endl;
        if(attrDef->isReadable())
            cout << "       Readable:  True"              << endl;
        else
            cout << "       Readable:  False"             << endl;
        if(attrDef->isWriteable())
            cout << "      Writeable:  True"              << endl;
        else
            cout << "      Writeable:  False"             << endl;
        cout << "          Min:  " << attrDef->getMin()   << endl;
        cout << "          Max:  " << attrDef->getMax()   << endl;
        cout << "         Size:  " << attrDef->getSize()  << endl;
        cout << "---------------------------------------" << endl;
    }

   /**
    * Print an Attribute Definition
    * @param dsICM     Connected DKDatastoreICM object.
    * @param attrName  Attribute Name.
    **/
    void SAttributeDefinitionRetrievalICM::printAttrDef(DKDatastoreICM * dsICM, char * attrName){
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();  // Get the datastore definition object from the connected datastore.
        DKAttrDefICM* attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr(attrName); // Retrieve the definition.
        printAttrDef(attrDef);   

        delete(attrDef);   // Free Memory
    }

   /**
    * Determine if the Attribute Definition exists in the Datastore Definition.
    * @param dsICM       Conncted DKDatastoreICM object.
    * @param attrName    Name of the Attribute Definition.
    * @return  Returns true if it exists, false if not.
    */
    bool SAttributeDefinitionRetrievalICM::doesAttrDefExist(DKDatastoreICM * dsICM, char * attrName){

        bool found = false;

        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM *) dsICM->datastoreDef();
        DKSequentialCollection* attrDefColl = (DKSequentialCollection *) dsDefICM->listAttrs();  // Get a collection containing all Attribute Definitions.
		if (attrDefColl && (attrDefColl->cardinality()>0) )
		{

			dkIterator* iter = attrDefColl->createIterator();    // Create an iterator to iterate through the collection
			while(iter->more()){                                 // while there are still items in the list, continue
				dkAttrDef* attrDef = (dkAttrDef *) iter->next()->value(); // Move pointer to next element & return that object	.
				if(!found) // Once found, save time by no longer doing string compares, but still continuing to free memory            
					if(strcmp(attrDef->getName(),attrName)==0)
						found = true;  // found it.
				delete(attrDef);       // Free Memory.  Most Sequential Collections must have their items deleted explicitly.
			}

			delete(iter);          // Free Memory
			delete(attrDefColl);
		}
        
        return(found);
    }
