/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    Doc Model Items are an extension of non-resource items.  Doc Model Parts are resource
    items.  Therefore, both the standar item & resource item samples are pre-requisit 
    knowledge.  Please review the SItem... samples and the SResourceItem... samples.

CM Document Model - Documents Modeled by Parts
    A document model is provided that structures documents as an Item made up of
    parts.  A document within the CM Document Model is an item that that extends
    the full functionality of the non-resource items to include parts.  Each part
    can be a number either a system-defined part or possibly a user-defined part
    that will each contain resource content.  A document may contain any number
    of parts for each part type selected.

    System Defined Part Types
        There are a number of system-defined part types which must be selected
        for use with the item type at design time.  Some or all of the part types
        may be selected.  

      Part Name        Semantic Type                Description
    -------------  -------------------------------  -----------------------------------------------
    ICMANNOTATION  DK_ICM_SEMANTIC_TYPE_ANNOTATION  LOB Resource that holds an annotation.
    ICMBASE        DK_ICM_SEMANTIC_TYPE_BASE        LOB Resource that holds part of the doc.
    ICMBASETEXT    DK_ICM_SEMANTIC_TYPE_BASETEXT    Text Resource that holds part of the doc.
    ICMBASESTREAM  DK_ICM_SEMANTIC_TYPE_BASESTREAM  Stream Resource that holds part of the doc.
    ICMNOTELOG     DK_ICM_SEMANTIC_TYPE_NOTE        Text Resource for any use.  Ex: Change History.
    -----------------------------------------------------------------------------------------------
        Note:  Constants listed are defined in DKConstantICM.h
               Part names are specified using string values containing the name listed.

    Adding / Modifying / Removing Parts
        Parts may be added or removed by simply adding or removing them from the Parts
        collection in the document.  Parts are modified by modifying them in the document
        item.  Do not call add(), update(), or del() directly on the Parts.  The document
        will be responsible for managing the parts.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKParts.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SDocModelItemICM.hpp"


/************************************************************************************************
 *          FILENAME: SDocModelItemICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating / Updating / Deleting Document Model Items, Creating Parts,
 *                    Accessing the Parts Collection, Adding Parts to a Document, Removing Parts
 *                    from a Document, Deleting Parts
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating / Updating / Deleting Document Model Items
 *                    Creating Parts
 *                    Accessing the Parts Collection
 *                    Adding Parts to a Document 
 *                    Removing Parts from a Document
 *                    Deleting Parts
 *                    List Document Parts
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SDocModelItemICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                        - SResourceMgrDefSetDefaultICM
 *                        - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SDocModelItemICM_launch.cpp
 *                    SDocModelItemICM.hpp
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Text2.txt
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SDocModelItemICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SDocModelItemICM <database> <userName> <password>"           << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SResourceItemICM"               << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create a Document
            //-------------------------------------------------------------
            // Please refer to SItemCreationICM for all non-resource related
            // information.
            //
            // Remember that "S_docModel" is an Item Type defined in
            // SItemTypeCreationICM.
            cout << "Creating a Document..." << endl;

                DKDDO* ddoDocument = dsICM->createDDO("S_docModel", DK_CM_DOCUMENT);

            cout << "Created a Document." << endl;

            //-------------------------------------------------------------
            // Create Parts
            //-------------------------------------------------------------
            // Please refer to SResourceItemCreationICM for all resource
            // related information.
            cout << "Creating Parts..." << endl;

                DKLobICM*   base      = (DKLobICM*)  dsICM->createDDO("ICMBASE",     DK_ICM_SEMANTIC_TYPE_BASE);
                DKTextICM*  baseText1 = (DKTextICM*) dsICM->createDDO("ICMBASETEXT", DK_ICM_SEMANTIC_TYPE_BASETEXT);
                DKTextICM*  baseText2 = (DKTextICM*) dsICM->createDDO("ICMBASETEXT", DK_ICM_SEMANTIC_TYPE_BASETEXT);

                cout << "Setting Parts' MIME Type / Type of Contents..." << endl;
                base->setMimeType("application/msword");
                baseText1->setMimeType("text/plain");
                baseText2->setMimeType("text/plain");

                cout << "Loading Content into Parts..." << endl;
                base->setContentFromClientFile("SResourceItemICM_Document1.doc"); // Load the file into memory.
                baseText1->setContentFromClientFile("SResourceItemICM_Text1.txt");
                baseText2->setContentFromClientFile("SResourceItemICM_Text2.txt");

            cout << "Created Parts." << endl;

            //-------------------------------------------------------------
            // Accessing the DKParts Attribute
            //-------------------------------------------------------------
            // The DKParts attribute is accessed the same way shown in 
            // SItemCreationICM & SItemRetrievalICM.  Note that this is
            // the same way a DKFolder collection is accessed.
            //
            // This attribute can be found programatically as described in
            // Sample SItemRetrievalICM.
            cout << "Accessing the DKParts Attribute..." << endl;
            
                DKParts* dkParts = (DKParts*)(dkCollection*) ddoDocument->getData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKPARTS)); 
                
                // Important:  If the DDO is not retrieved, the above may
                //             fail due to a null pointer exception.  The
                //             following accounts for this.
                
                short dataid = ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKPARTS); 
                
                if(dataid==0)
                    throw DKException("No DKParts Attribute Found!  DDO is either not a CM Document Model Item or Document has not been explicitly retrieved.");
                    
                dkParts = (DKParts*)(dkCollection*) ddoDocument->getData(dataid); 

            cout << "Accessed the DKParts Attribute." << endl;

            //-------------------------------------------------------------
            // Adding Parts to Document
            //-------------------------------------------------------------
            // Use the "Element" methods to group all updates into a single
            // call to the Library Server.  "Member" methods, if available,
            // are immediately persistent, but at a cost of a Library Server
            // call for every single call.
            cout << "Adding Parts to Document..." << endl;
            
                dkParts->addElement((dkDataObjectBase*)(DKDDO*)base);
                dkParts->addElement((dkDataObjectBase*)(DKDDO*)baseText1);
                dkParts->addElement((dkDataObjectBase*)(DKDDO*)baseText2);

            cout << "Added Parts to Document." << endl;

            //-------------------------------------------------------------
            // Adding New Document to Persistent Datastore
            //-------------------------------------------------------------
            cout << "Adding New Document to Persistent Datastore..." << endl;
        
                ddoDocument->add(); 

            cout << "Added New Document to Persistent Datastore." << endl;

            //-------------------------------------------------------------
            // Printing Document Part List
            //-------------------------------------------------------------
            cout << "Printing Document Part List..." << endl;

                printDocumentPartList(ddoDocument);

            cout << "Printed Document Part List." << endl;

            //-------------------------------------------------------------
            // Check out the Document for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking Document Before Update..." << endl;

                dsICM->checkOut(ddoDocument);  // Must check out / lock before updating.

            cout << "Checked Out / Locked Docuent Before Update." << endl;

            //-------------------------------------------------------------
            // Delete a Part / Remove a Part 
            //-------------------------------------------------------------
            cout << "Deleting a Part / Removing a Part..." << endl;

                // Look for the Part to Remove.  In this case, we will look for 
                // the baseText2.
                dkIterator* iter = dkParts->createIterator();
                while(iter->more()){                                        // while there are still items, continue searching
                    DKLobICM* part = (DKLobICM*) iter->next()->value();     // Move pointer to next element & return that object.
                    if(strcmp(((DKPidICM*)part->getPidObject())->pidString(),((DKPidICM*)baseText2->getPidObject())->pidString())==0){

                        dkParts->removeElementAt(*iter);  // Now that we found baseText2, remove it.

                    }
                }
                delete(iter);   // Free Memory
            
            cout << "Removed a Part from a Document." << endl;

            //-------------------------------------------------------------
            // Make Changes Persistent
            //-------------------------------------------------------------
            cout << "Making Changes Persistent..." << endl;
        
                ddoDocument->update(DK_CM_CHECKIN);   // Don't forget to explicitly check in when finished.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Recreate DDO to Demonstrate Retrieval
            //-------------------------------------------------------------
            cout << "Recreating Document DDO..." << endl;
            
                DKString ddoDocumentPidString = ((DKPidICM*)ddoDocument->getPidObject())->pidString();  // Get the PID String
                ddoDocument = dsICM->createDDO(ddoDocumentPidString);                   // Recreate Blank DDO Using PID String.

            cout << "Recreated Document DDO." << endl;

            //-------------------------------------------------------------
            // Retrieving Document
            //-------------------------------------------------------------
            // This section covers retrieving a Document Item with the 
            // DKParts attribute / collection.
            //
            // Please review SItemRetrievalICM's retrieval option documentation.
            cout << "Retrieving Document..." << endl;
            
                ddoDocument->retrieve(DK_CM_CONTENT_ITEMTREE);

            cout << "Retrieved Document." << endl;

            //-------------------------------------------------------------
            // Printing Document Part List
            //-------------------------------------------------------------
            cout << "Printing Document Part List..." << endl;

                printDocumentPartList(ddoDocument);

            cout << "Printed Document Part List." << endl;

            //-------------------------------------------------------------
            // Deleting Document
            //-------------------------------------------------------------
            cout << "Deleting Document..." << endl;

                ddoDocument->del();
                
                delete(ddoDocument);     // Free Memory

            cout << "Deleted Document." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print Document Part List
    * @param document  CM Document Model Item.
    **/
    void SDocModelItemICM::printDocumentPartList(DKDDO* document){
        
        DKString documentItemId = ((DKPidICM*)document->getPidObject())->getItemId();

        // Get the DKParts object.        
        short dataid = document->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKPARTS); 
        if(dataid==0)
            throw DKException("No DKParts Attribute Found!  DDO is either not a CM Document Model Item or Document has not been explicitly retrieved.");
        DKParts* dkParts = (DKParts*)(dkCollection*) document->getData(dataid); 
        
        // Print the list        
        cout << "Document (" << documentItemId << ") Parts:" << endl;
        dkIterator* iter = dkParts->createIterator();       // Create an Iterator go to through DDO Collection.
        while(iter->more()){                                // While there are items still to be printed, continue
            DKDDO* part = (DKDDO*) iter->next()->value();   // Move pointer to next element & return that object.
            cout << "     Item Id:  " << ((DKPidICM*)part->getPidObject())->getItemId() << " (" << ((DKPidICM*)part->getPidObject())->getObjectType() << ")" << endl;
        }
        delete(iter);      // Free Memory
    }
