/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Reference Attribute (Group):
    A Reference Attribute (Group) is an attribute (group) that refers to 
    another Item and possibly by a specific version of that Item.  A group
    of 6 system-defined attributes are automatically added and managed
    by the system to adequately represent the information needed to point 
    to a specific component, possibly by a specific version.  Officially 
    implemented as a group of attributes, called an Attribute Group, it 
    is often referred to simply as a "Reference Attribute" because only 
    together they are used to reference a single Item.

    The only time that the Reference Attribute is used as like an 
    Attribute Group is when at desing time, during Datamodel Definition, updating,
    retrieving, or deleting.  Once it is defined and added to a Component Type,
    it will be used just Attributes are.

    Reference Delete Rule (Design-Time Property)
        A Delete Rule can be specified, which tells the system what to do
        with this item when an item it references is deleted.

        Item Type Design-Time Property
            The Reference Attribute's Delete Rule is only applicable to the specific Item
            or Component Type that it is applied to.  This property pertains to the 
            individual Reference Attribute when added to an Item or Component Type, which
            will be demonstrated in SItemTypeCreationICM.
        
        IMPORTANT:  The Delete Rule setRefDeleteRule() operation only applies to the
                    Reference Attribute in memory to be used when adding to an Item or
                    Component Type.  It will not be saved persistently within the System's
                    Definition object and will no longer be set when retrieved at a later
                    time.  One would not specify this property in this sample because the
                    Reference Attribute definition object in local memory is not being
                    added to a Component Type at this time.
        
           Delete Rules               Constant
        ------------------  ---------------------------------
        Restrict (Default)  DK_ICM_DELETE_RULE_RESTRICT
        Cascade             DK_ICM_DELETE_RULE_CASCADE
        Set Null            DK_ICM_DELETE_RULE_SET_NULL
        No Action           DK_ICM_DELETE_RULE_NO_ACTION
        -----------------------------------------------------
        Note: Constants above are defined in DKConstantICM.h

        Restrict (Default)
            Restricts / Does not allow the deletion of an item referenced by this
            attribute.  The value of this attribute must be manually set to null or
            the item that contains this attribute must be deleted manually before 
            the item it references can be deleted.
            
        Cascade
            Deletes any item with this reference attribute that refers to 
            an item that is deleted.
            
        Set Null
            If the item referenced is deleted, the value of the reference attribute
            will automatically be set to null.
            
        No Action
            If the item referenced is deleted, no action will take place in terms 
            of the item containing the reference attribute that refers to the deleted 
            item or to the value of the reference attribute.  This will leave 
            the reference attribute in an inconsistent state since it will no 
            be pointing to a valid existing item.

    Using Reference Attributes (Setting / Modifying)
        Reference Attributes appear in a DDO just as attributes do.  They are
        set, modified, and accessed just as attributes are, not attribute groups
        In this case, a DKAny encapsulating a DDO is passed in or retrieved.  The
        system will automatically handle and manage the attributes that make up 
        the group.

      	Attr Type            Object                   DKAny Type Code
    	-------------------- ------------------------ -------------------
    	Reference Attribute  DKAny(dkDataObjectBase)  tc_dobase

        Item Types
            Reference Attributes may not reference an Item of the same Item Type
            without restriction.  References are meant to reference Items of 
            different Item Types.  It is possible to reference an Item of the same
            Item Type, but setting & modifying the reference attribute must made
            on an item with all other changes already persistent, and modifications 
            to the reference attribute value only made persistent without other changes.
            
            Example Procedure For This Special Case:
                1. Create DDO A & B.
                2. Set any other values in A & B, including Adding Child Components.
                3. Add BOTH A & B to the datastore..
                4. Check Out / Lock A.
                5. Set A's Reference Attribute to B.
                6. Update A.

        A Reference Attribute is set using a DKDDO object.  For a demonstraction of
        usage, please refer to SItemCreationICM.
    
        Important:  At the time the Reference Attribute is set, the ddo it will 
                    reference must be persistent in the datastore.  This means that
                    the DKDDO.add() operation has been completed on the DKDDO that 
                    it will reference, or it has been recreated given the full PIDs 
                    string (See SItemRetrievalICM Sample), or it has been retrieved 
                    through Retrieval or Query/Search.

    Defining Reference Attributes
        In order to create a Reference Attribute (Group), an Attribute Group
        must be created, specifying the Reference Attribute Option, 
        DKConstantICM.DK_ICM_REFERENCE_ATTRGROUP, and the six system-deined 
        attributes will automatically be added.

    The System-Defined Attributes that Make Up the Reference Attribute (Group):
    --------------------------------------------------------------------------
	Description               Attr Name / Constant
	------------------------  ------------------------------------------------
	Target Item ID            DK_ICM_SYSATTRID_TARGETITEMID
	Target Item Type ID       DK_ICM_SYSATTRID_TARGETITEMTYPEID
	Target Component ID       DK_ICM_SYSATTRID_TARGETCOMPONENTID
	Target Component Type ID  DK_ICM_SYSATTRID_TARGETCOMPTYPEID
	Target Version ID         DK_ICM_SYSATTRID_TARGETVERSIONID
	Referenital Integrity ID  DK_ICM_SYSATTRID_TARGETRRID
    --------------------------------------------------------------------------
    Note: These are automatically handled by the system and should not be modified.
    Note: Constants displayed here are defined in DKConstantICM.h

    References vs. Links:
        References are similar to links, except that references may appear in 
        both root and child component types, may refer to a specific version of
        an item, and are modeled at design-time / definition time.  References
        may have a delete rule set, which can delete items referenced if the
        item is deleted.  References may not reference an Item of the same Item
        Type without restrictions on usage.
        
        Links are a loose and free-form concept allowing maximum flexability.  
        They are specified at run-time, can be used with any Item of any Item Type, 
        can easily be defined and used in large numbers, may take the form of many
        Link Types, & may have descriptive DDOs called Link Items.  Links to not 
        imply ownership.  When an item assiciated in a link is deleted, the link
        is simply broken and all items referenced remain in the system.  For more 
        information on Links, please consult the Samples on Linking, SLinksICM.

NOTE: Attribute Group names are limited to 15 characters.  Use the descriiption
      field to add more detailed information.

NOTE: Names only need to be unique among those of its own namespace.  The 
      Attribute namespace is seperate from the Entity Type namespace, Link Type 
      Namespace, etc.

-------------------------------------
Insurance Model Reference Attributes:
-------------------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the attribute
      group is used by and was create by the samples.  This helps avoid using
      attribute group names that may already be in use.

       Reference 
    Attribute Group  Part of Insurance Model Used In
    ---------------  -------------------------------
    S_autoClaim      Adjuster Report

----------------------------------------------------
Reference Attributes (Groups) Defined in this Sample
----------------------------------------------------

    Reference Attribute  Description
    -------------------  --------------------------------
    S_autoClaim          Reference to Original Auto Claim
    S_myRef              My Reference Attribute

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include "SConnectDisconnectICM.hpp"
#include "SReferenceAttrDefCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining & Explaining Reference Attributes, Delete Rules, Insurance Model 
 *                    Reference Attributes
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Reference Attribute (Groups)
 *                    Setting the Reference Delete Rule
 *                    Reference Attribute (Group) Name Requirements
 *                    Defining Reference Attributes (Groups) used by samples & Insurance Model
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SReferenceAttrDefCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here must not exist.  If they have already been
 *                    created, please run SReferenceAttrDefDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SReferenceAttrDefCreationICM_launch.cpp
 *                    SReferenceAttrDefCreationICM.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SReferenceAttrDefCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                          << endl;
            cout << "  SReferenceAttrDefCreationICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."           << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="     << endl;
        cout << "IBM Information Integrator for Content   v" << ver << endl;
        cout << "Sample Program:  SReferenceAttrDefCreationICM"     << endl;
        cout << "---------------------------------------------"     << endl;
        cout << " Database: " << database                           << endl;
        cout << " UserName: " << userName                           << endl;
        cout << "=============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Define a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using attribute group names that may already be in use.
            //
            // A Delete Rule can be specified which determines whether or not
            // items referenced are automatically deleted when an item is
            // deleted.
            //
            // The name of this attribute group can be the same as an item or
            // component type.  This does not impact the reference in any way.
            // Since the Attribute namespace is seperate from the Item/Component
            // Type namespace, names only need to be unique among those of its
            // own namespace.
            cout << "Defining Reference Attribute 'S_autoClaim'..." << endl;
        
                // Create a new Ref Attr Definition Object.  Use the AttrGroup using the Reference Option.
                DKAttrGroupDefICM* refAttrGroup = new DKAttrGroupDefICM(dsICM, DK_ICM_REFERENCE_ATTRGROUP);
                
                refAttrGroup->setName("S_autoClaim");                                  // Set a name, maximum 15 characters
                refAttrGroup->setDescription("Reference to Original Auto Claim");      // For more description than name.

                refAttrGroup->add();                     // Add to persistent datastore definition

                delete(refAttrGroup);                    // Free Memory
        
            cout << "Defined Reference Attribute 'S_autoClaim'." << endl;

            //--------------------------------------------------------------
            // Define another Reference Attribute (Group)
            //--------------------------------------------------------------
            // Note that there is a wrapper function at the bottom of this sampe
            // demonstrating this functionality as well.
            cout << "Defining Reference Attribute 'S_myRef'..." << endl;
        
                // Create a new Ref Attr Definition Object.  Use the AttrGroup using the Reference Option.
                refAttrGroup = new DKAttrGroupDefICM(dsICM, DK_ICM_REFERENCE_ATTRGROUP);
                
                refAttrGroup->setName("S_myRef");                          // Set a name, maximum 15 characters
                refAttrGroup->setDescription("My Reference Attribute");    // For more description than name.

                refAttrGroup->add();                     // Add to persistent datastore definition

                delete(refAttrGroup);                    // Free Memory
        
            cout << "Defined Reference Attribute 'S_myRef'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Define a Reference Attribute (Group) with the given name & description.
    * Reference Attributes are actually Reference Attrube Groups.  Once defined,
    * they are accessed and modified just as regular attributes are.  The system
    * will manage the underlying system-defined attributes that make up the Reference
    * Attribute (Group).  Don't forget to delete the returned pointer when it is no
    * longer needed.
    * @param dsICM      Connected DKDatastoreICM object.
    * @param name       Name (15 char max) for the new Reference Attribute.
    * @param desc       Description for the new Reference Attribute.
    * @return Returns a reference to the newly defined Reference Attribute (Group).
    **/
    DKAttrGroupDefICM* SReferenceAttrDefCreationICM::defineReferenceAttr(DKDatastoreICM * dsICM, char * name, char * desc){
        cout << "Defining Reference Attribute '" << name << ":  " << desc << "'..." << endl;
        
            // Create a new Ref Attr Definition Object.  Use the AttrGroup using the Reference Option.
            DKAttrGroupDefICM* refAttrGroup = new DKAttrGroupDefICM(dsICM, DK_ICM_REFERENCE_ATTRGROUP);
            
            refAttrGroup->setName(name);                 // Set a name, maximum 15 characters
            refAttrGroup->setDescription(desc);          // For more description than name.

            refAttrGroup->add();                         // Add to persistent datastore definition
        
        cout << "Defined Reference Attribute '" << name << ":  " << desc << "'." << endl;
        return(refAttrGroup);
    }
