/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SSMSCollectionDefRetrievalICM.hpp"
#include "SSMSCollectionDefDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting SMS Collection Definitions, Deletes 3 Created in Creation Sample.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes SMS Collection Definitions.
 *                    Deletes SMS Collections Created by SSMSCollectionDefCreationICM.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SSMSCollectionDefDeletionICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SSMSCollectionDefSetDefaultICM MY_RM
 *                    ex: SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined.  The 3 SMS Collections
 *                    listed as the system pre-determined names must be created, as done through
 *                    SSMSCollectionDefCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefDeletionICM_launch.cpp
 *                    SSMSCollectionDefDeletionICM.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSMSCollectionDefDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: " << endl;
            cout << "   SSMSCollectionDefDeletionICM"                                             << endl;
            cout << "                         <Existing RM User-Defined Reference Name>"          << endl;
            cout << "                         <database> <userName> <password>"                   << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"   << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"    << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"     << endl;
            cout << "        Sample."                                                             << endl;
            cout << endl;
            cout << "  NOTE: Defaults will be used if the database, username, & password are not" << endl;
            cout << "        specified.  All others are required."                                << endl;
            cout << endl;
            cout << "  ex: SSMSCollectionDefDeletionICM MY_RM"                                    << endl;
            cout << "  ex: SSMSCollectionDefDeletionICM MY_RM icmnlsdb icmadmin password"         << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="         << endl;
        cout << "IBM Information Integrator for Content   v" << ver     << endl;
        cout << "Sample Program:  SSMSCollectionDefDeletionICM"         << endl;
        cout << "---------------------------------------------"         << endl;
        cout << "           Database:  " << database                    << endl;
        cout << "           UserName:  " << userName                    << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName  << endl;
        cout << "=============================================\n"       << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            cout << "Obtaining RM Code for RM Definition '" << rmUserDefinedReferenceName << "'..." << endl;  
            
                if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false){ // if it is not setup, stop here
                    cout << "NOTE: Resource Manager Definition Named '" << rmUserDefinedReferenceName << "' Not Defined." << endl;
                    return(SConnectDisconnectICM_RC_SAMPLE_PASSED);  // since no work is to be done, simply return.
                }

                short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            cout << "Obtained RM Code '" << rmCode << "' for RM Definition '" << rmUserDefinedReferenceName << "'." << endl;            

            //-------------------------------------------------------------
            // Obtaining the Configuration Management Objects
            //-------------------------------------------------------------
            // The sample code blow demonstrates how to obtain objects for
            // Datastore Definition, Datastore Administration, Configuration
            // Management, and RM Configuration Management.
            cout << "Obtaining Configuration Management Objects..." << endl;

                DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
                DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
                DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
                DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            cout << "Obtained Configuration Management Objects." << endl;

            //-------------------------------------------------------------
            // Delete the SMS Collections Definition                        
            //-------------------------------------------------------------
            cout << "Deleting Resource Manager Definition..." << endl;

                // The Non-Media SMS Collection is added by default when the RM Def is created.
                // It will not be deleted here.  If it is desired to deleted it, the following
                // may be used:  rmCfg.delSMSCollection(DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode);
                
                // Delete only if they are not deleted already:
                if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode))
                    rmCfg->delSMSCollection(RM_SMS_COLLECTION_MEDIA, rmCode);

                if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode))
                    rmCfg->delSMSCollection(RM_SMS_COLLECTION_VIDEO, rmCode);

            cout << "Deleted Resource Manager Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
