/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SResourceItem3rdPartyRetrievalICM Sample
    This Sample is an extension of the documentation of the 3rd Party Retrieve
    Sample.  This Sample will explain concepts that pertain specifically to 
    Items with Video Resource Content.  Please refer to SResourceItem3rdPartyRetrieveICM
    for all information on retrieving to a 3rd party destination that is not
    specific to Video Resource Items.
    
Retrieving Video Items
    Video Items may be retrieved just as any resource item can through 
    standard DKLobICM functionality or through the 3rd Party Retrieval 
    (Retrieving to a 3rd-Pary Destination).  This particular sample will
    demonstrate the 3rd-Party Retrieval as shown in SResourceItem3rdPartyRetrieveICM.
    
    Additional Requirements
        Video Charger must installed and properly configured with Resource Manager.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKVideoStreamICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SVideoItemRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SVideoItemRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Items with Video Resource Content.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Items with Video Resource Content.
 *                    Retrieving Videos to a 3rd-Pary Destination.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SVideoItemRetrieveICM  <3rd Party Host Name>
 *                                           <3rd Party Port>
 *                                           <3rd Party Protocol (http|ftp|file)>
 *                                           <3rd Party User Name>
 *                                           <3rd Party Password>
 *                                           <3rd Party Fully-Qualified File Name>
 *                                           <3rd Party File Path>
 *                                           <3rd Party MIME Type>
 *                                           <3rd Party Content Length>
 *                                           <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required for this Sample, but not
 *                           all is required for Retrieval.  The full informaiton is needed
 *                           to create an item that will be retrieved.  Please refer to
 *                           SResourceItem3rdPartyRetrievalICM for minimum info needed.
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SVideoItemRetrievalICM_launch.cpp
 *                    SVideoItemRetrievalICM.hpp
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SVideoItemRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // 3rd Party Information
        char * thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        char * thirdPartyProtocol       = "NOT_SPECIFIED";
        char * thirdPartyUserName       = "NOT_SPECIFIED";
        char * thirdPartyPassword       = "NOT_SPECIFIED";
        char * thirdPartyFullFileName   = "NOT_SPECIFIED";     
        char * thirdPartyFilePath       = "NOT_SPECIFIED";
        char * thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 13) { // if not all 12 arguments were specified, report usage.
            cout << "Usage: "                                                                       << endl;
            cout << "  SVideoItemRetrievalICM  <3rd Party Host Name>"                               << endl;
            cout << "                          <3rd Party Port>"                                    << endl;
            cout << "                          <3rd Party Protocol (http|ftp|file)>"                << endl;
            cout << "                          <3rd Party User Name>"                               << endl;
            cout << "                          <3rd Party Password>"                                << endl;
            cout << "                          <3rd Party Fully-Qualified File Name>"               << endl;
            cout << "                          <3rd Party File Path>"                               << endl;
            cout << "                          <3rd Party MIME TYpe>"                               << endl;
            cout << "                          <3rd Party Content Length>"                          << endl;
            cout << "                          <database> <userName> <password>"                    << endl;
            cout << endl;
            cout << "  NOTE:  All 3rd Party Information is Required for this Sample, but not"       << endl;
            cout << "         all is required for Retrieval.  The full informaiton is needed"       << endl;
            cout << "         to create an item that will be retrieved.  Please refer to"           << endl;
            cout << "         SResourceItem3rdPartyRetrievalICM for minimum info needed."           << endl;
            cout << endl;
            cout << "  NOTE:  Database, UserName, & Password are optional.  If all not specified"   << endl;
            cout << "         defaults will be used."                                               << endl;
            cout << endl;
            cout << "  Ex: SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428"                            << endl;
            cout << "  Ex: SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password" << endl;
            cout << endl;
        }
        if(argc < 10){ // If any 3rd Party info not specified, terminate
            cout << "===================================================" << endl;
            cout << " TERMINATED:  All 3rd Party info must be specified." << endl;
            cout << "===================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  
        } else if(argc < 13) {
            cout << "*** Some parameters not specified.  Using defaults for rest. ***" << endl;
        }
        // Set Command-line options
        if (argc > 1)  thirdPartyHostName      = argv[1];
        if (argc > 2)  thirdPartyPort          = atoi(argv[2]);
        if (argc > 3)  thirdPartyProtocol      = argv[3];
        if (argc > 4)  thirdPartyUserName      = argv[4];
        if (argc > 5)  thirdPartyPassword      = argv[5];
        if (argc > 6)  thirdPartyFullFileName  = argv[6];
        if (argc > 7)  thirdPartyFilePath      = argv[7];
        if (argc > 8)  thirdPartyMIMEType      = argv[8];
        if (argc > 9)  thirdPartyContentLength = atoi(argv[9]);
        if (argc > 10) database                = argv[10];
        if (argc > 11) userName                = argv[11];
        if (argc > 12) password                = argv[12];
        
        char * ver = SConnectDisconnectICM_EIP_VERSION;
        
        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SVideoItemRetrievalICM"         << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Content Manager:"                               << endl;
        cout << "       Database:  " << database                  << endl;
        cout << "       UserName:  " << userName                  << endl;
        cout << " 3rd Party:"                                     << endl;
        cout << "      Host Name:  " << thirdPartyHostName        << endl;
        cout << "           Port:  " << thirdPartyPort            << endl;
        cout << "       Protocol:  " << thirdPartyProtocol        << endl;
        cout << "       UserName:  " << thirdPartyUserName        << endl;
        cout << "       Password:  " << thirdPartyPassword        << endl;
        cout << "           File:  " << thirdPartyFullFileName    << endl;
        cout << "      Mime Type:  " << thirdPartyMIMEType        << endl;
        cout << " Content Length:  " << thirdPartyContentLength   << endl;
        cout << "==========================================="     << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Video Resource Item
            //-------------------------------------------------------------
            // Create a Video Resource Item as shown in SVideoItemCreateICM.
            // For more information, please refer to that Sample.
            cout << "Creating Video Resource Item..." << endl;
    
                DKVideoStreamICM* video = (DKVideoStreamICM*)dsICM->createDDO("S_videoType", DK_CM_ITEM); 

                // Set the User-Defined Attributes
                video->setData(video->dataId(DK_CM_NAMESPACE_ATTR,"S_author"), DKString("IBM")); 
                video->setData(video->dataId(DK_CM_NAMESPACE_ATTR,"S_length"), DKString("48Hours")); 

                // Setting the MIME Type (type of content)
                video->setMimeType(thirdPartyMIMEType);

                // Set 3rd Party Server Info
                video->setThirdPartyHostname(thirdPartyHostName);
                video->setThirdPartyPort(thirdPartyPort);
                video->setInputProtocol(thirdPartyProtocol);
                video->setThirdPartyUserid(thirdPartyUserName);
                video->setThirdPartyPassword(thirdPartyPassword);
                video->setInputFilename(thirdPartyFullFileName);
                video->setInputFilepath(thirdPartyFilePath);
                video->setInputContentlength(thirdPartyContentLength);

                // Synchronous Store
                video->addFrom();  // Add from the 3rd Party specified above.

                // Saving Pid String for later retrieval
                char * videoPidString = ((DKPidICM*)video->getPidObject())->pidString();
                
                delete(video);     // Free Memory

            cout << "Created Video Resource Item." << endl;

            //-------------------------------------------------------------
            // Recreate Item from PID
            //-------------------------------------------------------------
            cout << "Recreating Item from PID..." << endl;

                video = (DKVideoStreamICM*) dsICM->createDDO(videoPidString);

            cout << "Recreated Item from PID." << endl;

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be obtained from a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // for more informaiton.
            cout << "Setting 3rd Party Server Info..." << endl;

                video->setThirdPartyHostname(thirdPartyHostName);
                video->setThirdPartyPort(thirdPartyPort);
                video->setInputProtocol(thirdPartyProtocol);
                video->setThirdPartyUserid(thirdPartyUserName);
                video->setThirdPartyPassword(thirdPartyPassword);
                video->setInputFilename(thirdPartyFullFileName);
                video->setInputFilepath(thirdPartyFilePath);

            cout << "Set 3rd Party Server Info." << endl;

            //-------------------------------------------------------------
            // Synchronous Retrieval
            //-------------------------------------------------------------
            cout << "Retrieving Synchronously..." << endl;

                video->retrieveTo();  // Update from the 3rd Party specified above.
                
            cout << "Retrieved Synchronously." << endl;

            //-------------------------------------------------------------
            // Asynchronous Retrieval
            //-------------------------------------------------------------
            cout << "Retrieving Asynchronously..." << endl;

                // REQUIREMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.

                //video->retrieveToAsync();  // Add from the 3rd Party specified above.

                delete(video);             // Free Memory

            cout << "Retrieved Asynchronously." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
