/*************************************************************************************
* @copyright(disclaimer) 
* Licensed Materials - Property of IBM
* IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
* (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
*
* US Government Users Restricted Rights
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp.
*
* DISCLAIMER OF WARRANTIES :
*
* Permission is granted to copy and modify this  Sample code, and to 
* distribute modified versions provided that both the copyright
* notice,- and this permission notice and warranty disclaimer  appear in all copies and
* modified versions.
*
* THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
* IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF
* NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING
* OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,
* OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS
* AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT,
* SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
* THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGES.
*
* @endCopyright
*/
/************************************************************************/
/*
 * Filename       : TDatastorePoolIP.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKBlobIP.hpp>
#include <DKDatastoreIP.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefIP.hpp>
#include <DKAttrDefIP.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    // Version of IP/390 with Annotation support  s is the connect_string
    DKString s("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolIP <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
	
#if _MFC_VER <  0x0700
	DKString factoryIPLName = "cmbipfac816d";
#else
	DKString factoryIPLName = "cmbipfac817d";
#endif

   try {
      DKDatastorePool* dsrIP = new DKDatastorePool("IP", factoryIPLName);
      dsrIP->setDatastoreName(libsrv);
      dsrIP->setMaxPoolSize(10);
      dsrIP->setMinPoolSize(3);
      //dsrIP->setTimeOut(1); 
      dsrIP->setConnectString(s); 

      dsrIP->initConnections(userid, pw, 2);
      dsrIP->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrIP->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrIP->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrIP->getTimeOut()<<endl;

      dkDatastore *dsIP1, *dsIP2, *dsIP3, *dsIP4;;
      dsIP1 = dsrIP->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsIP1(ds1) =  " << dsIP1 << endl;
      dsIP2 = dsrIP->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsIP2(ds2) =  " << dsIP2 << endl;

      DKDatastoreSession *dsIdIP1, *dsIdIP2, *dsIdIP3, *dsIdIP4;
      dsIdIP1 = dsrIP->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdIP1(dsid1)="<< dsIdIP1->getDatastore()
        <<", sessionId=" << dsIdIP1->getId()<<endl;
      dsIdIP2 = dsrIP->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdIP2(dsid2)="<<dsIdIP2->getDatastore()
        <<", sessionId=" << dsIdIP2->getId()<<endl;

      dsrIP->returnConnection(dsIdIP2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdIP2(dsid2) =  " << 
        dsIdIP2->getDatastore() << ", sessionId=" << dsIdIP2->getId()<<endl;
      dsrIP->returnConnection(dsIP2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsIP2(ds2) =  " << dsIP2 <<endl; 

      dsIdIP3 = dsrIP->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdIP3(ds2)=" <<
        dsIdIP3->getDatastore() << ", sessionId=" << dsIdIP3->getId()<<endl;
      dsIdIP4 = dsrIP->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdIP4(dsid3)=" <<
        dsIdIP4->getDatastore() << ", sessionId=" << dsIdIP4->getId() <<endl;


      dsIdIP4 = dsrIP->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdIP4(dsid4)=" <<
        dsIdIP4->getDatastore() << ", sessionId=" << dsIdIP4->getId()<<endl;

      dsIP1 = dsrIP->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsIP1(ds3) =  " << dsIP1 <<endl;
      dsIP2 = dsrIP->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsIP2(ds4) =  " << dsIP2 <<endl;
 
      dsIP3 = dsrIP->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsIP3(ds5) =  " << dsIP3 <<endl;
      dsIP4 = dsrIP->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsIP4(ds6) =  " << dsIP4 <<endl;


      dsIdIP3 = dsrIP->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdIP3(dsid2)=" <<
      dsIdIP3->getDatastore() << ", sessionId=" << dsIdIP3->getId()<<endl;


/**/
      //**** use dsIP2 to do ListCatalog
      cout<<"||**** start: use dsIP2 to do ListCatalog ****||"<<endl;

      // List entities...
      DKEntityDefIP* docDef = 0;
      DKAttrDefIP* attrDef = 0;

      cout << "---List entities---" << endl;
      DKSequentialCollection* pCol = (DKSequentialCollection*)(dsIP2->listEntities());
      dkIterator* pIter = 0;

      if ( pCol == 0 )
      {
         cout << "collection of entities is null!" << endl;
      }
      else
      {

         // Method 1:
         cout << "---List attributes using listAttrNames and getAttr methods---" << endl;

         pIter = pCol->createIterator();
         while (pIter->more())
         {
            docDef = (DKEntityDefIP*)(pIter->next()->value());
            cout << " Document type     : " << docDef->getType() <<docDef<<endl;
            cout << " Document type name: " << docDef->getName() <<docDef<<endl;

            long tmpCount;
            DKString* attrNames;

            // Upon return, tmpCount contains the number of elements in the list.
            attrNames = docDef->listAttrNames(tmpCount);

            for (int i=0
                 ; i<tmpCount
                 ; i++ )
             {
                  cout << "   Attr name before lookup " << attrNames[i] << endl;
                  attrDef = (DKAttrDefIP*)(docDef->getAttr(attrNames[i]));
                  cout << "   Attr name [" << i << "] : " << attrDef->getName() << endl;
                  cout << "   Attr id       : " << attrDef->getId() << endl;
                  cout << "   Entity name   : " << attrDef->getEntityName() << endl;
                  cout << "   Datastore name: " << attrDef->datastoreName() << endl;
                  cout << "   Attr type     : " << attrDef->getType() << endl;
                  cout << "   Attr restrict : " << attrDef->getStringType() << endl;
                  cout << "   Attr min val  : " << attrDef->getMin() << endl;
                  cout << "   Attr max val  : " << attrDef->getMax() << endl;
                  cout << "   Attr display  : " << attrDef->getSize() << endl;
                  cout << "   Attr precision: " << attrDef->getPrecision() << endl;
                  cout << "   Attr scale    : " << attrDef->getScale() << endl;
                  cout << "   Attr update   ? " << attrDef->isUpdatable() << endl;
                  cout << "   Attr nullable ? " << attrDef->isNullable() << endl;
                  cout << "   Attr queryable? " << attrDef->isQueryable() << endl;
                  cout << "" << endl;
                  delete attrDef;
             }  // end for

             delete [] attrNames;

          }  // end while
         delete pIter;

         // Method 2:
         cout << "---List attributes using listEntityAttrs method---" << endl;

         pIter = pCol->createIterator();
         while (pIter->more())
         {
            docDef = (DKEntityDefIP*)(pIter->next()->value());  // iterator returns DKAny*
            cout << " Document type     : " << docDef->getType() << endl;
            cout << " Document type name: " << docDef->getName() << endl;

            DKSequentialCollection* pAttrCol = (DKSequentialCollection*)(dsIP2->listEntityAttrs(docDef->getName()));
            if ( pAttrCol == 0 )
            {
               cout << "collection of entity attrs is null for entity "
                    << docDef->getName()
                    << endl;
            }
            else
            {
               int i=0;
               dkIterator* pAttrIter = pAttrCol->createIterator();
               while (pAttrIter->more())
               {
                  i++;
                  attrDef = (DKAttrDefIP*)(pAttrIter->next()->value());   // iterator returns DKAny*
                  cout << "   Attr name [" << i << "] : " << attrDef->getName() << endl;
                  cout << "   Attr id       : " << attrDef->getId() << endl;
                  cout << "   Entity name   : " << attrDef->getEntityName() << endl;
                  cout << "   Datastore name: " << attrDef->datastoreName() << endl;
                  cout << "   Attr type     : " << attrDef->getType() << endl;
                  cout << "   Attr restrict : " << attrDef->getStringType() << endl;
                  cout << "   Attr min val  : " << attrDef->getMin() << endl;
                  cout << "   Attr max val  : " << attrDef->getMax() << endl;
                  cout << "   Attr display  : " << attrDef->getSize() << endl;
                  cout << "   Attr precision: " << attrDef->getPrecision() << endl;
                  cout << "   Attr scale    : " << attrDef->getScale() << endl;
                  cout << "   Attr update   ? " << attrDef->isUpdatable() << endl;
                  cout << "   Attr nullable ? " << attrDef->isNullable() << endl;
                  cout << "   Attr queryable? " << attrDef->isQueryable() << endl;
                  cout << "" << endl;
                  delete attrDef;
               }  // end while
               delete pAttrIter;
            }
            delete pAttrCol;
            delete docDef;
         }  // end while
         delete pIter;
      }
      delete pCol;
      cout<<"||**** end: use dsIP2 to do ListCatalog ****||"<<endl;
      //****end of use dsIP2 to do ListCatalog 
/**/
      dsrIP->returnConnection(dsIP2);
      cout<<" #returnConnection, dsIP2(ds4) =  "<< dsIP2 <<endl;
      dsIP4 = dsrIP->getConnection(userid, pw);
      cout<<" #getConnection, dsIP4 =  "<< dsIP4 <<endl;
      cout<<"    dsIP4 == dsIP2 =  " <<(dsIP4 == dsIP2)<<endl;

    //**** you can add codes to use dsIP4 to do other thing here  
    //**** end of use dsIP4 to do other thing

      dsrIP->returnConnection(dsIP1);
      cout<<" #returnConnection, dsIP1(ds3) =  " << dsIP1 <<endl;
      dsrIP->returnConnection(dsIP3);
      cout<<" #returnConnection, dsIP3(ds5) =  " << dsIP3 <<endl;
      dsrIP->returnConnection(dsIdIP4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdIP4(dsid4) =  " <<
        dsIdIP4->getDatastore() << ", sessionId=" << dsIdIP4->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsIP2 = (DKDatastoreIP*)dsrIP->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsIP2(ds7) =  " << dsIP2<<endl;
      cout<<" #call delete dsrIP..." <<endl;
      delete dsrIP;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}