
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
// sample connect/execute/disconnect to IP
/*
 * Filename       : TExecuteIP.cpp
 * Description    : Connect to IP Datastore.
 *                  Create query string.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes
 *                  Disconnect from IP Datastore.
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreIP.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResultSetCursorIP.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobIP.hpp>
#include <DKPidXDOIP.hpp>
#include <DKEnvironment.hpp>
#include <dkXDO.hpp>
#include <DKParts.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //--------------------------------------------------------------
   // Checking for input parameters
   //--------------------------------------------------------------
    if (argc < 4)
   {
      cout<<"Usage: "<<endl;
      cout<<"  TExecuteIP <libsrv> <userid> <pw>" << endl;
      return;
   }
   // ### get input parameters
   DKString libsrv = DKString(argv[1]);
   DKString userid = DKString(argv[2]);
   DKString pw = DKString(argv[3]);
   DKDatastoreIP dsIP;

   try
   {
      cout << "---Connect to ImagePlus server---" << endl;

      // Version of IP/390 with Annotation support
      DKString s("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
      dsIP.connect(libsrv, userid, pw, s);

      DKResultSetCursorIP* pCur = 0;
      DKString var1 = "IPSAMP";

      DKString cmd = "SEARCH=(ENTITY=DOCUMENT,";
      cmd += "MAX_RESULTS=5,";
      cmd += "COND=( FolderId==";
      cmd += var1;
      cmd += ") );";
      cmd += "OPTION=(CONTENT=YES)";

      cout << "---Executing query against ImagePlus server---" << endl;
      cout << " parametric query string " << cmd << endl;

      pCur = (DKResultSetCursorIP*)(dsIP.execute(cmd));
      cout << "query executed" << endl;

      DKDDO *p = 0;
      dkDataObjectBase *pDOBase = 0;
      DKDDO *pDDO = 0;
      dkXDO *pXDO = 0;
      DKAny a;
      ushort j = 0;
      ushort k = 0;
      ushort val = 0;
      ushort cnt = 1;
      DKString strData = "";
      DKString strDataName = "";
      dkCollection* pdCol = 0;
      dkIterator*  pdIter = 0;
      ushort numDataItems = 0;

      for ( p = pCur->fetchNext()
          ; pCur->isValid() && !(pCur->isEnd())
          ; p = pCur->fetchNext() )
      {
         //p->retrieve();  // used when CONTENT=NO

         cout << "=====> Item "
              << cnt
              << " <=====" << endl;

         numDataItems = p->dataCount();
         k = p->propertyId(DK_PROPERTY_ITEM_TYPE);
         if (k > 0)
         {
            a = p->getProperty(k);
            val = (unsigned short)a;
            cout << "******************************" << endl;
            switch (val)
            {
               case DK_DOCUMENT :
               {
                   cout << "Item is document " << endl;
                  break;
               }
              case DK_FOLDER :
               {
                 cout << "Item is folder " << endl;
                  break;
               }
            }
            cout << "******************************" << endl;
         }
         const DKPid* ddoPid = p->getPidObject();

         cout << "Pid = " << ((DKPid*)ddoPid)->getPrimaryId() << endl;

         cout << "Number of Data Items " << numDataItems << endl;
         for (j = 1; j <= numDataItems; j++)
         {
            a = p->getData(j);
            strDataName = p->getDataName(j);
            cout << j
                 << ": Attribute Name   : "
                 << strDataName
                 << endl;

            switch (a.typeCode())
            {
               case DKAny::tc_string :
               {
                  strData = a;
                  cout << "   String Value     : "
                       << strData << endl;
                  break;
               }
               case DKAny::tc_short :
               {
                  short shortData = (short)a.value();
                  cout << "   Short Value      : "
                       << shortData << endl;
                  break;
               }
               case DKAny::tc_date :
               {
                  DKDate* dateData = (DKDate*)a.value();
                  cout << "   Date Value       : "
                       << dateData->getYear()
                       << "-"
                       << dateData->getMonth()
                       << "-"
                       << dateData->getDay()
                       << endl;
                  break;
               }
               case DKAny::tc_timestamp :
               {
                  DKTimestamp* tsData = (DKTimestamp*)a.value();
                  cout << "   Timestamp Value  : "
                       << tsData->getYear()
                       << "-"
                       << tsData->getMonth()
                       << "-"
                       << tsData->getDay()
                       << "-"
                       << tsData->getHours()
                       << "-"
                       << tsData->getMinutes()
                       << "-"
                       << tsData->getSeconds()
                       << "-"
                       << tsData->getNanos()
                       << endl;
                  break;
               }
               case DKAny::tc_collection :
               {
                  pdCol = a;
                  cout << "--------------------------------------" << endl;

                  pdIter = pdCol->createIterator();
                  ushort b = 0;
                  while (pdIter->more() == TRUE)
                  {
                    b++;
                     cout << "  -------------------------------------------------" << endl;
                    a = *(pdIter->next());
                    pDOBase = a;
                    if (pDOBase->protocol() == DK_PDDO)
                     {
                       pDDO = (DKDDO*)pDOBase;
                       cout << "  DKDDO object " << b << " in " << strDataName << " collection " << endl;
                       k = pDDO->propertyId(DK_PROPERTY_ITEM_TYPE);
                       if (k > 0)
                        {
                          a = pDDO->getProperty(k);
                          val = a;
                          cout << "  ******************************" << endl;
                          switch (val)
                           {
                             case DK_DOCUMENT :
                              {
                                 cout << "  Item is document " << endl;
                                break;
                              }
                             case DK_FOLDER :
                              {
                                 cout << "  Item is folder " << endl;
                                break;
                              }
                           }
                           cout <<"  ******************************" << endl;
                        }
                     }
                    else if (pDOBase->protocol() == DK_XDO)
                     {
                       pXDO = (dkXDO*)pDOBase;
                       cout << "  dkXDO object " << b << " in " << strDataName << " collection " << endl;
                        //  Retrieve and open instance handler for an XDO
                       pXDO->retrieve();
                       /*if ( !pXDO->isNull() )
                       {
                          // NOTE:  If the default viewer frnjview is not in one of your
                          //   PATH directories, then you may either:
                          //      1)  comment out the following line to avoid the error message
                          //   or 2)  add the frnjview directory to your PATH environment variable.
                          pXDO->open();
                       }*/
                     }
                  }

                  if (pdIter != 0)
                  {
                     delete pdIter;
                  }
                  DKParts* tmpPart = (DKParts*)pdCol;
                  if (b == 0)
                  {
                    cout << strDataName << " collection has no elements " << endl;
                  }
                  cout << "  -------------------------------------------------" << endl;

                  break;
               }
               default:
               {
                  if (p->isNull(j))
                  {
                     cout << "   Value            : <null>" << endl;
                  }
                  break;
               }
            }  // end switch...
         }

         cnt++;
         delete p;
      }  // end for

      cout << "Total Item count is " << cnt-1 << endl;
      if (pCur != 0)
         delete pCur;

      dsIP.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   cout << "done ..." << endl;
}

