/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   *
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolOD.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKBlobOD.hpp>
#include <DKDatastoreOD.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefOD.hpp>
#include <DKServerDefOD.hpp>
#include <DKAppGrpDefOD.hpp>
#include <DKFieldDefOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void ListAppGrps(DKDatastoreOD& dsOD);

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;
  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TDatastorePoolOD <libsrv> <userid> <pw> <connect_string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
#if _MFC_VER <  0x0700
  #ifndef _DEBUG
    DKString factoryDLLName = "cmbodfac816";
  #else
    DKString factoryDLLName = "cmbodfac816d";
  #endif
#else
  #ifndef _DEBUG
    DKString factoryDLLName = "cmbodfac817";
  #else
    DKString factoryDLLName = "cmbodfac817d";
  #endif
#endif
  DKString connect_string = "";
  if (argc > 4)
  {
    connect_string = argv[4];
  }
  DKServerDefOD * pSV = NULL;
  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;

  try
  {
    cout<<"\n******factoryDLLName = " << factoryDLLName << "********\n" <<endl;
    DKDatastorePool* dsrOD = new DKDatastorePool("OD", factoryDLLName);
    dsrOD->setDatastoreName(libsrv);
    dsrOD->setMaxPoolSize(10);
    dsrOD->setMinPoolSize(3);
    //dsrOD->setTimeOut(1);
    dsrOD->setConnectString(connect_string);

    dsrOD->initConnections(userid, pw, 2);
    dsrOD->setTimeOut(1);
    cout<<" #getMaxPoolSize =  " << dsrOD->getMaxPoolSize()<<endl;
    cout<<" #getMinPoolSize =  " << dsrOD->getMinPoolSize()<<endl;
    cout<<" #getTimeOut=  " << dsrOD->getTimeOut()<<endl;

    dkDatastore *dsOD1, *dsOD2, *dsOD3, *dsOD4;;
    dsOD1 = dsrOD->getConnection(userid, pw);  //new ds1
    cout<<" #getConnection, dsOD1(ds1) =  " << dsOD1 << endl;
    dsOD2 = dsrOD->getConnection(userid, pw);  // new ds2
    cout<<" #getConnection, dsOD2(ds2) =  " << dsOD2 << endl;

    DKDatastoreSession *dsIdOD1, *dsIdOD2, *dsIdOD3, *dsIdOD4;
    dsIdOD1 = dsrOD->getConnection(userid, pw, 0);        //new dsid1
    cout<<" #getConnection, dsIdOD1(dsid1)="<< dsIdOD1->getDatastore()
    <<", sessionId=" << dsIdOD1->getId()<<endl;
    dsIdOD2 = dsrOD->getConnection(userid, pw, 0);        //new dsid2
    cout<<" #getConnection, dsIdOD2(dsid2)="<<dsIdOD2->getDatastore()
    <<", sessionId=" << dsIdOD2->getId()<<endl;

    dsrOD->returnConnection(dsIdOD2);                     //dsid2 in _freeIdPool
    cout<<" #returnConnection, dsIdOD2(dsid2) =  " <<
     dsIdOD2->getDatastore() << ", sessionId=" << dsIdOD2->getId()<<endl;
    dsrOD->returnConnection(dsOD2);                       //ds2 in _freePool
    cout<<" #returnConnection, dsOD2(ds2) =  " << dsOD2 <<endl;

    dsIdOD3 = dsrOD->getConnection(userid, pw, 1); //ds2 in _usedPool
    cout<<" #getConnection, dsIdOD3(ds2)=" <<
     dsIdOD3->getDatastore() << ", sessionId=" << dsIdOD3->getId()<<endl;
    dsIdOD4 = dsrOD->getConnection(userid, pw, 5); //new dsid3
    cout<<" #getConnection, dsIdOD4(dsid3)=" <<
     dsIdOD4->getDatastore() << ", sessionId=" << dsIdOD4->getId() <<endl;


    dsIdOD4 = dsrOD->getConnection(userid, pw, 6); // new dsid4
    cout<< " #getConnection, dsIdOD4(dsid4)=" <<
     dsIdOD4->getDatastore() << ", sessionId=" << dsIdOD4->getId()<<endl;

    dsOD1 = dsrOD->getConnection(userid, pw);  // new ds3
    cout<<" #getConnection, dsOD1(ds3) =  " << dsOD1 <<endl;
    dsOD2 = dsrOD->getConnection(userid, pw);  // new ds4
    cout<<" #getConnection, dsOD2(ds4) =  " << dsOD2 <<endl;

    dsOD3 = dsrOD->getConnection(userid, pw);  // new ds5
    cout<< " #getConnection, dsOD3(ds5) =  " << dsOD3 <<endl;
    dsOD4 = dsrOD->getConnection(userid, pw);  // new ds6
    cout<<" #getConnection, dsOD4(ds6) =  " << dsOD4 <<endl;


    dsIdOD3 = dsrOD->getConnection(userid, pw, 9); // dsid2
    cout<<" #getConnection, dsIdOD3(dsid2)=" <<
     dsIdOD3->getDatastore() << ", sessionId=" << dsIdOD3->getId()<<endl;


    /**/
    //**** use dsOD2 to do ListAppGrps
    cout<<"||**** start: use dsOD2 to do ListAppGrps ****||"<<endl;

    ListAppGrps(*(DKDatastoreOD*)dsOD2);

    cout<<"||**** end: use dsOD2 to do ListAppGrps ****||"<<endl;
    //****end of use dsOD2 to do ListAppGrps
    /**/
    dsrOD->returnConnection(dsOD2);
    cout<<" #returnConnection, dsOD2(ds4) =  "<< dsOD2 <<endl;
    dsOD4 = dsrOD->getConnection(userid, pw);
    cout<<" #getConnection, dsOD4 =  "<< dsOD4 <<endl;
    cout<<"    dsOD4 == dsOD2 =  " <<(dsOD4 == dsOD2)<<endl;

    //**** you can add codes to use dsOD4 to do other thing here
    //**** end of use dsOD4 to do other thing

    dsrOD->returnConnection(dsOD1);
    cout<<" #returnConnection, dsOD1(ds3) =  " << dsOD1 <<endl;
    dsrOD->returnConnection(dsOD3);
    cout<<" #returnConnection, dsOD3(ds5) =  " << dsOD3 <<endl;
    dsrOD->returnConnection(dsIdOD4); //dsid4 in _freeIdPool
    cout<<" #returnConnection, dsIdOD4(dsid4) =  " <<
     dsIdOD4->getDatastore() << ", sessionId=" << dsIdOD4->getId() <<endl;


    cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
    Sleep(120050);
#elif AIX
    sleep(120050);
#endif
    cout<<"**** end sleep for 120050 mini sec*******"<<endl;


    dsOD2 = (DKDatastoreOD*)dsrOD->getConnection(userid, pw);  //new ds7
    cout<<" #getConnection, dsOD2(ds7) =  " << dsOD2<<endl;
    cout<<" #call delete dsrOD..." <<endl;
    delete dsrOD;  //take a while for all 4 in usedPool to be disconnected
  }
  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }

    cout << "Exception Class Name: " << exc.name() << endl;
  }

  cout << "done ..." << endl;
}

void ListAppGrps(DKDatastoreOD& dsOD)
{
  DKString strAppGrp;
  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;
  DKSequentialCollection * pCol2 = NULL;
  dkIterator * pIter2 = NULL;
  DKSequentialCollection * pCol3 = NULL;
  dkIterator * pIter3 = NULL;
  DKAppGrpDefOD * agDef = NULL;
  DKFieldDefOD * attrDef = NULL;
  DKBoolean showAttributes = FALSE;

  cout << "\nshow app grps..." << endl;
  pCol = (DKSequentialCollection*)dsOD.listEntities();  // get app grps
  pIter = pCol->createIterator();
  int i = 0;

  while (pIter->more() == TRUE)
  {
    i++;
    agDef = (DKAppGrpDefOD*)((void*)(*pIter->next()));
    strAppGrp = agDef->getName();
    cout << "  app grp name[" << i << "]: ==>" << strAppGrp << "<==" << endl;
    cout << "  show attributes for " << strAppGrp << " app grp - " << endl;

    if (showAttributes == TRUE)
    {
      // get attributes
      pCol2 = (DKSequentialCollection*)dsOD.listEntityAttrs(strAppGrp);
      pIter2 = pCol2->createIterator();
      int j = 0;

      while (pIter2->more() == TRUE)
      {
        j++;
        attrDef = (DKFieldDefOD*)(void*)(*pIter2->next());
        cout << "    Attribute name[" << j << "]: ==>" << attrDef->getName() << "<==" << endl;
        cout << "      datastoreType: " << attrDef->datastoreType() << endl;
        cout << "      attributeOf: " << attrDef->getEntityName() << endl;
        cout << "      type: " << attrDef->getType() << endl;
        cout << "      size: " << attrDef->getSize() << endl;
        cout << "      id: " << attrDef->getId() << endl;
        cout << "      precision: " << attrDef->getPrecision() << endl;
        cout << "      scale: " << attrDef->getScale() << endl;
        cout << "      stringType: " << attrDef->getStringType() << endl;
        cout << "      NULLable: " << attrDef->isNullable() << endl;
        cout << "      Queryable: " << attrDef->isQueryable() << endl;
        cout << "      Updatable: " << attrDef->isUpdatable() << endl;
        // cleanup
        delete attrDef;
      }
      cout << "  " << j << " attribute(s) listed for ==>" << strAppGrp << "<==" << " app grp\n" << endl;
      // cleanup
      if ( pIter2 )
        delete pIter2;
      if ( pCol2 )
        delete pCol2;

      // get attributes (method #2)
      pCol3 = (DKSequentialCollection*)agDef->listAttrs();
      pIter3 = pCol3->createIterator();
      int k = 0;

      while (pIter3->more() == TRUE)
      {
        k++;
        attrDef = (DKFieldDefOD*)(void*)(*pIter3->next());
        cout << "#2: Attribute name[" << k << "]: ==>" << attrDef->getName() << "<==" << endl;
        cout << "#2:   datastoreType: " << attrDef->datastoreType() << endl;
        cout << "#2:   attributeOf: " << attrDef->getEntityName() << endl;
        cout << "#2:   type: " << attrDef->getType() << endl;
        cout << "#2:   size: " << attrDef->getSize() << endl;
        cout << "#2:   id: " << attrDef->getId() << endl;
        cout << "#2:   precision: " << attrDef->getPrecision() << endl;
        cout << "#2:   scale: " << attrDef->getScale() << endl;
        cout << "#2:   stringType: " << attrDef->getStringType() << endl;
        cout << "#2:   NULLable: " << attrDef->isNullable() << endl;
        cout << "#2:   Queryable: " << attrDef->isQueryable() << endl;
        cout << "#2:   Updatable: " << attrDef->isUpdatable() << endl;
        // cleanup
        delete attrDef;
      }
      cout << "#2:" << k << " attribute(s) listed for ==>" << agDef->getName() << "<==" << " app grp\n" << endl;
    }
    // cleanup
    if ( pIter3 )
      delete pIter3;
    if ( pCol3 )
      delete pCol3;
    if (agDef)
      delete agDef;
  }
  cout << " " << i << " app group(s) listed" << endl;
  // cleanup
  if (pCol)
    delete pCol;
  if ( pIter )
    delete pIter;
}

