/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolODBC.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKClobODBC.hpp>
#include <DKDatastoreODBC.hpp>
#include <DKDatastorePool.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKParametricQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKEnvironment.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
   DKString schema = ""; 
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolODBC <database> <userid> <pw>" << endl;
     return;
    }
    else if (argc == 5)
    {
      schema = DKString(argv[4]);
    }
    // ### get input parameters
    DKString database = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryODBCLName = "cmbodbcfac816d";

   try {
      DKDatastorePool* dsrODBC = new DKDatastorePool("ODBC", factoryODBCLName);
      dsrODBC->setDatastoreName(database);
      dsrODBC->setMaxPoolSize(10);
      dsrODBC->setMinPoolSize(3);
      dsrODBC->setTimeOut(1); 

      dsrODBC->initConnections(userid, pw, 2);
      //dsrODBC->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrODBC->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrODBC->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrODBC->getTimeOut()<<endl;

      dkDatastore *dsODBC1, *dsODBC2, *dsODBC3, *dsODBC4;;
      dsODBC1 = dsrODBC->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsODBC1(ds1) =  " << dsODBC1 << endl;
      dsODBC2 = dsrODBC->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsODBC2(ds2) =  " << dsODBC2 << endl;

      DKDatastoreSession *dsIdODBC1, *dsIdODBC2, *dsIdODBC3, *dsIdODBC4;
      dsIdODBC1 = dsrODBC->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdODBC1(dsid1)="<< dsIdODBC1->getDatastore()
        <<", sessionId=" << dsIdODBC1->getId()<<endl;
      dsIdODBC2 = dsrODBC->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdODBC2(dsid2)="<<dsIdODBC2->getDatastore()
        <<", sessionId=" << dsIdODBC2->getId()<<endl;

      dsrODBC->returnConnection(dsIdODBC2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdODBC2(dsid2) =  " << 
        dsIdODBC2->getDatastore() << ", sessionId=" << dsIdODBC2->getId()<<endl;
      dsrODBC->returnConnection(dsODBC2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsODBC2(ds2) =  " << dsODBC2 <<endl; 

      dsIdODBC3 = dsrODBC->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdODBC3(ds2)=" <<
        dsIdODBC3->getDatastore() << ", sessionId=" << dsIdODBC3->getId()<<endl;
      dsIdODBC4 = dsrODBC->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdODBC4(dsid3)=" <<
        dsIdODBC4->getDatastore() << ", sessionId=" << dsIdODBC4->getId() <<endl;


      dsIdODBC4 = dsrODBC->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdODBC4(dsid4)=" <<
        dsIdODBC4->getDatastore() << ", sessionId=" << dsIdODBC4->getId()<<endl;

      dsODBC1 = dsrODBC->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsODBC1(ds3) =  " << dsODBC1 <<endl;
      dsODBC2 = dsrODBC->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsODBC2(ds4) =  " << dsODBC2 <<endl;
 
      dsODBC3 = dsrODBC->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsODBC3(ds5) =  " << dsODBC3 <<endl;
      dsODBC4 = dsrODBC->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsODBC4(ds6) =  " << dsODBC4 <<endl;


      dsIdODBC3 = dsrODBC->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdODBC3(dsid2)=" <<
      dsIdODBC3->getDatastore() << ", sessionId=" << dsIdODBC3->getId()<<endl;


/**/
      //**** use dsODBC2 to do query
      cout<<"||**** start: use dsODBC2 to do query ****||"<<endl;
         dkQuery* pQry;
         DKAny any;
         DKString cmd; 
         DKResults* pResults;
         DKNVPair par[2];
         DKString strMax = "5";
         any = strMax;
         par[0].set(DK_CM_PARM_MAX_RESULTS, any);
         par[1].setName(DK_CM_PARM_END);

         if (schema == "") 
          cmd = "SELECT * FROM EMPLOYEE";
         else
          cmd = "SELECT * FROM " + schema + ".EMPLOYEE";  
    
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsODBC2->createQuery(cmd,DK_CM_SQL_QL_TYPE);
         cout << "executing query" << endl;
         pQry->execute(par);
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result();
         pResults = (DKResults*)((dkCollection*) any);

         processResults(pResults);

         dsODBC2->commit();
         delete pQry;
         delete pResults;
      cout<<"||**** end: use dsODBC2 to do query ****||"<<endl;
      //****end of use dsODBC2 to do query 

/**/
      dsrODBC->returnConnection(dsODBC2);
      cout<<" #returnConnection, dsODBC2(ds4) =  "<< dsODBC2 <<endl;
      dsODBC4 = dsrODBC->getConnection(userid, pw);
      cout<<" #getConnection, dsODBC4 =  "<< dsODBC4 <<endl;
      cout<<"    dsODBC4 == dsODBC2 =  " <<(dsODBC4 == dsODBC2)<<endl;
/**
      //**** use dsODBC4 to retrieve XDO  
      cout<<"||**** start: use dsODBC4 to retrieve XDO ****||"<<endl;
       DKString table;                           //from SAMPLE database of ODBC
       DKString column = "RESUME";               //blob column 
       DKString predicate = "EMPNO = '000130' and RESUME_FORMAT = 'ascii'" ; //primary key of exist clob
       if (schema == "") 
         table = "EMP_RESUME"; 
       else
         table = schema + ".EMP_RESUME"; 
       DKClobODBC* axdo = new DKClobODBC((DKDatastoreODBC*)dsODBC4);
       DKPidXDOODBC*  apid = new DKPidXDOODBC;
       apid->setTable(table);
       apid->setColumn(column);
       apid->setDataPredicate(predicate);   //exist blob
       axdo->setPidObject(apid);
       cout<<"table= "<<axdo->getTable()<<endl;
       cout<<"column="<<((DKPidXDOODBC*)(axdo->getPid()))->getColumn()<<endl;
       cout<<"dataPredicate="<<((DKPidXDOODBC*)(axdo->getPid()))->getDataPredicate()<<endl;
       cout<<"about to do retrieve()..."<<endl;
       axdo->retrieve("connV4.ascii");
       cout<<"retrieve successfully..."<<endl;
       cout<<"about to do open()"<<endl;
       axdo->setInstanceOpenHandler("notepad ", TRUE); //use notepad  in NT
       axdo->open(); 
       dsODBC4->commit();
       delete axdo;
       delete apid;   
      cout<<"||**** end: use dsODBC4 to retrieve XDO ****||"<<endl;
      //****end of use dsODBC4 to retrieve XDO
**/
      dsrODBC->returnConnection(dsODBC1);
      cout<<" #returnConnection, dsODBC1(ds3) =  " << dsODBC1 <<endl;
      dsrODBC->returnConnection(dsODBC3);
      cout<<" #returnConnection, dsODBC3(ds5) =  " << dsODBC3 <<endl;
      dsrODBC->returnConnection(dsIdODBC4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdODBC4(dsid4) =  " <<
        dsIdODBC4->getDatastore() << ", sessionId=" << dsIdODBC4->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsODBC2 = (DKDatastoreODBC*)dsrODBC->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsODBC2(ds7) =  " << dsODBC2<<endl;
      cout<<" #call delete dsrODBC..." <<endl;
      delete dsrODBC;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
//      item->retrieve();  //Only valid if Primary Key exists for Table
      itemType = item->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_CM_DOCUMENT:
      {
             cout << " Item is a document, "
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 }
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }
   }
   delete iter;
}
