/**************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/**************************************************************************/

// sample parametric query for VI400 datastore
/*
 * Filename       : TSamplePQryV4.cpp
 * Description    : Connect to VI400 Datastore
 *                  Create query string
 *                  specify to retrieve folder and document only.
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from VI400 Datastore
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreV4.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKParametricQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKEnvironment.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
   
   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 3)
   {
     cout << "Invoke Syntax: " << endl;
     cout << "  SamplePQryV4 <Library Server> <userid> <password>" << endl;
     return;
   }
   
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKDatastoreV4 dsV4;
    
    try 
    {
         dkQuery* pQry;
         DKAny any;
         DKResults* pResults;

         cout << "connecting to datastore..." << endl;
         dsV4.connect(libsrv,userid,pw);
         cout << "datastore connected, libsrv: " << libsrv 
              << ", userid: " << userid << endl;
         DKString cmd = "SEARCH=(INDEX_CLASS=DATATYPE,";
         cmd += "MAX_RESULTS=15,";
         cmd += "COND=(Data_FString <> NULL));";
         cmd += "OPTION=(CONTENT=YES;";
         cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";

         cout << "query string: " << cmd << endl;
         cout << "creating query..." << endl;
         pQry = dsV4.createQuery(cmd);
         cout << "executing query..." << endl;
         pQry->execute();
         cout << "query executed" << endl;
         cout << "getting query results..." << endl;
         any = pQry->result(); 
         pResults = (DKResults*)((dkCollection*) any);
         
         processResults(pResults);

         dsV4.disconnect();
         cout << "datastore disconnected" << endl;
         delete pQry;
         delete pResults;
    }
    catch(DKException &exc)
    {
         cout << "Error id:             " << exc.errorId() << endl;
         for(unsigned long i = 0; i < exc.textCount(); i++) 
         {
             cout << "Error text:       " << exc.text(i) << endl;
         }
         
         for (unsigned long j=0; j < exc.locationCount(); j++) 
         {
              const DKExceptionLocation* p = exc.locationAtIndex(j);
              cout << "Filename:        "   << p->fileName()     << endl;
              cout << "Function:        "   << p->functionName() << endl;
              cout << "LineNumber:      " << p->lineNumber()   << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
    }
    
    cout << "done!" << endl;
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;
   DKParts* parts;
   DKFolder* folder;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   cout << "  number of items = " << results->cardinality() 
        << ",  showing as follows ..." << endl;
        
   while(iter->more()) 
   {
      element = iter->next();
      item = (DKDDO*) element->value();
      
      if (!item) 
        continue;
      
      item->retrieve();
      itemType = item->getPropertyByName(DK_PROPERTY_ITEM_TYPE);
      
      switch(itemType) 
      {
        case DK_DOCUMENT:
             cout << "    Item is a document, " 
                  << "number of data item(s): " << item->dataCount() << endl;
                  
             for (i = 1; i <= item->dataCount(); i++) 
             {
                 dataName = item->getDataName(i);
                 cout << "      data-item[" << i << "] = "
                      << dataName  << ", value = "
                      << item->getData(i) << endl;
                      
                 if ((dataName != (DKString) 0) && dataName == DKPARTS) 
                 {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      
                      if (parts != NULL) 
                      {
                         cout << "\t-- this document has " << 
                              parts->cardinality()  << " part(s) " << endl;
                      }
                 }   
             }
             break;
        case DK_FOLDER:
             cout << "    Item is a folder, number of data item(s): " 
                  << item->dataCount() << endl;
             folder = 0;
             
             for (i = 1; i <= item->dataCount(); i++) 
             {
                 dataName = item->getDataName(i);
                 cout << "      data-item[" << i << "] = "
                      << dataName  << ", value = "
                      << item->getData(i) << endl;
                      
                 if ((dataName != (DKString) 0) && dataName == DKPARTS) 
                 {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      
                      if (parts != NULL) 
                      {
                          cout << "\t-- this folder has " << 
                                 parts->cardinality()  << " document(s) " << endl;
                          // iterate here to get parts
                      }
                 }   
                 
                 if ((dataName != (DKString) 0) && dataName == DKFOLDER) 
                 {
                     any    = item->getData(i);
                     folder = (DKFolder*) any.value();
                     
                     if (folder != NULL) 
                     {
                        cout << "\t-- this folder has "  << 
                                 folder->cardinality() << 
                                " document(s) " << endl;
                     }
                 }   
             }
             
             if (folder != NULL && folder && folder->cardinality() > 0) 
             { 
                 cout << "     ---------- list of folder contents -----------" << endl;
                 processResults(folder); 
                 cout << "     ----------------- end of list ----------------\n" << endl;
             }
             else cout << "\t-- empty folder " << endl;
             break;
        default:
             cout << "   Item is not recognized " << endl;
      } // switch    
   }  // while
   
   delete iter;
}
