/*
 * @copyright(disclaimer) 
 * Licensed Materials - Property of IBM
 * IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
 * 
 * US Government Users Restricted Rights                                  
 *  Use, duplication or disclosure restricted by GSA ADP Schedule          
 *  Contract with IBM Corp.     
 * 
 * DISCLAIMER OF WARRANTIES :                                             
 *                                                                        
 * Permission is granted to copy and modify this  Sample code, and to           
 * distribute modified versions provided that both the copyright        
 * notice,- and this permission notice and warranty disclaimer  appear in all copies and modified  
 * versions. 
 * 
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. 
 * IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM     
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN 
 * SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF 
 * NON-INFRINGEMENT AND THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 * PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
 * SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT 
 * OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, 
 * DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION 
 * OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO 
 * EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE 
 * LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, 
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED 
 * AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF 
 * IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGES.                                     
 *                                                                  
 * @endCopyright
 */
import java.util.*;
import java.awt.Image;
import java.text.MessageFormat;

/**
 * Determines Locale, loads resource bundles, and retrieves strings
 * from the bundles.  Classes desiring translatable resources should
 * call the static <code>getXXX</code> methods in this class.
 * Those getXXXX methods will load the bundles the first time one of the get
 * methods is called.
 * <P>
 * getString will retrieve String resources from MSGResources (if ID is
 * of form CMBnnnnx) or TextResources.
 * There are versions of getString which will retrieve a String and then format it.
 * <P>
 * getImageURL returns URLs for images where the image names are in OtherResources.
 * <P>
 * See the prologs in the default (e.g., MSGResources.java, not MSGResource_en_US)
 * versions of those three resource classes for naming conventions for IDs.
 *
 */
public final class ResourceMgr {

   // Do not remove - needed for copyright protection
   private static String COPYRIGHT = ClientPane.IBM_LONG_COPYRIGHT;


   private static Locale currentLocale = Locale.getDefault();
   private static ResourceBundle textResources = null;
   private static ResourceBundle otherResources = null;
   //private static ResourceBundle MSGResources = null;
   private static ResourceBundle propertiesResources = null;


   /**
    * Note that declaring "throws ..." on this method 
    * doesn't force the caller to catch it (compiler doesn't enforce that
    * RuntimeExceptions are caught).  So, the caller would have to put the
    * call to this method w/i a try block.
    * <P>
    * This method cannot issue System.exit in the catch block because
    * all browser frames would be killed if running an applet.
    */
   private static void initialize() throws MissingResourceException {
      //Load ResourceBundle for this system's locale.
      textResources = loadBundle("CMBCATextResources");
      otherResources= loadBundle("CAOtherResources");
   }

   private static ResourceBundle loadBundle(String baseName) {
      try {
         return ResourceBundle.getBundle(baseName, currentLocale);
      }
      catch(java.util.MissingResourceException e) {
         Object[] args = {baseName, currentLocale};
         System.out.println(MessageFormat.format(
            "No ResourceBundle was found for bundle {0} with locale {1}.", args));
         Object[] args2 = {baseName};
         System.out.println(MessageFormat.format(
            "Even the default bundle ({0}) was not found.", args2));
         handleException(e);
         return null;   // to satisfy compiler
      }
   }

   /**
    * Fetches locale-specific String from a ResourceBundle.
    * <P>
    * @param ID key of resource in bundle.  
    * The resource will be retrieved from CMBCATextResources.
    * @return translated String
    */
   public static String getString(String ID) throws MissingResourceException {
      if (textResources == null) {
         initialize();
      }
      ResourceBundle b = textResources;

      try {
         String str=b.getString(ID);
         return str;
      }
      catch(java.util.MissingResourceException e) {
         Object[] args = {ID, b.getClass().getName()};
         handleException(e, ResourceMgr.getString("RESOURCE_ID_NOT_FOUND", args));
         return null;
      }
   }

   /**
    * Fetches and formats a locale-specific String from a ResourceBundle.
    * Same as <code>getString(String ID)</code> except the returned string
    * is formatted, i.e., args are substituted.
    * <P>
    * @param ID See <code>getString(String ID)</code>'s ID parameter
    * @param array of Objects to be substituted for {0}, {1}, etc.
    * @return translated AND formatted String
    * <P>Example:
    * <PRE>
    * If the ID is "twoCats",
    *    and <code>args</code> is an array containing the Strings "Snowflake" and "Tabby",
    *    and the ResourceBundle contains:
    *       {"twoCats", "{0} and {1} are my favorite pets"}
    * then, this method returns "Snowflake and Tabby are my favorite pets".
    * </PRE>
    */
   public static String getString(String ID, Object[] args) throws MissingResourceException {
      return MessageFormat.format(getString(ID), args);
   }

   /**
    * Fetches and formats a locale-specific String from a ResourceBundle.
    * Same as <code>getString(String ID)</code> except the returned string
    * is formatted with one argument.
    * <P>
    * @param ID See <code>getString(String ID)</code>'s ID parameter
    * @param Object to be substituted for {0}
    * @return translated AND formatted String
    * <P>Example:
    * <PRE>
    * If the ID is "ItemCount",
    *    and <code>arg</code> is an Integer object whose value is 19,
    *    and the ResourceBundle contains:
    *       {"ItemCount", "The number of items found is {0}."}
    * then, this method returns "The number of items found is 19.".
    * </PRE>
    */
   public static String getString(String ID, Object arg) throws MissingResourceException {
      Object[] args = {arg};
      return MessageFormat.format(getString(ID), args);
   }


   /**
    * Fetches locale-specific String from a ResourceBundle, and prefixes the
    * returned string with the message number (the ID).
    * <P>
    * @param ID See <code>getString(String ID)</code>'s ID parameter
    * @return translated String prefixed with ID.
    * <P>Example:
    * <PRE>
    * If the ID is "CMB0413A",
    *    and the ResourceBundle contains:
    *       {"CMB013A", "Logon Failed!"},
    * then, this method returns "CMB4013A:  Logon Failed!".
    * </PRE>
    */
   public static String getStringWithMessageNumber(String ID) throws MissingResourceException {
      return ID + ":  " + getString(ID);
   }


   /**
    * Fetches and formats a locale-specific String from a ResourceBundle
    * and prefixes the returned string with the message number (the ID).
    * Same as <code>getString(String ID)</code> except the returned string
    * is formatted with one argument.
    * <P>
    * @param ID See <code>getString(String ID)</code>'s ID parameter
    * @param Object to be substituted for {0}
    * @return translated AND formatted String
    * <P>Example:
    * <PRE>
    * If the ID is "ItemCount",
    *    and <code>arg</code> is an Integer object whose value is 19,
    *    and the ResourceBundle contains:
    *       {"ItemCount", "The number of items found is {0}."}
    * then, this method returns "ItemCount: The number of items found is 19.".
    * </PRE>
    */
   public static String getStringWithMessageNumber(String ID, Object arg) throws MissingResourceException {
      return ID + ":  " + getString(ID, arg);
   }


   /**
    * Fetches and formats a locale-specific String from a ResourceBundle, and
    * prefixes the returned string with the message number (the ID).
    * Same as <code>getStringWithMessageNumber(String ID)</code> except the
    * returned string is formatted, i.e., args are substituted.
    * <P>
    * @param ID See <code>getString(String ID)</code>'s ID parameter
    * @param array of Objects to be substituted for {0}, {1}, etc.
    * @return translated String prefixed with ID.
    * <P>Example:
    * <PRE>
    * If the ID is "twoCats",
    *    and <code>args</code> is an array containing the Strings "Snowflake" and "Tabby",
    *    and the ResourceBundle contains:
    *       {"twoCats", "{0} and {1} are my favorite pets"}
    * then, this method returns "twoCats: Snowflake and Tabby are my favorite pets".
    * </PRE>
    */
   public static String getStringWithMessageNumber(String ID, Object[] args)
          throws MissingResourceException {
      return ID + ":  " + getString(ID, args);
   }


   /**
    * Fetches the filename of an image from OtherResources and
    * builds a URL object representing that image.
    * <P>
    * @param ID String identifying the image name
    * @return java.net.URL representing the image
    */
   public static java.net.URL getImageURL(String ID) throws MissingResourceException {
      if (otherResources == null) {
         initialize();
      }
      try {
         java.net.URL url = otherResources.getClass().getResource(otherResources.getString(ID));
         if (url == null) {
            return null;
         }
         else {
            return otherResources.getClass().getResource(otherResources.getString(ID));
         }
      }
      catch(java.util.MissingResourceException e) {
         Object[] args = {ID, otherResources.getClass().getName()};
         handleException(e, ResourceMgr.getString("RESOURCE_ID_NOT_FOUND", args));
         return null;
      }
   }
   /**
    * Fetches an Object from OtherResources
    * <P>
    * @param ID String identifying the resource
    * @return Object associated with ID
    */
   public static Object getOther(String ID) throws MissingResourceException {
      if (otherResources == null) {
         initialize();
      }
      try {
         return otherResources.getObject(ID);
      }
      catch(java.util.MissingResourceException e) {
         Object[] args = {ID, otherResources.getClass().getName()};
         handleException(e, ResourceMgr.getString("RESOURCE_ID_NOT_FOUND", args));
         return null;
      }
   }
   
    /** Load an image from package */
    public static Image getImage(String imageName) {
    	try {
	        java.net.URL url = ResourceMgr.class.getResource(imageName);
	        if (url == null)
	            return null;
	        java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
	        return tk.createImage((java.awt.image.ImageProducer) url.getContent());
	    } catch (Exception ex) { // occurs when image doesn't exist
	        return null;
	    }
    }
    /** Load an image from url */
    public static Image getImage(java.net.URL url) {
    	try {
	        if (url == null)
	            return null;
	        java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
	        Image i = tk.createImage((java.awt.image.ImageProducer) url.getContent());
	        return i;
	    } catch (Exception ex) { // occurs when image doesn't exist
	        return null;
	    }
    }
    /** Load a resource*/
    private static java.net.URL getResource(String imageName) {
    	try {
	        java.net.URL url = ResourceMgr.class.getResource(imageName);
	        return url;
	    } catch (Exception ex) { // occurs when image doesn't exist
	        return null;
	    }
    }
    
    public static  String safeGetString(String key) {
        String str = key;
        try {
            str = ResourceMgr.getString(key);
        }
        catch (java.util.MissingResourceException e) {
            // return key by default
        }
        return str;
    }
   

   private static void handleException(MissingResourceException e) throws MissingResourceException {
      throw e;
   }
   private static void handleException(MissingResourceException e, String msg) throws MissingResourceException {
      throw e;
   }

}
