/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample to create Federated search template and parametric search criteria
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

// TCreateSTParmCritFed class
/**
 * <pre>
 * Filename       : TCreateSTParmCritFed.java
 * Description    : test the following classes and funtions in each:
 *		     	- DKDatastoreFed:
 *                          + connect
 *                          + disconnect
 *                      - DKSearchTemplateFed:
 *                          + add()
 *                      - DKSearchCriteria:
 *                          + addCriteria
 *                          + setName
 *                          + setDisplayName
 *                          + setDefaultOperator
 *                          + setValidOperators
 *                          + setCriteriaPosition
 *                          + setDisplayPosition
 *                          + setQueryable
 *                          + setDisplayable
 *                          + setDisplayWidth
 *                          + setDefaultValues
 *                          + addPredefinedValues
 *                      - DKPredefinedValues:
 *                          + addValuePair
 * </pre>
 */

public class TCreateSTParmCritFed implements DKConstantFed {

  // Main method
  public static void main(String argv[]){

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
	DKDatastoreFed dsFed = new DKDatastoreFed();
	DKDatastoreDefFed dsDef = (DKDatastoreDefFed)dsFed.datastoreDef();

	String userName;
    	String dsName;

    	System.out.println("connecting to datastore");
	dsFed.connect("cmbdb","ccadmin","password","");
	System.out.println("datastore connected");

	userName = dsFed.userName();
	dsName = dsFed.datastoreName();

	System.out.println("user " + userName + " dsName " + dsName);

	DKSearchTemplateFed stObj = new DKSearchTemplateFed(dsFed);
	stObj.setName("Green Driver1 Demo with Parametric search A");
	stObj.setDescription("Green 1st driver demo template for parametric created by meis test case");
        stObj.setFolderName("MeiFolder1");
        stObj.setParametricOp(DKOpCode.OR);
	stObj.add();	//adding it to the federated database
        System.out.println("DKSearchTemplate add() success!!");

	DKSearchCriteria crit = new DKSearchCriteria("FedDemo", "FedTitle");
	short[] ops1 = { DKOpCode.EQ,
			 DKOpCode.NEQ,
			 DKOpCode.LIKE,
			 DKOpCode.NOTLIKE,
			 DKOpCode.IN,
			 DKOpCode.NOTIN }; 

	crit.setName("Abstract");
	crit.setDisplayName("Document Title");
	crit.setDefaultOperator(DKOpCode.LIKE);
	crit.setValidOperators(ops1);
	crit.setCriteriaPosition((short)1);
	crit.setDisplayPosition((short)2);
	crit.setDisplayWidth((short)40);
	crit.setQueryable(true);
	crit.setDisplayable(true);
	stObj.add(crit);
        System.out.println("DKSearchCriteria Title add() success!!");

	
        crit = new DKSearchCriteria("FedDemo", "FedAuthor");
	short[] ops = { DKOpCode.EQ,
			DKOpCode.NEQ,
			DKOpCode.LIKE,
			DKOpCode.NOTLIKE,
			DKOpCode.IN,
			DKOpCode.NOTIN };
	crit.setName("Author");
	crit.setDisplayName("Document Author");
	crit.setDefaultOperator(DKOpCode.LIKE);
	crit.setValidOperators(ops);
	crit.setCriteriaPosition((short)2);
	crit.setDisplayPosition((short)1);
	crit.setDisplayWidth((short)40);
	crit.setQueryable(true);
	crit.setDisplayable(true);
        String[] defaultvalues = {"Mei Huang"};
        //String[] defaultvalues = {"Mei Huang", "Mimi Vo", "John Xiao"}; //for multiple values
        crit.setDefaultValues(defaultvalues);

        //set predefined values
        DKPredefinedValues pValues = new  DKPredefinedValues("DLSAMPLE", "DLSEARCH_Author", "LSMUFASA");
        pValues.addValuePair("Kehsing Chou", "K. Chou");        
        pValues.addValuePair("An Feng-I Chen", "A. Chen");
        pValues.addValuePair("Jason Wu", "J. Wu");
        pValues.addValuePair("Taejae Lee ", "T. Lee");
        pValues.addValuePair("Mei-Ing Huang", "M. Huang");
        pValues.addValuePair("Robert Summers", "R. Summers");
        pValues.addValuePair("Basuki Soetarman", "B. Soetarman");
        crit.addPredefinedValues(pValues); 
       
	stObj.add(crit);
        System.out.println("DKSearchCriteria Author add() success!!");

 /**
	crit = new DKSearchCriteria("FedDemo", "FedDate");
	short[] ops2 = {DKOpCode.EQ,
			 DKOpCode.NEQ,
			 DKOpCode.BETWEEN,
			 DKOpCode.NOTBETWEEN,
			 DKOpCode.IN,
			 DKOpCode.NOTIN,
			 DKOpCode.LT,
			 DKOpCode.LEQ,
			 DKOpCode.GT,
			 DKOpCode.GEQ };

	crit.setName("Date");
	crit.setDisplayName("Date Published");
	crit.setDefaultOperator(DKOpCode.EQ);
	crit.setValidOperators(ops2);
	crit.setCriteriaPosition((short)3);
	crit.setDisplayPosition((short)3);
	crit.setDisplayWidth((short)20);
	crit.setQueryable(true);
	crit.setDisplayable(true);
        String[] values = {"Monday", "Wednesday", "Friday"};
        crit.setDefaultValues(values);
 **/    
 /**       
        //set predefined values
        DKPredefinedValues pValues = new  DKPredefinedValues(...);
        pValues.addValuePair("Sunday", "Sun");
        pValues.addValuePair("Monday", "Mon");
        pValues.addValuePair("Tuesday", "Tue");
        pValues.addValuePair("Wednesday", "Wed");
        pValues.addValuePair("Thursday", "Thu");
        pValues.addValuePair("Friday", "Fri");
        pValues.addValuePair("Saturday", "Sat");
        crit.addPredefinedValues(pValues);
 **/
 /** 
	stObj.del(crit);
        System.out.println("DKSearchCriteria Date_Published add() success!!");
 **/         
        //System.out.println("DKSearchTemplate add() success!!");

	dsFed.disconnect();
	System.out.println("datastore disconnected");
	userName = dsFed.userName();
	dsName = dsFed.datastoreName();
	System.out.println("user " + userName + " dsName " + dsName);
    }
    catch (DKException exc)
    {
	System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }

  }
}

