/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample connect/disconnect to Federated datastore & get search template/criteria
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

// TGetSTCriteriaFed class
/**
 * <pre>
 * Filename       : TGetSTCriteriaFed.java
 * Description    : test the following classes and funtions in each:
 *		     	- DKDatastoreFed:
 *                          + connect
 *                          + disconnect
 *                          + listSearchTemplateNames
 *                      - DKDatastoreDefFed:
 *                          + getSearchTemplate
 *                      - DKSearchTemplateFed:
 *                          + getDescription
 *                          + getId
 *                          + listCriteriaNames
 *                          + getCriteria
 *                      - DKSearchCriteria:
 *                          + getDisplayName
 *                          + getDefaultOperator
 *                          + getCriteriaPosition
 *                          + getDisplayWidth
 *
 * </pre>
 */

public class TGetSTCriteriaFed implements DKConstantFed {

  // Main method
  public static void main(String argv[])
  {
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
    	DKDatastoreFed dsFed = new DKDatastoreFed();
	DKDatastoreDefFed dsDef = (DKDatastoreDefFed)dsFed.datastoreDef();

    	String userName;
    	String dsName;

    	System.out.println("connecting to datastore");

	dsFed.connect("cmbdb","ccadmin","password","");

    	System.out.println("datastore connected");

	userName = dsFed.userName();
    	dsName = dsFed.datastoreName();

    	System.out.println("user " + userName + " dsName " + dsName);

	String[] names = dsFed.listSearchTemplateNames();
    	if (names != null) 
        {
	    for (int i=0; i<names.length; i++) 
            {
	    	System.out.println("name[" + i + "] = " + names[i] );
    		DKSearchTemplateFed stObj = (DKSearchTemplateFed)dsDef.getSearchTemplate(names[i]);
		System.out.println("description:  " + stObj.getDescription());
		System.out.println("id :          " + stObj.getId());
	    	String[] critNames = stObj.listCriteriaNames();
    		if (critNames != null) 
                {
		  System.out.println("critNames is not null");
    	    	  for (int j=0; j<critNames.length; j++) 
                  {
        	    System.out.println("CritName[" + j + "] = " + critNames[j]);
    	    	    DKSearchCriteria crit = stObj.retrieve(critNames[j]);
		    System.out.println("\t\tName        :  " + critNames[j]);
		    System.out.println("\t\tDispaly Name:  " + crit.getDisplayName());
		    System.out.println("\t\tDefault Op  :  " + crit.getDefaultOperator());
    	    	    System.out.println("\t\tQueryOrder  :  " + crit.getCriteriaPosition());
	            System.out.println("\t\tColumn Width:  " + crit.getDisplayWidth());
    		    String[] values = crit.getDefaultValues();
		    if (values != null) 
                    {
		       for (int k=0; k<values.length; k++) {
			  System.out.println("\t\tDefault val :  " + values[k]);
    	    	       }
	            }                                
    		    if (crit.isQueryable())
		    System.out.println("\t\t is queryable ....");
		    if (crit.isDisplayable())
			System.out.println("\t\t is displayable ....");
                    dkCollection predefV = crit.getPredefinedValues();
                    dkIterator iter = predefV.createIterator();
                    DKPredefinedValues pv = new DKPredefinedValues();
                    while (iter.more())
                    {
                      pv = (DKPredefinedValues)iter.next();
                      System.out.println("\t\tPredefinedValues: ");
                      System.out.println("\t\t\tEntityName:" +pv.getEntityName());
                      System.out.println("\t\t\tAttrName:" +pv.getAttrName());
                      System.out.println("\t\t\tServerName:" +pv.getServerName());
                      dkCollection vpairs = pv.getValuePairs();
                      dkIterator vpiter = vpairs.createIterator();
                      while (vpiter.more())
                      {
                        DKNVPair nvp = (DKNVPair)vpiter.next();
                        String  fedAttrValue = nvp.getName();
                        String  natAttrValue = (String)nvp.getValue();
                        System.out.println("\t\t\tfedAttr-natAttr:" + fedAttrValue + "-" + natAttrValue);
                      }                                 
                    } 
		  }
		}
	    }
    	 } 
         else {
		System.out.println("There is no search template defined in this db");
	 }
    	dsFed.disconnect();
	System.out.println("datastore disconnected");
    	userName = dsFed.userName();
	dsName = dsFed.datastoreName();
    	System.out.println("user " + userName + " dsName " + dsName);
    }
    catch (DKException exc)
    {
	System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
  }
}

