/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/* -------------------------------------------------------------------------
 * Class Name  : TQStringFed.java
 * Description : Test program to run ParametricQuery (PQ), TextQuery (TQ), 
 *               CombinedQueryWithOr (CO), combinedQueryWithAnd (CA) in a 
 *               loop for DKDatastoreFed.  This also logs Memory usage
 *               information to stdout and a logfile (MemTest.log)
 *               
 *               This program uses TMemUsage.c (TMemUsage.dll), 
 *               TMemUsage.java to get Virtual Memory size on NT.  To use 
 *               on AIX comment line that has getVMSize() 
 *
 *               java TQStringFed client PQTQCOCA 1000 cmbdb edms1 edms1
 *
 * Author      : Chandra S Potula
 * Create Date : 10/15/2000
 * 
 * Change History
 * -------------------------------------------------------------------------
 */

import com.ibm.mm.sdk.common.*;

import java.io.*;
import java.text.*;
import java.util.*;

public class TQStringFed implements DKConstant {
public TQStringFed(String aPackageName) {
	_startTime   = System.currentTimeMillis();
	_dateFormat  = new SimpleDateFormat("HH:mm:ss"); 
	_dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));

	try {
		if(     aPackageName.equalsIgnoreCase("cs")) {
			_datastore   = new com.ibm.mm.sdk.cs.DKDatastoreFed();
			_packageName = "cs";
		}
		else if(aPackageName.equalsIgnoreCase("client")) {
			_datastore   = new com.ibm.mm.sdk.client.DKDatastoreFed();
			_packageName = "client";
		}
		else {
			_datastore   = new com.ibm.mm.sdk.server.DKDatastoreFed();
			_packageName = "server";
		}

		logmsg("loaded : " + _datastore.getClass().getName());
		_logFile       = new PrintWriter(new FileWriter("MemTest.log", false), true);
	}
	catch(Exception e) {
		e.printStackTrace();
	}
}

// -------------------------------------------------------------------------
public static void logmsgdkResultSetCursor(dkResultSetCursor aRSCursor) {
	if(aRSCursor == null) {
		//logmsg("*** error *** got null cursor");
	}
	else {
		try {
			//logmsg("RSCCardinality : " + aRSCursor.cardinality());

			DKDDO lDKDDO = null;
			int nPosition = 0;
			while(aRSCursor.isValid()) {
				lDKDDO = aRSCursor.fetchNext();

				if(aRSCursor.isValid()) {
					nPosition = aRSCursor.getPosition();
				}

				if(lDKDDO != null) {
					int lDDOCount = lDKDDO.dataCount();
					//logmsg("    " + nPosition + ". DDOPid:" + lDKDDO.getPidObject().pidString() + "  Count:" + lDDOCount);
				}
			}
			aRSCursor.destroy();
		}
		catch (Exception exc) {
			exc.printStackTrace();
		}
	}
}

// -------------------------------------------------------------------------
public static void logmsg(String aMsg) {
	System.out.println(aMsg);
}

// -------------------------------------------------------------------------
public long executeQString(String aQString, long nInd, short nQLType) {
	long              lTimeTaken = 0;
	dkResultSetCursor lCursor    = null;

	try {
		long   lQStart;
		long   lQEnd;
		String lstrClock = "";
		String lstrTime;
		String lstrInd;

		//logmsg(aQString);
		lQStart    = System.currentTimeMillis();
		lCursor    = _datastore.execute(aQString, nQLType, null);
		lQEnd      = System.currentTimeMillis();
		lTimeTaken = lQEnd - lQStart;
		lstrTime   = "      " + lTimeTaken;
		logmsgdkResultSetCursor(lCursor);

		lstrClock  = _dateFormat.format(new Date(lQEnd - _startTime));
		lstrInd    = "     " + nInd + ".";

		lstrTime   = lstrTime.substring(lstrTime.length() - 6);
		lstrInd    = lstrInd.substring(lstrInd.length() - 6);


		logmsg(lstrInd + lstrClock + " Took:" + lstrTime + "ms  " + MemUsage.getMemUsage());
		//logmsg("---------------------------------------------------------------------------");
		_logFile.println(lstrInd + lstrClock + " " + lQStart + " Took:" + lstrTime + "ms " + MemUsage.getMemUsage());
		_logFile.flush();

		lCursor = null;

		// This will notify RMI server that the the cursor is dereferenced.
		System.gc();
		System.runFinalization();
		System.gc();
	}
	catch(Exception e) {
		lTimeTaken = -1;
		e.printStackTrace();
	}

	return lTimeTaken;
}

// -------------------------------------------------------------------------
public void startQueryLoop(String aQueryType, String nLoopTimes, 
		String aDBName, String aUserID, String aPassword) {
	try {

		try {
			// following lines are to trace Server activity if running 
			// Client/Server.
			// To do this the server shoud define a counter and increment
			// every time execute is called, also a boolean _printMemUsage
			// static variable and if it is set to true print Server
			// stats every time execute is called.
			com.ibm.mm.sdk.server.DKDatastoreFed._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreDL ._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreTS ._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreDES._printMemUsage = false;
		}
		catch(Exception e) {
		}

		boolean fParametric  = (aQueryType.indexOf("PQ") >= 0);
		boolean fText        = (aQueryType.indexOf("TQ") >= 0);
		boolean fCombinedOr  = (aQueryType.indexOf("CO") >= 0);
		boolean fCombinedAnd = (aQueryType.indexOf("CA") >= 0);

		if(!fParametric && !fText && !fCombinedOr && !fCombinedAnd) {
			logmsg("Invalid Query type specified...");
			printUsage();
			System.exit(0);
		}

		long nRunTimes  = toLong(nLoopTimes);
		if(nRunTimes <= 0) {
			nRunTimes  = 1;
		}

		// String queries are hard coded this uses schema mapping to the 
		// server LSIKE (DL) and TMHSIA (TS).  You may need to change these 
		// if you want to run this program against some other server.
		//
		// SchemaMapping to LSIKE, TMHSIA is as below...
		// Entity_DL1
		//     Entity_DL1;LSIKE;DLSAMPLE
		//         Entity_DL1.FedAttr_Name     -> LSIKE.DLSAMPLE.DLSEARCH_Author
		//         Entity_DL1.FedAttr_Publisher-> LSIKE.DLSAMPLE.DLSEARCH_Publisher
		//         Entity_DL1.FedAttr_Title    -> LSIKE.DLSAMPLE.DLSEARCH_Title
		// Fed_TMHSIA
		//     Fed_TMHSIA;TMHSIA;TMINDEX
		//         Fed_TMHSIA-Entity_DL1       -> TMHSIA-LSIKE.TMINDEX-DLSAMPLE
		String lstrPCond   = "FedAttr_Name <> 'Chandra'";
		String lstrTCond   = "'IBM' OR 'Tivoli'";
		String lstrPSQuery = "PARAMETRIC_SEARCH=(ENTITY=Entity_DL1, MAX_RESULTS=4, COND=(" + lstrPCond + "))";
		String lstrTSQuery = "TEXT_SEARCH=(COND=(" + lstrTCond + "));OPTION=(SEARCH_INDEX=(Fed_TMHSIA);MAX_RESULTS=4)";

		_datastore.connect(aDBName, aUserID, aPassword, "");

		for(long nInd=1; nInd <= nRunTimes;) {
			if(fParametric ) executeQString(lstrPSQuery,                         nInd++, DK_CM_FEDERATED_QL_TYPE);
			if(fText       ) executeQString(lstrTSQuery,                         nInd++, DK_CM_FEDERATED_QL_TYPE);
			if(fCombinedOr ) executeQString(lstrPSQuery + " OR  " + lstrTSQuery, nInd++, DK_CM_FEDERATED_QL_TYPE);
			if(fCombinedAnd) executeQString(lstrPSQuery + " AND " + lstrTSQuery, nInd++, DK_CM_FEDERATED_QL_TYPE);

			if(false) {
				break;
			}
		}

		// after executing specified number times give a chance to the user
		// to note down VM memory size
		logmsg("Ran execute for : " + nRunTimes + "\ntype QUIT to disconnect and quit");
		BufferedReader in = new BufferedReader( new InputStreamReader( System.in ) );
		String strIn = "";
		while(!strIn.equalsIgnoreCase("QUIT")) {
			strIn = in.readLine();
		}

		_datastore.disconnect();
	}
	catch(Exception exp) {
		exp.printStackTrace();
	}
}

// -------------------------------------------------------------------------
public static void main(String argv[]) {
	if(argv.length < 3) {
		logmsg("Invalid number of arguments specified...");
		printUsage();
	}
	else {
		String  ldbName   = (argv.length > 3) ? argv[3] : "cmbdb";
		String  luserId   = (argv.length > 4) ? argv[4] : "edms1";
		String  lPassword = (argv.length > 5) ? argv[5] : "edms1";

		TQStringFed lQStringFed = new TQStringFed(argv[0]);
		lQStringFed.startQueryLoop(argv[1], argv[2], ldbName, luserId, lPassword);
	}
}

// -------------------------------------------------------------------------
public static void printUsage() {
	logmsg("usage:");
	logmsg("    java TQStringFed packageName      QueryType ExecuteTimes DBName UID Password");
	logmsg("    java TQStringFed packageName      QueryType ExecuteTimes");
	logmsg("    java TQStringFed client/server/cs PQTQCOCA  ExecuteTimes\n");
	logmsg("    e.g. java TQStringFed server PQTQCA 10000 cmbdb edms1 edms1");
	logmsg("         java TQStringFed server PQTQCA 10000");
}

// -------------------------------------------------------------------------
	private  dkDatastore  _datastore   = null;
	private  PrintWriter  _logFile     = null;
	private  DateFormat   _dateFormat  = null;
	private  long         _startTime   = 0;
	private  String       _packageName = "";

// -------------------------------------------------------------------------
public static final long toLong(String aNum) {
	long rc = 0;

	if(aNum != null) {
		try {
			rc   = (Long.valueOf(aNum.trim())).longValue();
		}
		catch(Exception e) {
		}
	}
	return rc;
}

// -------------------------------------------------------------------------
}
