/* -------------------------------------------------------------------
 * @copyright(disclaimer) 
 * IBM Enterprise Information Portal for Multiplatforms V7.1  (program number 5697-G29) 
 * (c ) Copyright IBM Corp. 1998, 2000. All Rights Reserved. 
 * IBM Content Manager for Multiplatforms V6.1  (program number 5648-D03) 
 * (c ) Copyright IBM Corp. 1994, 2000. All Rights Reserved. 
 * 
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.  Licensed Materials - Property of IBM 
 * 
 * DISCLAIMER OF WARRANTIES : 
 * 
 * Permission is granted to copy and modify this code, and to 
 * distribute modified versions provided that both the copyright 
 * notice and this permission notice appear in all copies and modified 
 * versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS 
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM 
 * WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE 
 * OF THE CODE.
 * 
 * @endCopyright
 */

/*
 * Description    : sample evaluate - Information Catalog
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*; // using server package: 3-tier
import java.io.*;
import java.lang.*;

public class TEvaluateIC implements DKConstantIC
{
   // Main method
   public static void main(String argv[])
   {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);


      //*** for input parameters
      String database = new String(""),    // database
             userid = new String(""),    //userid
              pw = new String("");        // pw


      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
         System.out.println("Usage: " );
         System.out.println( "  java TEvaluateIC "
                           + "<database> <userid> <pw>" );
         return;
      }
      else
      {
         database = argv[0];
         userid = argv[1];
         pw = argv[2];
      }

      try
      {
         String cmd = null;
         DKNVPair parms[] = null;
         DKResults pResults;
         DKDDO item = null;
         dkQuery pQry;
         DKResultSetCursorIC pCur;
         dkIterator pIter;
         int count = 0;
         short prop_id;
         short type;


         DKDatastoreIC dsIC = new DKDatastoreIC();

         dsIC.connect( database, userid, pw, "");                    // connect to server
         System.out.println("...datastore connected");



// Parametric Query - count = 3
// Use LIKE and OR
// TABLE - DIMENSON - Dimensions within a multi-dimensional database
// NAME - Name
// SHRTDESC - Short description
         String qry = "SEARCH=(ENTITY=Dimensions within a multi-dimensional database," +
               "MAX_RESULTS=10,COND=( ('Name' LIKE 'M%')" +
               " OR ('Short description' = 'Product dimension')));" +
               "OPTION=(CONTENT=YES)";

         pResults = (DKResults)dsIC.evaluate(qry, DK_PARAMETRIC_QL_TYPE, parms);
         System.out.println("--- Evaluate query results ---");
// Call processResults
       processResults((dkCollection)pResults);
       System.out.println("--- Number of items = " + pResults.cardinality());
       System.out.println("query results processed");


// SQL Query - count = 0
// Use LIKE and AND
// TABLE - DIMENSON - Dimensions within a multi-dimensional database
// NAME - Name
// SHRTDESC - Short description
         qry = "Select * from dimenson where name like 'M%'" +
               " and shrtdesc = 'Product dimension'"; 

         pResults = (DKResults)dsIC.evaluate(qry, DK_CM_SQL_QL_TYPE, parms);
         System.out.println("--- Evaluate query results ---");
// Call processResults
       processResults((dkCollection)pResults);
       System.out.println("--- Number of items = " + pResults.cardinality());
       System.out.println("query results processed");




// evaluate with DKCQExpr

         DKNVPair soptions[] = new DKNVPair[1];
         DKNVPair qoptions[] = new DKNVPair[1];
         DKNVPair tnv = null;
         DKCQExpr cqe = new DKCQExpr();
         DKQExpr qe = new DKQExpr();
         DKQTerm qterm = new DKQTerm("Short description", DKOpCode.EQ, "'Product dimension'");  // Modified query to make sense for IC
         DKQSubExpr sqe = new DKQSubExpr();
         String strTmp = "";
         strTmp = "5";
         tnv = new DKNVPair("MAX_RESULTS",strTmp);
         soptions[0] = tnv;
         sqe.setOptionList(soptions);

         cqe.setQLType(DK_PARAMETRIC_QL_TYPE);
         qe.setQLType(DK_PARAMETRIC_QL_TYPE);
         sqe.setTranslation(false);
         sqe.setEntityName("Dimensions within a multi-dimensional database");
         sqe.setQueryTerm(qterm);
         strTmp = "YES";
         tnv = new DKNVPair("CONTENT", strTmp);
         qoptions[0] = tnv;
         qe.setOptionList(qoptions);
         qe.addSubQuery(sqe);            // CC - Have to set the subquery!
         cqe.setQueryExpr(qe);

         pResults = (DKResults)dsIC.evaluate(cqe);

         System.out.println("");
         System.out.println("  Num of query results: " + pResults.cardinality());


         dsIC.disconnect();             // disconnect from server
         System.out.println("...datastore disconnected");
         dsIC.destroy();
      }
      catch (DKException exc)
      {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
         System.out.println("EXCEPTION FAILURE");
      }
      catch (Exception exc)
      {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
         System.out.println("EXCEPTION FAILURE");
      }
   }


  public static void processResults(dkCollection results)
                throws DKException, Exception
  {
    Short    sVal;
    Integer  iVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println( "number of items : " + results.cardinality());

    dkIterator iter = results.createIterator();

    while(iter.more()) {
       item = (DKDDO) iter.next();
       if (item != null) {
     //       item.retrieve();
      sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      itemType = sVal.shortValue();
      switch(itemType) {
        case DK_CM_DOCUMENT:
        {
          System.out.println(" Item is a document, number of data item : " + item.dataCount());
          for (i = 1; i <= item.dataCount(); i++) {
         dataName = item.getDataName(i);
         System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
         }
          }
          break;
        default:
          System.out.println("  Item is not recognized ");
          break;
      }
       }
    }
  }
}

