/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Work Package
    When an instance of a Routing Process is in progress, a Work Package is the vehicle
    through with an Item moves along through the Routing Process.  A Work Package 
    contains all necessary information about the Process and the Item that it is
    transporting.  The Work Package is the object that the User-Application will use
    and manipulate.
        
    Priority
        One of the properties of a Work Package is its priority.  Any number may
        be specified and used.  Using & enforcing priority is the responsibility
        of the User-Program to allow maximum flexability.

Starting a Process
    An Item may be easily entered into a process.  The process will begin at the first
    work node of the given process.  A Work Package will be created for the first 
    Work Node and the PID String for that new Work Package will be returned.  That
    Work Package PID String may be used to access that package or a Work List may
    be used to access packages in general.

Performing Work at Node
    The Item specified by the Item PID String in the Work Package may be retrieved
    and manipulated.  In general, retrieve the Folder, examine the contents, and
    perform any work that is needed.
    
    NOTE:  If Routing a Folder, do not forget to retrieve with the Outbound Link
           Retrieval Options as explained in SFolderICM.

Select Route & Continue Process
    Select a Route & move the Item referenced by the Item Pid in the Work Package
    from the current node to the next node.  Select the route by using the 
    "Selection Name" for one of the routes exiting the current node.  Available 
    routes exiting this node were defined when defining the Routing Process.
    
    The existing Work Package will be removed from the system and a new Work Package
    will be created at the next Work Node.  The new Work Package will be owned by the
    owner specified.  The PID of the new Work Package will be returned.

    Detecting End of Proccess / End Node
        If the package reaches the end node, 'null' will be returned from the
        continueProcess() function.  When this happens, th process and Work Package
        have been removed from the system.

    Auto Jump
        When an item reaches a Collection Point who requirements are already met,
        The system will automatically continue the item on to the next node.  
        Similarly, when an item waiting at a collection point meets the collection
        points criteria, it will automatically move on to the next node.  Only
        one route may exit a collection point node.

Suspend Process
    An instance of a process may be suspended for a period of time and/or pending
    a set of requirements that must be met before it is resumed.  This does not 
    relate to processes & threads in a programming environment.  The thread or
    process in Java or C++ will not be stopped.
    
    Time Period
        When the time period (in minutes) expires, it will automatically resume.
        
    Resume List
        A Resume List is used if requirements are specified just as they are used 
        with Collection Points (See SDocRoutingDefinitionCreationICM).  It is 
        specified as a DKSequentialCollection of DKResumeListEntryICM objects,
        which consists of Item Type & Quantity.  When the requirements are met,
        it will automatically resume.

Resume Process / Break Suspension
    A suspended process may be resumed explicitly.  This simply takes the process
    off suspension, resuming it to normal operation.
    
Terminate Process
    A process may be explicitly terminated before it reaches the end node.  The 
    Work Package / Process Instance will be removed from the system.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.util.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SDocRoutingProcessingICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Routing Process, Work Packages, Starting a Process, Performing Work
 *                    at Node, Selecting Route & Advancing Process, Detecting End of Process
 *                    or End Node, Auto Jump, Supsending a Process, Resuming a Process, 
 *                    Terminating a Process, & Accessing the Routing Service Object.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Routing Process
 *                    Work package
 *                    Starting a Process
 *                    Performing Work at Node
 *                    Select Route & Continue Process
 *                    Detecting End of Process / End Node
 *                    Auto Jump
 *                    Suspend Process
 *                    Resume Process / Break Suspension
 *                    Terminate Process
 *                    Accessing the Routing Service Object.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SDocRoutingProcessingICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemCreationICM.java           (Needed by SInsuranceModelItemCreationICM)
 *                    SItemRetrievalICM.java          (Needed by SInsuranceModelItemCreationICM)
 *                    SLinksICM.java                  (Needed by SInsuranceModelItemCreationICM)
 *                    SInsuranceModelItemCreationICM.java
 *                    SDocRoutingListingICM.java
 ************************************************************************************************/
public class SDocRoutingProcessingICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SDocRoutingProcessingICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SDocRoutingProcessingICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //---------------------------------------------------------------------
            // Connect to datastore
            //---------------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Create the Items now that will be used as the work performed
            // at the nodes in the process.
            System.out.println("Creating Insurance Model Items...");

                java.util.Vector ddoVector = SInsuranceModelItemCreationICM.createInsuranceModelItems(dsICM);  // Uses Function provided in the creation sample.
                
                DKLobICM autoClaim1      = (DKLobICM) ddoVector.elementAt(0);
                DKLobICM autoClaim2      = (DKLobICM) ddoVector.elementAt(1);
                DKLobICM autoClaim3      = (DKLobICM) ddoVector.elementAt(2);
                DKImageICM photo1a       = (DKImageICM) ddoVector.elementAt(3);
                DKImageICM photo1b       = (DKImageICM) ddoVector.elementAt(4);
                DKImageICM photo2        = (DKImageICM) ddoVector.elementAt(5);
                DKImageICM photo3a       = (DKImageICM) ddoVector.elementAt(6);
                DKImageICM photo3b       = (DKImageICM) ddoVector.elementAt(7);
                DKLobICM adjusterReport1 = (DKLobICM) ddoVector.elementAt(8);
                DKLobICM adjusterReport2 = (DKLobICM) ddoVector.elementAt(9);
                DKLobICM adjusterReport3 = (DKLobICM) ddoVector.elementAt(10);
                DKLobICM policeReport1   = (DKLobICM) ddoVector.elementAt(11);
                DKLobICM policeReport2   = (DKLobICM) ddoVector.elementAt(12);
                DKLobICM policeReport3   = (DKLobICM) ddoVector.elementAt(13);
                DKLobICM policeReport4   = (DKLobICM) ddoVector.elementAt(14);
                DKLobICM policy1         = (DKLobICM) ddoVector.elementAt(15);
                DKLobICM policy2         = (DKLobICM) ddoVector.elementAt(16);
                DKLobICM policy3         = (DKLobICM) ddoVector.elementAt(17);

            System.out.println("Created Insurance Model Items.");

            //---------------------------------------------------------------------
            // Routing Service Object
            //---------------------------------------------------------------------
            // DKDocRoutingServiceICM:  Provides the core routing services such as
            //                          like starting, terminating, continuing, 
            //                          suspending, and resuming a process.
            System.out.println("Creating Routing Service Object...");

                DKDocRoutingServiceICM routingService = new DKDocRoutingServiceICM(dsICM);     // Obtain the core routing service object.

            System.out.println("Created Routing Service & Management Objects.");

            //------------------------------------------------------------------------
            // Create a Document or Folder to be Routed
            //------------------------------------------------------------------------
            // A Folder or a Document may be routed.  In this Sample, a Folder will
            // be routed and used to contain an number of documents to route together.
            //
            // Create an Item of Semantic Type Folder to be routed.  Any Item Type
            // may be used.  A Non-Resource Item is recommended since this is a 
            // simple folder to route the contents as a package.
            //
            // Please refer to Sample SFolderICM for comprehensive Folder explanation.
            // Please refer to Sample SItemCreationICM for comprehensive Non-Resource
            //   Item Creation explanation.
            System.out.println("Creating Folder to be Routed...");

                DKDDO ddoFolder = dsICM.createDDO("S_simple", DKConstant.DK_CM_FOLDER); // Using Item Type Created in SItemTypeCreationICM.  May be replaced by any Item Type.
                ddoFolder.add();  // Save folder to the persistent datastore.

            System.out.println("Created Folder to be Routed.");

            //------------------------------------------------------------------------
            // Start a Process
            //------------------------------------------------------------------------
            // Start an instance of the process defined in the SDocRoutingDefinitionICM
            // Sample, S_claimProcess.  The process will begin at the first work node
            // of the given process.  A Work Package will be created for the first 
            // Work Node.  The PID String for the new Work Package will be returned.
            System.out.println("Starting an Instance of Process 'S_claimProcess'...");

  	            String workPackagePidStr = routingService.startProcess("S_claimProcess", ddoFolder.getPidObject().pidString(), 1, userName); // Using the UserName specified by commandline to log on in this Sample.

            System.out.println("Started an Instance of Process 'S_claimProcess'.");

            //------------------------------------------------------------------------
            // Obtain / Retrieve Work Package
            //------------------------------------------------------------------------
            // Obtain the work package started above.  The work package pid string
            // was returned.  That may be used to easily obtain an instance of that
            // work package.
            System.out.println("Obtaining / Retrieving Work Package...");
        
                DKWorkPackageICM workPackage = routingService.retrieveWorkPackage(workPackagePidStr,false); // specifying false to not check it out.  'true' would check out the object as well.
                SDocRoutingListingICM.printWorkPackage(workPackage); // Print the Work Package using the wrapper provided in Listing Sample.

            System.out.println("Obtained / Retrieved Work Package.");

            //------------------------------------------------------------------------
            // Perform Work at Node
            //------------------------------------------------------------------------
            // This is the node that the Claimant Fills out a Claim.
            // The SInsuranceModelItemCreationICM's claim creation will be used to 
            // create a claim, and it will be added to the Work Package's Folder.
            //
            // In general, retrieve the Folder as shown below, examine the contents
            // and perform any work that is needed.  Please refer to SFolderICM for
            // information on retrieving & using folder contents.
            System.out.println("Performing Work at Node:  Claimant Fills Out a Claim...");

                // First, obtain the Work Package's Folder, using the Item Pid given.
                ddoFolder = dsICM.createDDO(workPackage.getItemPidString());  // Refer to SItemRetrievalICM for more information on Item Retrieval
                ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND + DKConstant.DK_CM_CONTENT_ATTRONLY);  // Refer to SFolderICM & SItemRetrievalICM for more information.
                
                // Create an Insurance Claim Item.  
                // NOTE:  Normally, all work be performed here, but for Sample purposed, it
                //        was created in the beginning of this sample.
                // Adding Auto Claim to Folder...                
                dsICM.checkOut(ddoFolder);                   // Check Out the Folder for Modification
                DKFolder dkFolder = (DKFolder) ddoFolder.getData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));   // Obtain the dkFolder collection.
                dkFolder.addElement(autoClaim1);             // add the claim to the folder.
                ddoFolder.update();                          // Save changes.
                dsICM.checkIn(ddoFolder);                    // Explicitly check in or use the DK_CM_CHECKIN update option.
                
            System.out.println("Performed Work at Node:  Claimant Fills Out a Claim.");

            //------------------------------------------------------------------------
            // Select Route & Continue Process
            //------------------------------------------------------------------------
            // Select a Route & move the Item referenced by the Item Pid in the Work
            // Package from the current node to the next node.  The existing Work Package
            // will be removed from the datastore and a new Work Package will be created
            // at the next Work Node.  The new work package will now be owned by the owner
            // specified.  The PID of the new Work Package will be returned.
            //
            // When the work package reaches the End node, 'null' will be returned in
            // place of the PID String.
            //
            // In this sample, moving from Filling Claim Node to Gather Documents Node.
            System.out.println("Selecting Route & Continuing Process...");

                // Select Route by specifying the "Selection Name" specified for the desired
                // route to take between this node and the next.  Any number of routes & 
                // names may be created.  This sample created one called "Continue".
            
                workPackagePidStr = routingService.continueProcess(workPackagePidStr, "Continue", userName); // Select the route named "Continue" and assign to the user specified in the commandline of this Sample.

            System.out.println("Selected Route & Continued Process.");

            //------------------------------------------------------------------------
            // Detect End of Process / End Node
            //------------------------------------------------------------------------
            // If the package reached the end node, 'null' will be returned from the
            // continueProcess() function.  When this happens, the process and Work
            // Package have been removed from the system.
            //
            // In this Sample, it will have not reached the End Node yet.
            System.out.println("Detecting End of Process / End Node...");

                if(workPackagePidStr == null)
                    throw new Exception("Work Package has reached the End Node and was removed from the system.");
                else
                    System.out.println("Work Package has not yet reached the End Node.");

            System.out.println("Detected End of Process / End Node.");

            //------------------------------------------------------------------------
            // Obtain / Retrieve Work Package
            //------------------------------------------------------------------------
            // Obtain the new Work Package create above for the next Work Node.  The 
            // work package pid string was returned. 
            System.out.println("Obtaining / Retrieving New Work Package...");
        
                workPackage = routingService.retrieveWorkPackage(workPackagePidStr,false); // specifying false to not check it out.  'true' would check out the object as well.
                SDocRoutingListingICM.printWorkPackage(workPackage);                       // Print the Work Package using the wrapper provided in the Listing Sample.

            System.out.println("Obtained / Retrieved New Work Package.");

            //------------------------------------------------------------------------
            // Perform Work at Node
            //------------------------------------------------------------------------
            // This is the node, a collection point, a number of documents / Items are
            // required.  An agent at the insurance company will perform some research
            // or invistation and will submit all required documents in the Work 
            // Package's Folder.
            System.out.println("Performing Work at Node:  Gathering Related Documents...");

                // First, obtain the Work Package's Folder, using the Item Pid given.
                ddoFolder = dsICM.createDDO(workPackage.getItemPidString());  // Refer to SItemRetrievalICM for more information on Item Retrieval
                ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND + DKConstant.DK_CM_CONTENT_ATTRONLY);  // Refer to SFolderICM & SItemRetrievalICM for more information.
                
                // Obtain Policy, Police Report, & Photographs.
                // NOTE:  Normally, all work be performed here, but for Sample purposed, it
                //        was created in the beginning of this sample.  Query is a likely
                //        alternative since these items may already exist in the datastore.
                // Adding Documents...
                dsICM.checkOut(ddoFolder);                   // Check Out the Folder for Modification
                dkFolder = (DKFolder) ddoFolder.getData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));   // Obtain the dkFolder collection.

                dkFolder.addElement(policy1);
                dkFolder.addElement(policeReport1);
                dkFolder.addElement(photo1a);
                dkFolder.addElement(photo1b);       

                ddoFolder.update();                          // Save changes.
                dsICM.checkIn(ddoFolder);                    // Explicitly check in or use the DK_CM_CHECKIN update option.
                
            System.out.println("Performed Work at Node:  Gathered Related Documents.");

            //------------------------------------------------------------------------
            // Obtain Work Package After Collection Point Complete / Auto Jump
            //------------------------------------------------------------------------
            // Once a collection point meets the criteria specified, the process will
            // automatically be continued and the item being routing will automatically
            // move to the next node.  This is referred to as "auto jump".  A collection
            // point can only have one route leaving it, which the system will
            // automatically use to route to the next node after the requirements are
            // met.
            //
            // Since the process has moved on, the work package is no longer valid.
            // It must be obtained again.  As will be shown in great detail in the
            // SDocRoutingListingICM sample, it is very easy to access the lists of
            // work packages.  In this particular example, we know that exactly one
            // item will be waiting at the next node.
            System.out.println("Obtaining Work Pacackage After Collection Point Complete...");

                // This will be covered in more detail in the SDocRoutingListingICM Sample.
                String[] workPackagePIDs = routingService.listWorkPackagePidStrings("S_agentWL",userName); // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.
                workPackagePidStr = workPackagePIDs[0];  // In this example, we know that 
                                                         // exactly one item is waiting.

            System.out.println("Obtained Work Pacackage After Collection Point Complete.");

            //------------------------------------------------------------------------
            // Suspend Process
            //------------------------------------------------------------------------
            // An instance of a process may be suspended for a period of time and/or 
            // pending a set of requirements that must be met before it is resumed.
            //
            // Time Period:  The time period is in minutes.  When the time period
            //               expires, it will automatically resume.
            //
            // Resume List:  A Resume List is used if requirements are specified just
            //               as they are used with Collection Points.  It is specified
            //               as a DKSequentialCollection of DKResumeListEntryICM objects,
            //               which consists of Item Type & Quantity.  When the
            //               requirements are met, it will automatically resume.
            //
            // In this Sample, the process will be suspended for 2 minutes and no
            // requirements will be specified.
            System.out.println("Suspending Process for 2 minutes (no new requirements)...");

                dkCollection requirements = new DKSequentialCollection();
                routingService.suspendProcess(workPackagePidStr, 2, requirements);

            System.out.println("Suspended Process for 2 minutes (no new requirements).");

            //------------------------------------------------------------------------
            // Resuming Process / Breaking Suspension
            //------------------------------------------------------------------------
            // A suspended process may be resumed explicitly.  This simply takes the
            // process off suspension, resuming it to normal operation.
            System.out.println("Resuming Process / Breaking Suspension...");
            
                routingService.resumeProcess(workPackagePidStr);
            
            System.out.println("Resumed Process / Broke Suspension.");
            
            //------------------------------------------------------------------------
            // Perform Work at Node
            //------------------------------------------------------------------------
            // This is the node where the Adjuster fills out the Adjustment Report.
            System.out.println("Performing Work at Node:  Adjustment Report...");

                // First, obtain the Work Package's Folder, using the Item Pid given.
                ddoFolder = dsICM.createDDO(workPackage.getItemPidString());  // Refer to SItemRetrievalICM for more information on Item Retrieval
                ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND + DKConstant.DK_CM_CONTENT_ATTRONLY);  // Refer to SFolderICM & SItemRetrievalICM for more information.
                
                // Create the Adjustment Report.
                // NOTE:  Normally, all work be performed here, but for Sample purposed, it
                //        was created in the beginning of this sample.
                // Adding Adjuster Report to Folder...                
                dsICM.checkOut(ddoFolder);                   // Check Out the Folder for Modification
                dkFolder = (DKFolder) ddoFolder.getData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));   // Obtain the dkFolder collection.
                dkFolder.addElement(adjusterReport1);
                ddoFolder.update();                          // Save changes.
                dsICM.checkIn(ddoFolder);                    // Explicitly check in or use the DK_CM_CHECKIN update option.
                
            System.out.println("Performed Work at Node:  Adjustment Report.");

            //------------------------------------------------------------------------
            // Select Route & Continue Process
            //------------------------------------------------------------------------
            // In this Sample, moving from Adjustment Node to End Node.
            System.out.println("Selecting Route & Continuing Process...");

                workPackagePidStr = routingService.continueProcess(workPackagePidStr, "Complete", userName); // Select the route named "Complete" and assign to the user specified in the commandline of this Sample.

            System.out.println("Selected Route & Continued Process.");

            //------------------------------------------------------------------------
            // Detect End of Process / End Node
            //------------------------------------------------------------------------
            // If the package reached the end node, 'null' will be returned from the
            // continueProcess() function.  When this happens, the process and Work
            // Package have been removed from the system.
            //
            // In this Sample, it will have reached the End Node.
            System.out.println("Detecting End of Process / End Node...");

                if(workPackagePidStr == null)
                    System.out.println("Work Package has reached the End Node and was removed from the system.");
                else
                    throw new Exception("Work Package has not yet reached the End Node.");

            System.out.println("Detected End of Process / End Node.");

            //------------------------------------------------------------------------
            // Terminate a Process
            //------------------------------------------------------------------------
            // A process may be explicitly terminated before it reaches the end node.
            // All Work Packages and the process instance will be removed from the system.
            //
            // Since the previous process was completed and has been removed by the system,
            // a new process will be started to demonstrate termination of a process.
            System.out.println("Terminating a Process (New Process Started)...");

                // Create a new Folder and Start a Process
                ddoFolder = dsICM.createDDO("S_simple", DKConstant.DK_CM_FOLDER); // Using Item Type Created in SItemTypeCreationICM.  May be replaced by any Item Type.
                ddoFolder.add();                                                  // Save folder to the persistent datastore.
  	            workPackagePidStr = routingService.startProcess("S_claimProcess", ddoFolder.getPidObject().pidString(), 1, userName); // Using the UserName specified by commandline to log on in this Sample.
  	            
  	            // Terminate Process
                routingService.terminateProcess(workPackagePidStr);

            System.out.println("Terminated a Process (New Process Started).");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SDocRoutingProcessingICM
              
