/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Creating Blank DDOs for Existing Items
    All DDOs must be created through the DKDatastoreICM.createDDO() functions.  If the 
    Item has already been added to the datastore, the DDO can be re-created using its
    PID.  The PID is a Persistent Identification that uniquely identifies the particular
    component / row in the database.  The full pid string must be used in the 
    DKDatastoreICM.createDDO(pidString) function.
    
    It is technically possible to create a retrievable DDO by using the DKDDO constructor
    and manually setting the PID information, but it is easiest to use the Datastore's
    createDDO function.
    
    Query/Search is another way of obtaining DDOs that will be created for you.

Retrieving Items
    Items are retrieved through the DKDDO.retrieve(options) function.  Retrieval
    options may be specified and are recommended to ensure that only what is 
    desired is being retrieved.  This will save performance, avoiding extra
    calls to the library server for children or links if the user program knows
    that there are none in the data model or that they will not be used at this
    time.  Retrieval Options are binary, so they may be added together to set more
    than one option bit.

    Retrieval Option      Constant                     Explanation
    --------------------  ---------------------------  ------------------------------------------
    Pid Information Only  DK_CM_CONTENT_IDONLY         Useful in Query/Search.  Query Will return
                                                       collection with empty DDOs with complete
                                                       PID information.
    .............................................................................................
    Attributes Only       DK_CM_CONTENT_ATTRONLY       Retrieves only Attributes
    .............................................................................................
    Children              DK_CM_CONTENT_CHILDREN       Retrieve the Child Components. 
    .............................................................................................
    One Level             DK_CM_CONTENT_ONELEVEL       Retrieve One-Level of Information, 
                                                       Chilren & Attributes
    .............................................................................................
    Outbound Links        DK_CM_CONTENT_LINKS_OUTBOUND Retrieve Outbound Link information.
    .............................................................................................
    Inbound Links         DK_CM_CONTENT_LINKS_INBOUND  Retrieve Inbound Link information.
    .............................................................................................
    Item Tree             DK_CM_CONTENT_ITEMTREE       Retrieve the entire tree of information,
                                                       including all meta data / attributes, 
                                                       children & sub-children, links, etc.
    .............................................................................................
    Item Tree             DK_CM_CONTENT_ITEMTREE...    Retrieve the entire tree of information,
                                        ..._NO_LINKS   but save time and increase performance by 
                                                       not checking for links.
    .............................................................................................
    No Resource Content   DK_CM_CONTENT_NO             If it is a Resource Item, do not retrieve
                                                       the Resource object content.
    .............................................................................................
    Resource Content      DK_CM_CONTENT_YES            If it is a Resource Item, retrieve the
                                                       Resource object content.
    .............................................................................................
    Only Resource Content DK_CM_CONTENT_ONLY           If it is a Resource Item, retrieve only 
                                                       the Resource object content.
    .............................................................................................
    Check Out Item        DK_CM_CHECKOUT               In addition, check out / lock the item.
    .............................................................................................
    Latest Version        DK_CM_VERSION_LATEST         If versioning is enabled, retrieve the 
                                                       latest version, despite what the specific
                                                       object the Pid information specifies.
    ---------------------------------------------------------------------------------------------
    Note: Constants available in com.ibm.mm.sdk.common.DKConstant
  

Retrieving Child Components
    It is possible to retrieve an individual child component, given its PID string, 
    However, Items cannot be updated by retrieving the component and calling the
    DKDDO.update() method.  All Updates should be handled through the Root Item.
    The child component may be modified within the retrieved structure of the 
    Root Item, and then the DKDDO.update() method should be called on the Root Component.

Reading Attributes / Data Items Programmatically
    The user program does not need to know the type of java.lang.Object
    that each attribute is.  It can be determined either through the
    attribute definition (see SAttributeDefinitionCreation/RetrievalICM),
    or by the data property type.  Valid data property types are listed below.

    Note: The following applies to all connectors.  A subset is used
          by the IBM Content Management Connector (ICM).
          
    Data Type         Constant                             Object
    ----------------  -----------------------------------  ----------------------------
    Undefined         DK_CM_DATAITEM_TYPE_UNDEFINED        java.lang.Object (Or Other)
    String (Varchar)  DK_CM_DATAITEM_TYPE_STRING           java.lang.String
    Short             DK_CM_DATAITEM_TYPE_SHORT            java.lang.Short
    Long              DK_CM_DATAITEM_TYPE_LONG             java.lang.Long
    Float             DK_CM_DATAITEM_TYPE_FLOAT            java.lang.Double
    Decimal           DK_CM_DATAITEM_TYPE_DECIMAL          java.math.BigDecimal
    Date              DK_CM_DATAITEM_TYPE_DATE             java.sql.Date
    Time              DK_CM_DATAITEM_TYPE_TIME             java.sql.Time
    Timestamp         DK_CM_DATAITEM_TYPE_TIMESTAMP        java.sql.Timestamp
    Double            DK_CM_DATAITEM_TYPE_DOUBLE           java.lang.Double
    Byte Array        DK_CM_DATAITEM_TYPE_BYTEARRAY        byte bytes[]
    DDO               DK_CM_DATAITEM_TYPE_DDOOBJECT        com.ibm.mm.sdk.common.DKDDO
    XDO               DK_CM_DATAITEM_TYPE_XDOOBJECT        com.ibm.mm.sdk.common.DKLobICM / dkXDO / or Sub-Class
    Data Object Base  DK_CM_DATAITEM_TYPE_DATAOBJECTBASE   com.ibm.mm.sdk.common.dkAbstractDataObjectBase
    Collection        DK_CM_DATAITEM_TYPE_COLLECTION       com.ibm.mm.sdk.common.DKSequentialCollection, or dkCollection
    DDO Collection    DK_CM_DATAITEM_TYPE_COLLECTION_DDO   com.ibm.mm.sdk.common.DKChildCollection, DKFolder, DKSequentialCollection, dkCollection, or other.
    XDO Collection    DK_CM_DATAITEM_TYPE_COLLECTION_XDO   com.ibm.mm.sdk.common.DKParts, DKSequentialCollection, dkCollection, or other.
    Link Collection   DK_CM_DATAITEM_TYPE_LINKCOLLECTION   com.ibm.mm.sdk.common.DKLinkCollection
    Array             DK_CM_DATAITEM_TYPE_ARRAY            java.lang.Object[]
    ----------------------------------------------------------------------------------
    ** Constants Defined in com.ibm.mm.sdk.common.DKConstant

Detecting Nonexistant Data Items
    When obtaining the dataid of any attribute, property, etc. of a DDO, if '0'
    is returned, it indicates that the data item is nonexistant in this ddo object.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Items, PID Information, Retrieving & Accessing Child Components
 *                    & Multivalue Attributes, Retrieval Options, Accessing Attributes
 *                    Explicitly & Programmatically, Detecting Nonexistant Attributes & 
 *                    Properties in DDO, Printing DDOs.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: PID Information
 *                    Retrieving Items
 *                    Retrieving Child Components
 *                    Accessing Child Components / Multivalue Attributes
 *                    Retrieval Options
 *                    Accessing Attributes
 *                    Accessing Attributes Programmatically Using the Component Definition
 *                    Accessing All DDO Data Items Programmatically
 *                    Detecting Nonexistant Attributes / Properties in DDO
 *                    Printing DDOs
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemCreationICM.java
 ************************************************************************************************/
public class SItemRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemRetrievalICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of Item Property type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            System.out.println("Creating & Saving Sample Items...");

	            DKDDO ddoDocument = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_DOCUMENT);
	            DKDDO ddoFolder   = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_FOLDER);   
	            DKDDO ddoItem     = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_ITEM);     

                // Set the reference attribute for ddoItem before it is added, and after the others were added.
                dsICM.checkOut(ddoItem);   // Covered in SItemUpdateICM
                ddoItem.setData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_myRef"),ddoFolder); // Using DKDDO named 'ddoFolder' made persistent above.
                ddoItem.update();          // Covered in SItemUpdateICM

            System.out.println("Created & Saved Sample Items.");

            //-------------------------------------------------------------
            // Obtain the Pid Strings so that the DDOs may be recreated.
            //-------------------------------------------------------------
            // First, obtain the DKPidICM object using the getPidObject() operation.
            // If the DDO was created by DKDatastoreICM, it may be cast from dkPid to DKPidICM.
            // Next, call the pidString() function for a String representation.
            System.out.println("Obtaining PID Information...");

                String ddoDocumentPidString = ddoDocument.getPidObject().pidString();
                String ddoFolderPidString   = ddoFolder.getPidObject().pidString();
                String ddoItemPidString     = ddoItem.getPidObject().pidString();

            System.out.println("Obtained PID Information.");

            //-------------------------------------------------------------
            // Re-create Blank DDOs for Existing Items
            //-------------------------------------------------------------
            // As stated in the header documentation, this is one valid 
            // means to re-create a blank ddo for an single existing item 
            // and should be used over using the DKDDO() constructor.
            // Query/Search is the most popular and useful approach to 
            // obtaining empty items with Pid information filled out.
            System.out.println("Re-creating Blank DDOs for Existing Items...");

                ddoDocument = dsICM.createDDO(ddoDocumentPidString);
                ddoFolder   = dsICM.createDDO(ddoFolderPidString);
                ddoItem     = dsICM.createDDO(ddoItemPidString);

            System.out.println("Re-created Blank DDOs for Existing Items.");

            //-------------------------------------------------------------
            // Retrieve Items
            //-------------------------------------------------------------
            // Retrieve the items, specifying Retrieval Options if desired.
            // For a list and explanation on Retrieval Options, see header documentation.
            System.out.println("Retrieving Items...");
            
               ddoDocument.retrieve();  // Use default retrieval options
               ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND + DKConstant.DK_CM_CONTENT_LINKS_INBOUND); // Only Outbound is required for Folders.
               ddoItem.retrieve(DKConstant.DK_CM_CONTENT_ITEMTREE);  // Retrieve the whole tree of information.

            System.out.println("Retrieved Items.");

            //-------------------------------------------------------------
            // Reading Attribute Values, Knowing Type
            //-------------------------------------------------------------
            // If the type of each attribute is known by the user program, the returned
            // java.lang.Object values can be cast into the correct value.
            System.out.println("Reading Attribute Values, Knowing Type...");

                String  attrVal1 = (String) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_varchar")); 
                Integer attrVal2 = (Integer)ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_integer")); 
                Double  attrVal3 = (Double) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_double")); 

                System.out.println("Attr 'S_varchar' value: "+attrVal1);
                System.out.println("Attr 'S_integer' value: "+attrVal2);
                System.out.println("Attr 'S_double'  value: "+attrVal3);

            System.out.println("Read Attribute Values, Knowing Type.");

            //-------------------------------------------------------------
            // Read / Access Attribute Groups
            //-------------------------------------------------------------
            // As explained in both SAttributeGroupDefCreationICM and in
            // the Set / Modify Attributes portion of SItemCreationICM,
            // the correct format for accessing Attributes Belonging to
            // Attribute Groups is: Format:  <Attribute Group Name>.<Attribute Name>
            System.out.println("Read / Access Attribute Groups...");

                String  attrVal4 = (String) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_address.S_street")); 
                String  attrVal5 = (String) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_address.S_city")); 
                String  attrVal6 = (String) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_address.S_state")); 
                String  attrVal7 = (String) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_address.S_zipCode")); 

                System.out.println("Group 'S_address', Attr 'S_street' value: "+attrVal4);
                System.out.println("Group 'S_address', Attr 'S_city'   value: "+attrVal5);
                System.out.println("Group 'S_address', Attr 'S_state'  value: "+attrVal6);
                System.out.println("Group 'S_address', Attr 'S_zipCode value: "+attrVal7);

            System.out.println("Read / Accessed Attribute Groups.");

            //-------------------------------------------------------------
            // Read / Access Reference Attributes
            //-------------------------------------------------------------
            // As explained in SItemCreationICM & SReferenceAttrDefICM, usage
            // of Reference Attributes is just like regurlar Attributes.  A
            // DKDDO value is contained.  The system will manage everything.
            System.out.println("Read / Access Reference Attributes...");

                DKDDO ddo = (DKDDO) ddoItem.getData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_myRef")); 

                if(ddo!=null){
                    System.out.println("             Reference:     Type:  "+ddo.getObjectType());
                    System.out.println("                         Item ID:  "+ ( (DKPidICM)ddo.getPidObject() ).getItemId());
                }
                else
                    System.out.println("             Reference:  null");

            System.out.println("Read / Accessed Reference Attributes.");

            //-------------------------------------------------------------
            // Reading Attribute Values Programmatically: All Data Items
            //-------------------------------------------------------------
            // The type of attribute may be determined Programmatically.  A
            // mechanism is built in, or Java's "instanceof" operator may be
            // used. 
            //
            // All data items can be listed, which will obtain more than
            // the user-defined attributes.  It will find all data members
            // of the DDO.
            System.out.println("Reading All DDO Data Items Programmatically...");

                System.out.println("Number of Attributes / Data Items: "+ddoItem.dataCount());
                for(short dataid=1; dataid<=ddoItem.dataCount(); dataid++) { // go through all attributes in the ddo
                    String name  = ddoItem.getDataName(dataid);
                    Object value = ddoItem.getData(dataid);
                    Short type   = (Short) ddoItem.getDataPropertyByName(dataid,DKConstant.DK_CM_PROPERTY_TYPE); 
                    String typeDescription = "To Be Determined...";
                    switch(type.intValue()){
                        case DKConstant.DK_CM_DATAITEM_TYPE_UNDEFINED:
                            typeDescription = "Undefined";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_STRING:
                            typeDescription = "String";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_SHORT:
                            typeDescription = "Short";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_LONG:
                            typeDescription = "Long";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_FLOAT:
                            typeDescription = "Float";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_DECIMAL:
                            typeDescription = "Decimal";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_DATE:
                            typeDescription = "Date";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_TIME:
                            typeDescription = "Time";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_TIMESTAMP:
                            typeDescription = "Timestamp";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_DOUBLE:
                            typeDescription = "Double";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_BYTEARRAY:
                            typeDescription = "Byte Array";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_DDOOBJECT:
                            typeDescription = "DDO";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_XDOOBJECT:
                            typeDescription = "XDO";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_DATAOBJECTBASE:
                            typeDescription = "Data Object Base";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION:
                            typeDescription = "Collection";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION_DDO:
                            typeDescription = "DDO Collection";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION_XDO:  
                            typeDescription = "XDO Colleciton";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_LINKCOLLECTION:
                            typeDescription = "DKLinkCollection";
                            break;
                        case DKConstant.DK_CM_DATAITEM_TYPE_ARRAY:
                            typeDescription = "Array";
                            break;
                        default:
                            typeDescription = "Unknown ("+type+")";
                            break;
                    }// end swith on data item type
                    
                    if(value!=null)
                        System.out.println("Data Item '"+name+"' of type '"+typeDescription+"' with value: "+value);
                    else
                        System.out.println("Data Item '"+name+"' of type '"+typeDescription+"' with value: null");
                }// end for all data items in DDO

            System.out.println("Read All DDO Data Items Programmatically.");

            //-------------------------------------------------------------
            // Access Child Components
            //-------------------------------------------------------------
            System.out.println("Accessing Child Components...");

                // First, get the child colleciton within the parent for the child object type.
                short dataid = ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_simpleChild");
                DKChildCollection children = (DKChildCollection) ddoItem.getData(dataid);

                if(children!=null)
                    System.out.println("Found '"+children.cardinality()+"' Children of type 'S_address'");
                else
                    System.out.println("Found 'null' Children of type 'S_simpleChild'");

            System.out.println("Accessed Child Components.");
            
            //-------------------------------------------------------------
            // Print DDO & Child components
            //-------------------------------------------------------------
            // Print a DDO using the wrapper at the bottom of this sample.            
            System.out.println("Printing DDO...");
                printDDO(ddoItem); // Use wrapper defined at bottom of sample.
            System.out.println("Printed DDO.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print the specified DDO.
    * @param ddo  DKDDO to print.
    **/
    public static void printDDO(DKDDO ddo) throws DKException, Exception{
        java.util.Vector childCollections = new java.util.Vector(); // when obtaining data items, collect child collections.
        java.util.Vector linkCollections = new java.util.Vector(); // when obtaining data items, collect link collections.

        DKPidICM pid = (DKPidICM) ddo.getPidObject(); // Obtain the Pid Information
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) ddo.getDatastore().datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKComponentTypeDefICM compType = (DKComponentTypeDefICM) dsDefICM.retrieveComponentTypeView(pid.getObjectType());
        
        System.out.println("--------------------------------------------------");
        System.out.println("                       DDO");
        System.out.println("--------------------------------------------------");
        System.out.println("PID:");
        System.out.println("        Component Type:  "+pid.getObjectType());
        System.out.println("               Item ID:  "+pid.getItemId());
        System.out.println("          Component ID:  "+pid.getComponentId());
        System.out.println("     Component Type ID:  "+pid.getComponentTypeId());
        System.out.println("            Primary ID:  "+pid.getPrimaryId());
        System.out.println("        Datastore Type:  "+pid.getDatastoreType());
        System.out.println("               Version:  "+pid.getVersionNumber());
        System.out.println("Item Info:");
        if(compType.isRoot()==false){ // if it isn't the root
            if(ddo.getParent()==null) // if no parent is specified and it is a child, this means that this DDO was retrieved seperately from the parent and therefore the parent is not known to this object in memory.  Obtaining the parent and retrieving "One Level" or "Item Tree" would set this property.  Alternatively, query/search may be used to find the parent, as shown in the retrieveParent() function provided in this sample.  Refer to Information Integrator for Content eSupport for more information.
                System.out.println("                Parent:  Exists, But Unknown to Object in Memory");
            else{
                System.out.println("                Parent:     Type:  "+ddo.getParent().getObjectType());
                System.out.println("                         Item ID:  "+((DKPidICM)((DKDDO)ddo.getParent()).getPidObject()).getItemId());
            }
        }else
            System.out.println("                Parent:  None, Is Root");
        if(ddo instanceof dkResource){
            if(ddo instanceof DKTextICM)
                System.out.println("          Resource XDO:  Yes - Text");
            else if(ddo instanceof DKImageICM)
                System.out.println("          Resource XDO:  Yes - Image");
            else if(ddo instanceof DKVideoStreamICM)
                System.out.println("          Resource XDO:  Yes - Video Stream");
            else if(ddo instanceof DKStreamICM)
                System.out.println("          Resource XDO:  Yes - Stream");
            else if(ddo instanceof DKLobICM)
                System.out.println("          Resource XDO:  Yes - Lob");
            else
                System.out.println("          Resource XDO:  Yes - dkResource");
        }
        else
            System.out.println("          Resource XDO:  No");
        
        if(compType.isRoot()){ // only applicable to roots
            Integer semanticType     = (Integer) ddo.getPropertyByName(DKConstantICM.DK_ICM_PROPERTY_SEMANTIC_TYPE);
            Short   itemPropertyType = (Short)   ddo.getPropertyByName(DKConstant.DK_CM_PROPERTY_ITEM_TYPE);

            switch(itemPropertyType.intValue()){
                case DKConstant.DK_CM_DOCUMENT:
                    System.out.println("         Item Property:  Document");
                    break;
                case DKConstant.DK_CM_FOLDER:
                    System.out.println("         Item Property:  Folder");
                    break;
                case DKConstant.DK_CM_ITEM:
                    System.out.println("         Item Property:  Item");
                    break;
                default:
                    System.out.println("         Item Property:  Other ("+itemPropertyType+")");
                    break;
            }//end switch on semantic type
            switch(semanticType.intValue()){
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_DOCUMENT:
                    System.out.println("         Semantic Type:  Document");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_FOLDER:
                    System.out.println("         Semantic Type:  Folder");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_CONTAINER:
                    System.out.println("         Semantic Type:  Folder");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_ANNOTATION:
                    System.out.println("         Semantic Type:  Annotation");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_HISTORY:
                    System.out.println("         Semantic Type:  History");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_NOTE:
                    System.out.println("         Semantic Type:  Note");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_BASE:
                    System.out.println("         Semantic Type:  Base");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_BASETEXT:
                    System.out.println("         Semantic Type:  Base Text");
                    break;
                case DKConstantICM.DK_ICM_SEMANTIC_TYPE_BASESTREAM:
                    System.out.println("         Semantic Type:  Base Stream");
                    break;
                default:
                    System.out.println("         Semantic Type:  Other ("+semanticType+")");
                    break;
            }//end switch on semantic type
        }// end if is root
        if(ddo.updatable())
            System.out.println("            Updateable:  Yes");
        else
            System.out.println("            Updateable:  No");

        System.out.println("Attributes / Data Items:");
        System.out.println("                Number:  "+ ddo.dataCount());
        // List all Data Items
        for(short dataid=1; dataid<=ddo.dataCount(); dataid++) { // go through all attributes in the ddo
            System.out.println("             Attribute:     Name:  "+ddo.getDataName(dataid));
            String name  = ddo.getDataName(dataid);
            Object value = ddo.getData(dataid);
            Short type   = (Short) ddo.getDataPropertyByName(dataid, DKConstant.DK_CM_PROPERTY_TYPE);
            String typeDescription = "To Be Determined...";
            switch(type.intValue()){
                case DKConstant.DK_CM_DATAITEM_TYPE_UNDEFINED:
                    System.out.println("                            Type:  Undefined");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_STRING:
                    System.out.println("                            Type:  String");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_SHORT:
                    System.out.println("                            Type:  Short");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_LONG:
                    System.out.println("                            Type:  Long");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_FLOAT:
                    System.out.println("                            Type:  Float");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_DECIMAL:
                    System.out.println("                            Type:  Decimal");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_DATE:
                    System.out.println("                            Type:  Date");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_TIME:
                    System.out.println("                            Type:  Time");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_TIMESTAMP:
                    System.out.println("                            Type:  Timestamp");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_DOUBLE:
                    System.out.println("                            Type:  Double");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_BYTEARRAY:
                    System.out.println("                            Type:  Byte Array (Blob)");
                    if(value!=null) 
                        System.out.println("                           Value:  <Byte Array Object>");
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_DDOOBJECT:
                    System.out.println("                            Type:  DDO");
                    if(value!=null){ 
                        System.out.println("                           Value:  DDO of type '"+((DKDDO)value).getPidObject().getObjectType()+"'");
                        System.out.println("                                       Item ID '"+((DKPidICM)((DKDDO)value).getPidObject()).getItemId()+"'");
                    }
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_XDOOBJECT:
                    System.out.println("                            Type:  XDO");
                    if(value!=null){
                        System.out.println("                           Value:  XDO of type '"+((DKDDO)value).getPidObject().getObjectType()+"'");
                        System.out.println("                                       Item ID '"+((DKPidICM)((DKDDO)value).getPidObject()).getItemId()+"'");
                    }
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_DATAOBJECTBASE:
                    System.out.println("                            Type:  Data Object Base");
                    if(value!=null) 
                        System.out.println("                           Value:  <Data Object Base Object>");
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION:
                    System.out.println("                            Type:  Collection");
                    if(value!=null) 
                        System.out.println("                           Value:  Collection with '"+((dkCollection)value).cardinality()+"' Objects.");
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION_DDO:
                    System.out.println("                            Type:  DDO Collection");
                    if(value!=null){
                        System.out.println("                           Value:  DDO Collection with '"+((dkCollection)value).cardinality()+"' DDOs.");
                        if(value instanceof DKChildCollection) // if it is a child collection, add it to a list for printing children later
                            childCollections.add(value);
                    }
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_COLLECTION_XDO:  
                    System.out.println("                            Type:  XDO Collection");
                    if(value!=null)
                        System.out.println("                           Value:  XDO Collection with '"+((dkCollection)value).cardinality()+"' XDOs.");
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_LINKCOLLECTION:
                    System.out.println("                            Type:  Link Collection");
                    if(value!=null){
                        System.out.println("                           Value:  Link Collection with '"+((dkCollection)value).cardinality()+"' Links.");
                        if(value instanceof DKLinkCollection) // if it is a link collection, add it to a list for printing Links later
                            linkCollections.add(value);
                    }
                    break;
                case DKConstant.DK_CM_DATAITEM_TYPE_ARRAY:
                    System.out.println("                            Type:  Array");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
                default:
                    System.out.println("                            Type:  Unknown ("+type+")");
                    if(value!=null) 
                        System.out.println("                           Value:  "+value);
                    break;
            }// end swith on data item type

            if(value==null) 
                    System.out.println("                           Value:  null");
            System.out.println("");
        }// end for all data items in DDO

        // Print Children (Brief)
        System.out.println("Children / Multivalue Attributes:");
        for(int i=0; i<childCollections.size(); i++){  // go through all collection we gathered when listing the data items of the DDO.
            DKChildCollection childCollection = (DKChildCollection) childCollections.elementAt(i); // get each child collection
            dkIterator iter = childCollection.createIterator(); // Create an iterator to go through Child Collection
            while(iter.more()){ // While there are children, print list
                DKDDO child = (DKDDO) iter.next(); // Move pointer to next child & return that object.
                System.out.println("                 Child:     Type:  "+child.getPidObject().getObjectType());
                System.out.println("                         Item ID:  "+((DKPidICM)child.getPidObject()).getItemId());
                System.out.println("");
            }
        }
            
        // Print Parts, if a Parts Collection Exists (See Document Model Samples for more information)
        short dataid = ddo.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKPARTS);
        if(dataid > 0){ // if a DKParts collection exists, print contents
            DKParts dkParts = (DKParts) ddo.getData(dataid); // obtain the DKParts collection
            System.out.println("Parts:");
            System.out.println("                Number:  "+dkParts.cardinality());
            dkIterator iter = dkParts.createIterator(); // Create an iterator to go through Collection
            while(iter.more()){ // While there are children, print list
                DKDDO part = (DKDDO)iter.next(); // Move pointer to next part & return that object.
                System.out.println("                  Part:     Type:  "+part.getPidObject().getObjectType());
                System.out.println("                         Item ID:  "+((DKPidICM)part.getPidObject()).getItemId());
                System.out.println("");
            }
        }

        // Print Folder Contents, if a DKFolder Collection Exists (See Folder Samples for more information)
        dataid = ddo.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER);
        if(dataid > 0){ // if a DKFolder collection exists, print contents
            DKFolder dkFolder = (DKFolder) ddo.getData(dataid); // obtain the DKFolder collection
            System.out.println("Folder Contents:");
            System.out.println("                Number: "+dkFolder.cardinality());
            dkIterator iter = dkFolder.createIterator(); // Create an iterator to go through Collection
            while(iter.more()){ // While there are children, print list
                DKDDO contents = (DKDDO)iter.next(); // Move pointer to next element & return that object.
                System.out.println("                  Item:     Type:  "+contents.getPidObject().getObjectType());
                System.out.println("                         Item ID:  "+((DKPidICM)contents.getPidObject()).getItemId());
                System.out.println("");
            }
        }
        
        // Print Links
        System.out.println("Links:");
        for(int i=0; i<linkCollections.size(); i++){  // go through all collection we gathered when listing the data items of the DDO.
            DKLinkCollection linkCollection = (DKLinkCollection) linkCollections.elementAt(i); // get each Link Collection
            dkIterator iter = linkCollection.createIterator(); // Create an iterator to go through Link Collection
            while(iter.more()){ // While there are links, print list
                DKLink dkLink = (DKLink)iter.next(); // Move pointer to next element & return that object.
                System.out.println("                  Link:     Type:  "+dkLink.getTypeName());
                System.out.println("                       Source ID:  "+((DKPidICM)((DKDDO)dkLink.getSource()).getPidObject()).getItemId()+"("+((DKDDO)dkLink.getSource()).getPidObject().getObjectType()+")");
                System.out.println("                       Target ID:  "+((DKPidICM)((DKDDO)dkLink.getTarget()).getPidObject()).getItemId()+"("+((DKDDO)dkLink.getTarget()).getPidObject().getObjectType()+")");
                if(dkLink.getLinkItem()!=null)
                    System.out.println("                     LinkItem ID:  "+((DKPidICM)((DKDDO)dkLink.getLinkItem()).getPidObject()).getObjectType());
                else
                    System.out.println("                     LinkItem ID:  null");
                System.out.println("");
            }
        }
        System.out.println("--------------------------------------------------------");
        System.out.println("");
        
        // Print Full Children
        for(int i=0; i<childCollections.size(); i++){  // go through all collection we gathered when listing the data items of the DDO.
            DKChildCollection childCollection = (DKChildCollection) childCollections.elementAt(i); // get each child collection
            dkIterator iter = childCollection.createIterator(); // Create an iterator to go through Child Collection
            while(iter.more()){ // While there are children, print each
                DKDDO child = (DKDDO) iter.next(); // Move pointer to next child & return that object.
                printDDO(child);
            }
        }
    }// end printDDO

   /**
    * Retrieve the parent DKDDO of the specified child component
    * if one exists.  If the parent reference is set in the 
    * child component, it will be returned from memory without
    * performing a search.  If the DKDDO is discovered to be
    * a root component, NULL will be returned before searching.
    * The following query will be executed if this is a child
    * component.  
    * 
    * //A[@ITEMID="B" AND (./C[@COMPONENTID="D" AND @VERSIONID=E])]
    * A:  Parent Component View Type Name or *.
    * B:  Item ID (All components of the same item share the same Item ID).
    * C:  Child Component View Type Name.
    * D:  Child Component ID.
    * E:  Version ID (All components of the same item version share the same version ID). 
    *
    * Faster solution exist.  This function covers one alternative
    * to retrieve the parent when DKDDO.getParent() returns 'null'.
    * Please refer to the Information Integrator for Content eSupport 
    * for more information on obtaining the parent of a child component.
    *
    * @param ddo - DDO to get the parent of.
    * @return Returns the parent with completed PID information or
    *         NULL if no parent exists.
    **/
    public static DKDDO retrieveParent(DKDDO ddo) throws DKException, Exception{

        System.out.println("Retrieving Parent DDO...");

        // First Check if Parent Reference is Set
        DKDDO parentDDO = (DKDDO) ddo.getParent();
        if(parentDDO!=null){ // If it was set, return it.
            System.out.println("Parent reference was set in memory.  Returned local reference.");
            return(parentDDO);   
        }

        // Gether Information
        DKPidICM pid = (DKPidICM) ddo.getPidObject();
        String itemId        = pid.getItemId();        // B
        String childViewName = pid.getObjectType();    // C
        String compId        = pid.getComponentId();   // D
        String verId         = pid.getVersionNumber(); // E

        // Next, check if it is a root component.
        DKDatastoreICM    dsICM    = (DKDatastoreICM)    ddo.getDatastore();
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef();
        DKComponentTypeViewDefICM childViewDef = (DKComponentTypeViewDefICM) dsDefICM.retrieveComponentTypeView(childViewName);
        if(childViewDef.isRoot()){ // If it is a root component, return 'null'.
            System.out.println("Type '"+childViewName+"' determined to be a root component.  Returning 'null' since no parent can exist.");
            return(null);
        }
        
        // Look up parent component view type name.
        DKComponentTypeViewDefICM parentViewDef = (DKComponentTypeViewDefICM) childViewDef.retrieveParent();
        String parentViewName = parentViewDef.getName(); // A

        // Build Query String
        StringBuffer query = new StringBuffer();
        query.append("//");
        query.append(parentViewName);
        query.append("[@ITEMID=\"");
        query.append(itemId);
        query.append("\" AND (./");
        query.append(childViewName);
        query.append("[@COMPONENTID=\"");
        query.append(compId);
        query.append("\" AND @VERSIONID=");
        query.append(verId);
        query.append("])]");
            
        // Build Options
        DKNVPair[] options = new DKNVPair[3];
        options[0]         = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "1");                                            // Specify max using a string value.
        options[1]         = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_IDONLY));
        options[2]         = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                           // Must mark the end of the NVPair

        // Process Query
        System.out.println("Evaluating Query:  "+query.toString());
        DKResults results = (DKResults)dsICM.evaluate(query.toString(), DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
            
        // Return Parent
        dkIterator iter = results.createIterator();
        parentDDO       = (DKDDO) iter.next();
        
        if(parentDDO!=null) System.out.println("Parent DDO found:  Type = '"+parentDDO.getObjectType()+"', CompID = '"+((DKPidICM)parentDDO.getPidObject()).getComponentId()+"'");
        else                System.out.println("No parent found.");
        
        return(parentDDO);
    }//end retrieveParent
    
}//end class SItemRetrievalICM
              
