/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SResourceMgrDefDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Resource Manager Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes Resource Manager Definitions.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SResourceMgrDefDeletionICM <Existing RM User-Defined Reference Name>
 *                                                    <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined.  It should be
 *                          all upper-case and should be the same name specified in the
 *                          SResourceMgrDefCreationICM Sample.
 *
 *                    NOTE: SMS Collection Definitions for this RM Definition must first be
 *                          deleted.  Please refer to the SSMSCollectionDefDeletionICM Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SResourceMgrDefDeletionICM MY_RM
 *                    ex: java SResourceMgrDefDeletionICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined.  This should be 
 *                    the same name specified in the SResourceMgrDefCreationICM Sample.
 *                    All SMS Collection must first be deleted.  Please refer to the SMS
 *                    Collection Definition Deletion Sample, SSMSCollectionDefDeletionICM.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SResourceMgrDefRetrievalICM
 ************************************************************************************************/
public class SResourceMgrDefDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceMgrDefDeletionICM <Existing RM User-Defined Reference Name>");
            System.out.println("                                  <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined.  This should");
            System.out.println("        be all upper-case and should be the same name specified in the");
            System.out.println("        SResourceMgrDefCreationICM Sample.");
            System.out.println(""); 
            System.out.println("  NOTE: SMS Collection Definitions for this RM Definition must first be");
            System.out.println("        deleted.  Please refer to the SSMSCollectionDefDeletionICM Sample.");
            System.out.println(""); 
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SResourceMgrDefSetDefaultICM RMDB");
            System.out.println("  ex: java SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  All RM Options Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SResourceMgrDefDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("===========================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Obtaining the Configuration Management Objects
            //-------------------------------------------------------------
            // The sample code blow demonstrates how to obtain objects for
            // Datastore Definition, Datastore Administration, Configuration
            // Management, and RM Configuration Management.
            System.out.println("Obtaining Configuration Management Objects...");

                DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
                DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
                DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
                DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            System.out.println("Obtained Configuration Management Objects.");

            //-------------------------------------------------------------
            // Delete the Resource Manager Definition                        
            //-------------------------------------------------------------
            System.out.println("Deleting Resource Manager Definition...");

                // Only Delete if It Exists
                if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM,rmUserDefinedReferenceName))
                    rmCfg.delResourceMgr(rmUserDefinedReferenceName);

            System.out.println("Deleted Resource Manager Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // deleteResourceMgrDef
    //******************************************************************
   /**
    * Delete the specified Resource Manager Definition.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that Items of this Item Type should be stored in.  It should consist of all upper-case letters.
    **/
    public static void deleteResourceMgrDef(DKDatastoreICM dsICM, String rmUserDefinedReferenceName) throws DKException, Exception{
        System.out.println("Deleting Resource Manager Definition Named '"+rmUserDefinedReferenceName+"'...");
        
            // Make sure the RM Definition is setup.
            if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
                throw new Exception("Resource Manager Definition not defined.");

            // Obtaining the Configuration Management Objects
            DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
            DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
            DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
            DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            rmCfg.delResourceMgr(rmUserDefinedReferenceName); // Delete the ResourceMgrDef

        System.out.println("Deleted Resource Manager Definition Named '"+rmUserDefinedReferenceName+"'.");
    }
   
}//end class SResourceMgrDefDeletionICM
              
