
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Description    : Add a folder and a document to an ImagePlus/390 server
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*; // using server package: 2-tier
import java.io.*;
import java.lang.*;

public class TAddSampIP implements DKConstantIP
{

   // Main method
   public static void main(String argv[])
   {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),    // library server
             userid = new String(""),    //userid
              pw = new String("");        // pw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
         System.out.println("Invoke Syntax: " );
         System.out.println( "  java TAddSampIP "
                           + "<libSrv> <userid> <pw>" );
         return;
      }
      else
      {
         libSrv = argv[0];
         userid = argv[1];
         pw = argv[2];
      }

      DKDatastoreIP dsIP = null;
      try
      {
         System.out.println(" *** ImagePlus Connector ");

         dsIP = new DKDatastoreIP();
         System.out.println("---Connect to ImagePlus server---");

         String s = new String( "APPL=01;FAFIP=9.67.43.83;"
                              + "IODMIP=9.67.43.83;FAFPORT=3061;"
                              + "IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;"
                              + "FAFSITE=CS61" );

         dsIP.connect( libSrv, userid, pw, s );
         System.out.println("datastore ImagePlus connected");
         dsIP.setOption( DK_CM_OPT_ACCESS_MODE, new Integer(DK_CM_READWRITE) );

         BufferedReader iStream;
         String lines = null;

         // Add folder
         DKDDO newFolder = dsIP.createDDO( DK_IP_FOLD, 0 );
         short newDataId = 0;
         DKPidIP foldPid = null;

         iStream = new BufferedReader(new FileReader("addSamp.txt"));

         while ((lines = iStream.readLine()) != null)
         {
            if (lines.startsWith("FoldFolderId"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_ID );
               newFolder.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("FoldCodePage"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_CODEPG );
               newFolder.setData( newDataId
                                , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("FoldCreateDate"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_DATE );
               newFolder.setData( newDataId
                                , DKDate.valueOf(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("FoldEmptyKeep"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_EMPTY );
               newFolder.setData( newDataId
                                , new Short(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("FoldDescription"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_DESC );
               newFolder.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("FoldFolderType"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_TYPE );
               newFolder.setData( newDataId
                                , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("FoldUserData"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_USER );
               newFolder.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("FoldSecurity"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_SECODE );
               newFolder.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("FoldIndexId"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_IDXID );
               newFolder.setData( newDataId
                                , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("FoldIndexValue"))
            {
               newDataId = newFolder.dataId( DK_IP_ATTR_FOLD_IDXVAL );
               newFolder.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
         }

         System.out.println("---Add folder---");
         dsIP.addObject( newFolder );

         // Add document
         DKDDO newDoc = dsIP.createDDO( DK_IP_DOC, 0 );
         DKPidIP docPid = null;
         short numPages = 1;
         String fileName = null;
         int contentClass = 0;

         iStream = new BufferedReader(new FileReader("addSamp.txt"));

         while ((lines = iStream.readLine()) != null)
         {
            if (lines.startsWith("DocFolderId"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_FLDID );
               newDoc.setData( newDataId
                             , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocReceiveDate"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_RCVDATE );
               newDoc.setData( newDataId
                             , DKDate.valueOf(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocFileTab"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_FILETAB );
               newDoc.setData( newDataId
                             , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocStatus"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_STATUS );
               newDoc.setData( newDataId
                             , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocCollection"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_COLNAME );
               newDoc.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocCodePage"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_CODEPG );
               newDoc.setData( newDataId
                                , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocOrigKept" ))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_ORIGKP );
               newDoc.setData( newDataId
                                , new Short(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocUserDate"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_USRDATE );
               newDoc.setData( newDataId
                             , DKDate.valueOf(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocFileDate"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_FILDATE );
               newDoc.setData( newDataId
                             , DKDate.valueOf(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocDesc"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_DESC );
               newDoc.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocFormCode"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_FORMCD );
               newDoc.setData( newDataId
                                , new Integer(lines.substring(lines.indexOf('=')+1)) );
               continue;
            }
            else if (lines.startsWith("DocFormNum"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_FORMNUM );
               newDoc.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocUserData"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_USRDATA );
               newDoc.setData( newDataId
                             , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocStoreSite"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_STRSITE  );
               newDoc.setData( newDataId
                                , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocSecurity"))
            {
               newDataId = newDoc.dataId( DK_IP_ATTR_DOC_SECLASS );
               newDoc.setData( newDataId
                             , lines.substring(lines.indexOf('=')+1) );
               continue;
            }
            else if (lines.startsWith("DocNumPages"))
            {
               // Used when adding the document content Blob.
               Integer tmpInt = new Integer( lines.substring(lines.indexOf('=')+1) );
               numPages = tmpInt.shortValue();
               continue;
            }
            else if (lines.startsWith("ContentClass"))
            {
               // Used when adding the document content Blob.
               Integer tmpInt = new Integer(lines.substring(lines.indexOf('=')+1));
               contentClass = tmpInt.intValue();
               newDoc.setData( newDataId
                             , tmpInt );
               continue;
            }
            else if (lines.startsWith( "ContentFile" ))
            {
               fileName = lines.substring(lines.indexOf('=')+1 );
               continue;
            }
         }

         System.out.println("---Add document---");
         dsIP.addObject( newDoc );

         docPid = (DKPidIP)newDoc.getPidObject();
         String tempId = docPid.getIdString( DK_IP_PID_DOC_ID);

         if ( tempId == null || tempId.equals("") )
         {
            System.out.println("  Doc Id is not set.");
         }
         else
         {
            if (  fileName != null
               && !(fileName.equals("")) )
            {
               DKBlobIP newBlobIP = new DKBlobIP(dsIP);
               DKPidXDOIP tmpPid = new DKPidXDOIP();
               tmpPid.setPartId(1);  // only 1 part.
               tmpPid.setDocId( tempId );
               newBlobIP.setPidObject(tmpPid);

               // Set content class
               newBlobIP.setContentClass( contentClass );
               // Set number of pages
               newBlobIP.setPageCount(numPages);

               newBlobIP.setContentFromClientFile(fileName);

               System.out.println("---Add document content---");
               newBlobIP.add();
            }
         }

         dsIP.disconnect();
         System.out.println("datastore ImagePlus disconnected");
         dsIP.destroy();
      }
      catch (DKException exc)
      {
     	  try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc)
      {
     	  try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
   }
}

