/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TDatastorePoolJDBC.java
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.cs.*;
import java.io.*;
import java.lang.reflect.*;

// TDatastorePoolJDBC class
public class TDatastorePoolJDBC implements DKConstantJDBC {

  // Main method
  public static void main(String argv[]){
    
      //*** for input parameters
      String url = new String(""),      // url = "jdbc:db2:sample";
             userid = new String(""),   // userid 
  	     pw = new String(""),       // pw
             schema = new String("");   // schema   

      String fullClassName = "com.ibm.mm.sdk.server.DKDatastoreJDBC";
      String configuration = "JDBCDRIVER=(COM.ibm.db2.jdbc.app.DB2Driver)";
      //  String configuration = "JDBCDRIVER=(sun.jdbc.odbc.JdbcOdbcDriver)";
      //  url = "jdbc:db2:sample";
      //  url = "jdbc:odbc:sample";
      //  url = "jdbc:odbc:mysample";

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 

      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TDatastorePoolJDBC " +
        "<url> <userid> <pw> [<schema-name> - optional]" );
       return;
      }
      else if (argv.length == 4)
      {
       schema = argv[3];
      }
      url = argv[0];
      userid = argv[1];
      pw = argv[2];


    try {
    DKDatastorePool dsrJDBC = new DKDatastorePool(fullClassName);
    dsrJDBC.setDatastoreName(url);
    dsrJDBC.setMaxPoolSize(10);
    dsrJDBC.setMinPoolSize(3);
    dsrJDBC.setConfigurationString(configuration);

    dsrJDBC.initConnections(userid, pw, 2);
    dsrJDBC.setTimeOut(1); 
    System.out.println(" getMaxPoolSize =  " + dsrJDBC.getMaxPoolSize());
    System.out.println(" getMinPoolSize =  " + dsrJDBC.getMinPoolSize());
    System.out.println(" getTimeOut=  " + dsrJDBC.getTimeOut());

    dkDatastore dsJDBC1, dsJDBC2, dsJDBC3, dsJDBC4;
    dsJDBC1 = dsrJDBC.getConnection(userid, pw);             // new ds1
    System.out.println(" getConnection, dsJDBC1(ds1) =  " + dsJDBC1);
    dsJDBC2 = dsrJDBC.getConnection(userid, pw);             // new ds2
    System.out.println(" getConnection, dsJDBC2(ds2) =  " + dsJDBC2);

    DKDatastoreSession dsIdJDBC1, dsIdJDBC2, dsIdJDBC3, dsIdJDBC4;
    dsIdJDBC1 = dsrJDBC.getConnection(userid, pw, 0);        //new dsid1
    System.out.println(" getConnection, dsIdJDBC1(dsid1)=" +
     dsIdJDBC1.getDatastore() + ", sessionId=" + dsIdJDBC1.getId());
    dsIdJDBC2 = dsrJDBC.getConnection(userid, pw, 0);        //new dsid2
    System.out.println(" getConnection, dsIdJDBC2(dsid2)=" +
     dsIdJDBC2.getDatastore() + ", sessionId=" + dsIdJDBC2.getId()); 

    dsrJDBC.returnConnection(dsIdJDBC2);                     //dsid2 in _freeIdPool
    System.out.println(" returnConnection, dsIdJDBC2(dsid2) =  " + 
      dsIdJDBC2.getDatastore() + ", sessionId=" + dsIdJDBC2.getId());
    dsrJDBC.returnConnection(dsJDBC2);                       //ds2 in _freePool
    System.out.println(" returnConnection, dsJDBC2(ds2) =  " + dsJDBC2); 

    dsIdJDBC3 = dsrJDBC.getConnection(userid, pw, 1); //ds2 in _usedPool
    System.out.println(" getConnection, dsIdJDBC3(ds2)=" +
     dsIdJDBC3.getDatastore() + ", sessionId=" + dsIdJDBC3.getId());
    dsIdJDBC4 = dsrJDBC.getConnection(userid, pw, 5); //new dsid3  
    System.out.println(" getConnection, dsIdJDBC4(dsid3)=" +
     dsIdJDBC4.getDatastore() + ", sessionId=" + dsIdJDBC4.getId());


    dsIdJDBC4 = dsrJDBC.getConnection(userid, pw, 6); // new dsid4
    System.out.println(" getConnection, dsIdJDBC4(dsid4)=" +
     dsIdJDBC4.getDatastore() + ", sessionId=" + dsIdJDBC4.getId());

    dsJDBC1 = dsrJDBC.getConnection(userid, pw);  // new ds3
    System.out.println(" getConnection, dsJDBC1(ds3) =  " + dsJDBC1);
    dsJDBC2 = dsrJDBC.getConnection(userid, pw);  // new ds4
    System.out.println(" getConnection, dsJDBC2(ds4) =  " + dsJDBC2); 

    dsJDBC3 = dsrJDBC.getConnection(userid, pw);  // new ds5
    System.out.println(" getConnection, dsJDBC3(ds5) =  " + dsJDBC3);
    dsJDBC4 = dsrJDBC.getConnection(userid, pw);  // new ds6
    System.out.println(" getConnection, dsJDBC4(ds6) =  " + dsJDBC4); 


    dsIdJDBC3 = dsrJDBC.getConnection(userid, pw, 9); // dsid2 
    System.out.println(" getConnection, dsIdJDBC3(dsid2)=" +
     dsIdJDBC3.getDatastore() + ", sessionId=" + dsIdJDBC3.getId());


    //**** use dsJDBC2 to do query
/**
    System.out.println("||**** start: use dsJDBC2 to do query ****||");       
       dkQuery pQry = null;
       String cmd = "";
       DKResults pResults = null;
       DKNVPair parms1[] = null;
       DKNVPair parms[] = new DKNVPair[2];
       String strMax = "5";
       parms[0] = new DKNVPair(DK_CM_PARM_MAX_RESULTS,strMax);
       parms[1] = new DKNVPair(DK_CM_PARM_END,null);
       System.out.println("connecting to datastore");
       if (schema.equals("")) 
        cmd = "SELECT * FROM EMPLOYEE";
       else
        cmd = "SELECT * FROM " + schema + ".EMPLOYEE"; 
       System.out.println("query string " + cmd);
       System.out.println("create query");
       pQry = dsJDBC2.createQuery(cmd,DK_CM_SQL_QL_TYPE,parms1);
       System.out.println("executing query");
       pQry.execute(parms);
       System.out.println("query executed");
       System.out.println("number of results " + pQry.numberOfResults());
       System.out.println("process query results");
       pResults = (DKResults)pQry.result();
       processResults((dkCollection)pResults);
       System.out.println("query results processed");
       dsJDBC2.commit();
    System.out.println("||**** end: use dsJDBC2 to do query ****||");
**/
    //****end of use dsJDBC2 to do query 

    dsrJDBC.returnConnection(dsJDBC2);
    System.out.println(" returnConnection, dsJDBC2(ds4) =  " + dsJDBC2);
    dsJDBC4 = dsrJDBC.getConnection(userid, pw);
    System.out.print(" getConnection, dsJDBC4 =  " + dsJDBC4); 
    System.out.println("    dsJDBC4 == dsJDBC2 =  " + (dsJDBC4 == dsJDBC2)); //true

    //**** use dsJDBC4 to retrieve XDO - need to change the itemId and partId
/**/  
    System.out.println("||**** start: use dsJDBC4 to retrieve XDO ****||");
      
      String table;                                   //from SAMPLE database of JDBC
      String column = "RESUME";                       //clob column 
      String predicate = "EMPNO = '000130' and RESUME_FORMAT = 'ascii'" ; //primary key of exist clob
      if (schema.equals("")) 
        table = "EMP_RESUME";
      else
        table = schema + ".EMP_RESUME"; 
      DKClobJDBC axdo = new DKClobJDBC(dsJDBC4);
      DKPidXDOJDBC  apid = new DKPidXDOJDBC();
      apid.setTable(table);
      apid.setColumn(column);
      apid.setDataPredicate(predicate);   //exist clob
      axdo.setPidObject(apid);
      System.out.println("table= " + axdo.getTable());
      System.out.println("column=" + ((DKPidXDOJDBC)(axdo.getPidObject())).getColumn());
      System.out.println("dataPredicate=" + ((DKPidXDOJDBC)(axdo.getPidObject())).getDataPredicate());
      //### retrieve the exist clob - 000130 ascii
      System.out.println("about to do retrieve()...");
      axdo.retrieve();
      System.out.println("retrieve successfully...");    
      System.out.println("about to do open()");
      axdo.setInstanceOpenHandler("notepad ", true);   //use notepad in NT
      axdo.open();     //iexplore requires the file extension
      dsJDBC4.commit();
    System.out.println("||**** end: use dsJDBC4 to retrieve XDO ****||");
/**/
    //****end of use dsJDBC4 to retrieve XDO
       
    dsrJDBC.returnConnection(dsJDBC1);
    System.out.println(" returnConnection, dsJDBC1(ds3) =  " + dsJDBC1);
    dsrJDBC.returnConnection(dsJDBC3);
    System.out.println(" returnConnection, dsJDBC3(ds5) =  " + dsJDBC3);
    dsrJDBC.returnConnection(dsIdJDBC4); //dsid4 in _freeIdPool
    System.out.println(" returnConnection, dsIdJDBC4(dsid4) =  " + 
      dsIdJDBC4.getDatastore() + ", sessionId=" + dsIdJDBC4.getId());
     

    System.out.println("**** start sleep for 120050 mini sec*******");
    Thread.sleep(120050);  //the two in freePool will be disconnected
    System.out.println("**** end sleep for 120050 mini sec*******");

    dsJDBC2 = dsrJDBC.getConnection(userid, pw);     //new ds7
    System.out.println(" getConnection, dsJDBC2(ds7) =  " + dsJDBC2); 
    System.out.println(" call destroy()..." );
    dsrJDBC.destroy();  //take a while for all 4 in usedPool to be disconnected

    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }

  public static void processResults(dkCollection results) 
                throws DKException, Exception
  {
    Short    sVal;
    Integer  iVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println( "number of items " + results.cardinality());
 
    dkIterator iter = results.createIterator();
 
    while(iter.more()) {
       item = (DKDDO) iter.next();
       if (item != null) {
	   //   	  item.retrieve(); //Only valid if Primary Key exists for Table
   	  sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
   	  itemType = sVal.shortValue();
   	  switch(itemType) {
   	    case DK_CM_DOCUMENT:
	      {
   	      System.out.println(" Item is a document, number of data item : " + item.dataCount());
   	      for (i = 1; i <= item.dataCount(); i++) {
   		   dataName = item.getDataName(i);
   		   System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   		   }
   	      }
   	      break;
   	    default:
   	      System.out.println("  Item is not recognized ");
   	      break;
   	  } 
       }
    } 
  }
}

