/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// client side sample connect/execute/retrieve/disconnect to OnDemand

/*
 * Filename       : TClientRetrieveWithCallbackOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  if folders found display them and their documents/
 *                  folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TClientRetrieveWithCallbackOD class
public class TClientRetrieveWithCallbackOD implements DKConstantOD
{
  static String appgrp = new String("");
  static String cmd = new String("");

  // Main method
  public static void main(String argv[])
  {
    int threshold = 2, iterations = 1;
    boolean retrieveHits = true;

    DKDatastoreOD dsOD = null;
    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw = new String(""),         // pw
             connect_string = new String(""),
             configuration_string = new String(""),
             threshold_string = new String(""),
             iterations_string = new String("");

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TClientRetrieveWithCallbackOD " + "<libSrv> <userid> <pw> <connect string> <configuration string> threshold iterations" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
        if (argv.length > 4)
          configuration_string = argv[4];
        if (argv.length > 5 )
        {
          threshold_string = argv[5];
          threshold = Integer.parseInt(threshold_string);
        }
        if (argv.length > 6 )
        {
          iterations_string  = argv[6];
          iterations = Integer.parseInt(iterations_string);
        }
      }
      System.out.println("connection string=>" + connect_string + "<=");
      System.out.println("configuration string=>" + configuration_string + "<=");


      if (configuration_string.length() > 0)
      {
        dsOD = new DKDatastoreOD(configuration_string);
      }
      else
      {
        dsOD = new DKDatastoreOD();
      }

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Select a testcase by changing this number!!!
      int testcase = 10;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setAGQueryCriteria(testcase, libSrv);

      DKNVPair[] parms = new DKNVPair[3];
      parms[0] = new DKNVPair("APPL_GROUP", appgrp);
      parms[1] = new DKNVPair("MAX_RESULTS",
                 new String(Integer.toString(threshold*iterations)));
      parms[2] = new DKNVPair("CONTENT", new String("ATTRONLY"));

      System.out.println("\nApplication group =>" + appgrp + "<=");
      System.out.println("Query string =>" + cmd + "<=\n");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " + libSrv + ", userid: " +
                         userid );

      System.out.println("executing query");

      long  t = 0;
      Date date  = new Date();
      System.out.println("Before query, time is " + date.toString());
      t = (new Date()).getTime();

      // Execute with callback!!!
      TClientCallbackOD callback = new TClientCallbackOD(dsOD,
                                                         threshold,
                                                         iterations,
                                                         retrieveHits); // whether to save hit for retrieval
      System.out.println("***Threshold is set to "+threshold+", iterations is set to "+iterations );
      dsOD.executeWithCallback(cmd,DK_CM_SQL_QL_TYPE,parms,callback);
      // wait until the callback object is done...
      while (!callback.isDone())
        Thread.sleep (1000);

      // check for exception
      DKException cbExc = callback.getException();
      if (cbExc != null)
      {
        System.out.println("Exception name " + cbExc.name());
        System.out.println("Exception message " + cbExc.getMessage());
        System.out.println("Exception error code " + cbExc.errorCode());
        System.out.println("Exception error state " + cbExc.errorState());
        cbExc.printStackTrace();
        callback.resetException(); // reset since it is not needed any more
      }

      t = (new Date()).getTime() - t;
      System.out.println("Execute elapsed time: " + t + " msec");
      date  = new Date();
      System.out.println("After query, time is " + date.toString());
      System.out.println("datastore executed query");

      // retrieve and display the hits (DDOs)
      if (retrieveHits)
      {
        System.out.println("");
        System.out.println("================================");
        System.out.println("==the retrieval process begins==");
        System.out.println("================================");
        callback.retrieveDDO();
      }

      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
    System.out.println("TClientRetrieveWithCallbackOD done ...");
  }

  // --------------------------------------------------------------------
  // Convert regular month, day, year values to an OnDemand integer date
  // Asumming (and hopefully) the year is in the yyyy format
  // --------------------------------------------------------------------
  public static String ODdate( int month, int day, int year )
  {
    int[] days = {31,28,31,30,31,30,31,31,30,31,30,31};
    int OD_days = 0;
    String od_days_str;
    Integer OD_DAY = new Integer(0);

    // Just in case someone entered a 2 digit year
    if ( year < 100 )
    {
       // 21st century
       if (year < 70)
          year += 2000;
       else
          year += 1900;
    }

    // Get the numbers of days since 1/1/1970 to 1/1/yyyy
    OD_days = ( year - 1970 ) * 365;

    // Add days from leap years
    for ( int yr = 1970; yr < year; yr++ )
    {
      if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
      {
        OD_days++;
      }
    }

    // Add days sicne 1/1 of this year
    for ( int mon = 0; mon < month-1; mon++ )
    {
      OD_days += days[mon];
    }

    // Add the current day
    OD_days += day;

    // Additional Leap year adjustment
    if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
      OD_days++;

    od_days_str = OD_DAY.toString(OD_days);
    return od_days_str;
  }

  // --------------------------------------------------------------------
  // set application group and query SQL
  // --------------------------------------------------------------------
  public static void setAGQueryCriteria(int testcase, String libSrv)
  {
    int month = 6;
    int day   = 2;
    int year  = 1997;
    Integer temp_int = new Integer(0);
    String  mon_str, day_str, yr_str, date_str;
    String od_date = "";

    // Application Group and SQL string
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          appgrp = "INSURE  01 ";
          cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
          break;
        case 2:
          appgrp = "CHKSTMN301 ";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line/large object
          appgrp = "DELINQCY02 ";
          cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
          break;
        case 4:  // pdf
          appgrp = "KEYBANK 01 ";
          cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
          break;
      }
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          appgrp = "Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          appgrp = "CHKSTMNT";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line data
          appgrp = "Delinqcy-AG";
          cmd = "WHERE ( pagenum != 0 )";
          break;
        case 4:  // line data large object
          appgrp = "LOAN LO";
          od_date = ODdate( 10, 1, 1994 );
          cmd = "WHERE ( load_date = " + od_date + " )";
          //od_date = ODdate( 2, 2, 1970 );
          //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 5:  // pdf
          appgrp = "PDF landscape";
          cmd = "WHERE ( region LIKE '%' )";
        case 6:  // AFP
          appgrp = "afpdata";
          cmd = "WHERE (TaxYear = 1995)";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // Large object AFP - gunnar
          appgrp = "OnDemand Publications";
          cmd = "where bookname LIKE '%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest"))
    {
      switch (testcase)
      {
        default:
        case 1:
          // AFP data
          appgrp = "CRD";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          // Large Object line data
          appgrp = "sp-ebcdic-lo";
          //od_date = ODdate( 1, 1, 1990 );
          //od_date2 = ODdate( 12, 31, 2001 );
          //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
          od_date = ODdate( 12, 15, 2000 );
          cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
          break;
        case 3: // Loan Delinquency Reports - Large Objects
          appgrp = "LOAN LO";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 4:
          // Large Object line data
          appgrp = "eliz-largeline";
          od_date = ODdate( 3, 19, 1999 );
          cmd = "WHERE ( rptdate = " + od_date + " )";
          break;
      case 5:
        // Large Object line data
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CHECKSTMTS ";
          cmd = "WHERE ( KY1VAL LIKE '%' )";
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CheckStatements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          appgrp = "Few Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2: // Line - Loan Delinquency Reports
          appgrp = "Loan Delinquency Reports";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
      }
    }
    else // can be any server!
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          mon_str = temp_int.toString( month );
          day_str = temp_int.toString( day );
          yr_str  = temp_int.toString( year );
          date_str = mon_str + '/' + day_str + '/' + yr_str;
          // native OD integer date
          od_date = ODdate( month, day, year );
          System.out.println("The date '"+ date_str +"' equals to the OnDemand integer date '"+ od_date+"'" );

          appgrp = "CRD";
          //cmd = "WHERE ( crd_date = " + od_date + " )";
          cmd = "WHERE ( name LIKE '%')";
          break;
        case 2:  // line
          appgrp = "Balance";
          cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
          break;
      }
    }
  }
}

