// IBM Confidential OCO Source Material
// 5648-C83, 5648-C84 (C) COPYRIGHT International Business Machines Corp. 1997,1998,1999
// The souce code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright office.

import java.io.*;
import org.omg.Security.*;
import org.omg.SecurityLevel2.Credentials;
import com.ibm.IExtendedSecurity.*;
import com.ibm.CORBA.iiop.ORB;
import com.ibm.ISecurityLocalObjectBaseL13Impl.CurrentImpl;

/**

LoginHelper is a utility class that uses CORBA security APIs to perform
a programmatic login. It uses the CORBA "SecurityCurrent" to obtain a
CORBA login_helper and invokes request_login on that object. This method
sets up the userid and password on the thread so that any further
method invocation on a secure server will be invoked under that userid/password.
The server will extract the userid and password and will perform authentication
before invoking the method.

 */
public class LoginHelper {

    protected static final int PUBLIC   = 0;

    protected static org.omg.Security.AttributeType secAttrType[];

    // use the CORBA attribute type to get public attribute (user name),
    // access id (user registry specific unique ID)

    static {
        secAttrType = new org.omg.Security.AttributeType[1];

        org.omg.Security.ExtensibleFamily familyOMG =
        new org.omg.Security.ExtensibleFamily((short)0, (short)1);

        secAttrType[PUBLIC] = new org.omg.Security.AttributeType(familyOMG,
        org.omg.Security.Public.value);
    }

    // IBM's extension of SecurityCurrent
    private com.ibm.IExtendedSecurity.Current current = null;
    //private com.ibm.ISecurityLocalObjectBaseL13Impl.CurrentImpl current = null;

    public LoginHelper() throws IllegalStateException {
        // obtain the security current from the ORB instance
        current = getSecurityCurrent();
        System.out.println("***LoginHelper c'tr: getting SecurityCurrent from ORB");
    }


    /**
    The method login() sets the authentication data on the thread of execution
    by calling the login_helper.request_login

    @param userid userid
    @param password user's password
    @return Credentials set on the security context based on the userid and password
    @throws LoginFailed exception
     */

    public org.omg.SecurityLevel2.Credentials login(String userid, String password)
    throws org.omg.SecurityLevel2.LoginFailed

    {

        if (!(userid != null && (userid.length()!=0)) ) {
            // Invalid userID (empty string)
            throw new org.omg.SecurityLevel2.LoginFailed();
        }

        if (!(password != null && (password.length()!=0))) {
            // Invalid password (empty string)
            throw new org.omg.SecurityLevel2.LoginFailed();
        }

        com.ibm.IExtendedSecurity._LoginHelper loginHelper = current.login_helper();
        return loginHelper.request_login(userid, "", password,
        new org.omg.SecurityLevel2.CredentialsHolder(),
        new org.omg.Security.OpaqueHolder());
    }


    /**

    This method sets the specified credentials as the invocation
    credentials. Any method invoked after this will be executed under
    the authority of the credentials.

    @param invokedCreds credentials under which future methods should be invoked
    @throws InvalidCredentialType, InvalidCredential

     */
    public void setInvocationCredentials(org.omg.SecurityLevel2.Credentials invokedCreds)
    throws org.omg.Security.InvalidCredentialType,
    org.omg.SecurityLevel2.InvalidCredential
    {

        current.set_credentials(org.omg.Security.CredentialType.SecInvocationCredentials,
        invokedCreds);

    }


    /**

    This method gets the credentials under which the current
    method is getting executed.

    @return credentials under which the current method is invoked

    @throws InvalidCredentialType

     */
    public org.omg.SecurityLevel2.Credentials getInvocationCredentials()
    throws org.omg.Security.InvalidCredentialType
    {
        return current.get_credentials(org.omg.Security.CredentialType.SecInvocationCredentials);
    }


    /**
    Returns a human-readable user name attribute of the specified credentials.

    @return String the user name
     */

    public String getUserName(org.omg.SecurityLevel2.Credentials creds)
    throws org.omg.Security.DuplicateAttributeType,
    org.omg.Security.InvalidAttributeType
    {

        org.omg.Security.Attribute[] publicAttrs =
        creds.get_attributes(secAttrType);
        String uname = "";
        try {
            uname = new String(publicAttrs[PUBLIC].value, "UTF8");
        } catch  (java.io.UnsupportedEncodingException e) {

        }
        return uname;

    }


    private com.ibm.IExtendedSecurity.Current getSecurityCurrent()
    throws IllegalStateException
    {
        com.ibm.IExtendedSecurity.Current current =null;

        try {
            // obtain a reference to the underlying ORB
            com.ibm.CORBA.iiop.ORB orb = com.ibm.ejs.oa.EJSORB.getORBInstance();
            
            // obtain the SecurityCurrent from the ORB
            if (orb != null) {
                current = (com.ibm.IExtendedSecurity.Current)
                orb.resolve_initial_references("SecurityCurrent");
            } else {
                throw new IllegalStateException("SecurityCurrent: null");
            }

        } catch (Exception e) {
            throw new IllegalStateException("Error getting SecurityCurrent from the ORB");
        }
        return current;
    }
/**

    private com.ibm.ISecurityLocalObjectBaseL13Impl.CurrentImpl getSecurityCurrent()
    throws IllegalStateException
    {
        com.ibm.ISecurityLocalObjectBaseL13Impl.CurrentImpl current =null;
        try {
            // obtain a reference to the underlying ORB
            com.ibm.CORBA.iiop.ORB orb =
            com.ibm.ejs.oa.EJSORB.getORBInstance();
            // obtain the SecurityCurrent from the ORB
            if (orb != null) {
                current=(com.ibm.ISecurityLocalObjectBaseL13Impl.CurrentImpl)
                orb.resolve_initial_references("SecurityCurrent");
            } else {
                throw new IllegalStateException("SecurityCurrent: null");
            }
        } catch (Exception e) {
            throw new IllegalStateException("Error getting SecurityCurrent from the ORB");
        }
        return current;
    }

**/
    /**
    The method login() sets the authentication data on the thread of execution
    by calling the login_helper.request_login

    @param userid userid
    @param password user's password
    @return Credentials set on the security context based on the userid and password
    @throws LoginFailed exception
     

    public org.omg.SecurityLevel2.Credentials login(byte[] ltpaCookie)
    throws org.omg.SecurityLevel2.LoginFailed

    {
        org.omg.SecurityLevel2.Credentials creds = null;

        try{
            com.ibm.ISecurityLocalObjectBaseL13Impl.LoginHelperImpl loginHelper = (com.ibm.ISecurityLocalObjectBaseL13Impl.LoginHelperImpl) current.login_helper();

            creds = (org.omg.SecurityLevel2.Credentials) loginHelper.request_login(com.ibm.ISecurityUtilityImpl.AuthenticationTarget.LTPA,ltpaCookie);

            if (true){
                throw new IllegalStateException("Hahahah, it is in classes");
            }


        }catch (org.omg.SecurityLevel2.LoginFailed exp){
            exp.printStackTrace();
            throw exp;
        }

        return creds;
    }

    **/
}
